#include <bits/stdc++.h>

template <typename T>
struct in {
    T x;

    in() {
        std::cin >> x;
    }

    in(const T &_x) : x(_x) {
    }

    operator T&() {
        return x;
    }

    operator const T&() const {
        return x;
    }
};

template <typename T>
struct decr : T {
    decr() : T() {
        --(*this);
    }
};

using namespace std;

struct solve {
    in<int> n, m, p, g;
    vector<in<int>> w;
    struct edge {
        decr<in<int>> a, b;
        in<int> s;
    };
    vector<edge> edges;
    vector<vector<pair<int, int>>> graph;

    solve() : w(n), edges(m), graph(n) {
        for (auto [a, b, s] : edges) {
            graph[a].emplace_back(b, s);
        }

        switch (g) {
            case 1:
                solve_dijk();
                break;
            case 2:
                solve_bambuk();
                break;
            case 0:
            case 3:
                solve_maskdp();
                break;
            default:
                throw logic_error("Not implemented");
        }
    }

    void solve_dijk() {
        const long long INF = 1'000'000'000'000'000'001LL;
        vector<long long> dist(n, INF);

        auto cmp = [&](int i, int j) { return dist[i] < dist[j] || (dist[i] == dist[j] && i < j); };
        set<int, decltype(cmp)> queue(cmp);

        dist[0] = 0;
        for (int i = 0; i < n; ++i) {
            queue.insert(i);
        }
        while (!queue.empty()) {
            int v = *queue.begin();
            queue.erase(queue.begin());
            for (auto [u, s] : graph[v]) {
                if (dist[u] > dist[v] + s) {
                    auto it = queue.find(u);
                    if (it != queue.end()) {
                        queue.erase(it);
                    }
                    dist[u] = dist[v] + s;
                    queue.insert(u);
                }
            }
        }
        if (dist[n - 1] > INF / 2) {
            cout << "-1\n";
        } else {
            cout << max(0LL, dist[n - 1] - p) << "\n";
        }
    }

    void solve_bambuk() {
        long long cur_money = p;
        long long cnt = 0;
        long long max_w = 0;
        for (int i = 0; i < n - 1; ++i) {
            max_w = max(max_w, (long long) w[i]);
            long long new_predst = max(0LL, edges[i].s - cur_money + max_w - 1) / max_w;
            cnt += new_predst;
            cur_money += max_w * new_predst;
            cur_money -= edges[i].s;
        }
        cout << cnt << "\n";
    }

    void solve_maskdp() {
        auto max_w = [&](unsigned mask) {
            int res = 0;
            for (int bit = 0; bit < n; ++bit) {
                if (mask & (1 << bit)) {
                    res = max(res, (int)w[bit]);
                }
            }
            return res;
        };

        vector<vector<pair<long long, long long>>> dp(1 << n, vector<pair<long long, long long>> (n, {-1, 0}));
        dp[1][0] = {0, -p};
        long long res = -1;
        for (unsigned mask = 0; mask < (1 << n); ++mask) {
            long long mw = max_w(mask);
            for (int iter = 0; iter <= n; ++iter) {
                for (int v = 0; v < n; ++v) {
                    if (!(mask & (1 << v))) {
                        continue;
                    }
                    if (dp[mask][v].first == -1) {
                        continue;
                    }
                    auto [cur_cnt, cur_money] = dp[mask][v];
                    for (auto [u, s]: graph[v]) {
                        unsigned nmask = mask | (1 << u);
                        long long next_cnt = max(0LL, s + cur_money + mw - 1) / mw;
                        pair<long long, long long> cand{cur_cnt + next_cnt, cur_money + s - next_cnt * mw};
                        if (dp[nmask][u].first == -1 || cand < dp[nmask][u]) {
                            dp[nmask][u] = cand;
                        }
                    }
                }
            }
            if (dp[mask][n - 1].first != -1) {
                if (res == -1 || dp[mask][n - 1].first < res) {
                    res = dp[mask][n - 1].first;
                }
            }
        }
        cout << res << "\n";
    }
};

int main() {
    ios::sync_with_stdio(false);
#ifndef LOCAL
    cout.tie(nullptr);
#endif
    in<int> t = 1;
    while (t--) {
        solve();
    }
}
