#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <map>
#include <math.h>
#include <stack>
#include <random>
#include <set>
#define int long long

using namespace std;

mt19937_64 rnd(17112004);

const int inf = (1 << 31) - 1;

struct Node {
    int val, pr, sz;
    Node* l;
    Node* r;

    Node(int x) {
        val = x;
        pr = rnd();
        sz = 1;
        l = r = nullptr;
    }
};

int getSz(Node* t) {
    if (t == nullptr) {
        return 0;
    }
    return t->sz;
}

void upd(Node* t) {
    if (t == nullptr) {
        return;
    }
    t->sz = getSz(t->l) + getSz(t->r) + 1;
}

pair<Node* , Node*> split(Node* t, int x) {
    if (t == nullptr) {
        return {nullptr, nullptr};
    }
    if (getSz(t->l) + 1 < x) {
        auto n = split(t->r, x - getSz(t->l) - 1);
        t->r = n.first;
        upd(t);
        return {t, n.second};
    } else if (getSz(t->l) + 1 == x) {
        Node* n = t->r;
        t->r = nullptr;
        upd(t);
        return {t, n};
    } else {
        auto n = split(t->l, x);
        t->l = n.second;
        upd(t);
        return {n.first, t};
    }
}

Node* merge(Node* l, Node* r) {
    if (l == nullptr) {
        return r;
    }
    if (r == nullptr) {
        return l;
    }
    if (l->pr > r->pr) {
        auto n = merge(l->r, r);
        l->r = n;
        upd(l);
        return l;
    } else {
        auto n = merge(l, r->l);
        r->l = n;
        upd(r);
        return r;
    }
}

void sw(Node* &t, int m) {
    auto n = split(t, m);
    t = merge(n.second, n.first);
}

void print(Node* t) {
    if (t == nullptr) {
        return;
    }
    print(t->l);
    cout << t->val << " ";
    print(t->r);
}

struct segtree {
    vector<vector<int>> tree;
    int size = 1;

    void build(int x, int lx, int rx, vector<int> &a) {
        if (rx - lx == 1) {
            if (lx < a.size()) {
                tree[x] = {a[lx]};
            }
            return;
        }
        int m = (rx + lx) / 2;
        build(x * 2 + 1, lx, m, a);
        build(x * 2 + 2, m, rx, a);
        int i = 0, j = 0;
        int y = x * 2 + 1, z = x * 2 + 2;
        tree[x] = vector<int> (tree[y].size() + tree[z].size());
        while (i < tree[y].size() && j < tree[z].size()) {
            if (tree[y][i] < tree[z][j]) {
                tree[x][i + j] = tree[y][i];
                i++;
            } else {
                tree[x][i + j] = tree[z][j];
                j++;
            }
        }
        while (i < tree[y].size()) {
            tree[x][i + j] = tree[y][i];
            i++;
        }
        while (j < tree[z].size()) {
            tree[x][i + j] = tree[z][j];
            j++;
        }
    }

    segtree(int n, vector<int> &a) {
        while (size < n) size <<= 1;
        tree = vector<vector<int>> (size * 2 - 1, vector<int>(0));
        build(0, 0, size, a);
    }

    tuple<int, int, int> mim(int l, int r, int k, int x, int lx, int rx) {
        if (r <= lx || rx <= l) {
            return {0, -1, inf};
        }
        if (l <= lx && rx <= r) {
            int ll = -1, rr = tree[x].size();
            while (rr - ll > 1) {
                int m = (rr + ll);
                if (tree[x][m] > k) {
                    rr = m;
                } else {
                    ll = m;
                }
            }
            int ass = ll;
            ll = -1, rr = tree[x].size();
            while (rr - ll > 1) {
                int m = (rr + ll) / 2;
                if (tree[x][m] < k) {
                    ll = m;
                } else {
                    rr = m;
                }
            }
            if (ass == -1) {
                if (rr == tree[x].size()) {
                    return {0, -1, inf};
                } else {
                    return {0, -1, tree[x][rr]};
                }
            } else {
                if (rr == tree[x].size()) {
                    return {ass + 1, tree[x][ass], inf};
                }
                return {ass + 1, tree[x][ass], tree[x][rr]};
            }
        }
        int m = (rx + lx) / 2;
        auto x1 = mim(l, r, k, x * 2 + 1, lx, m);
        auto x2 = mim(l, r, k, x * 2 + 2, m, rx);
        return {get<0>(x1) + get<0>(x2), max(get<1>(x1), get<1>(x2)), min(get<2>(x1), get<2>(x2))};
    }

    tuple<int, int, int> mim(int l, int r, int k) {
        return mim(l, r, k, 0, 0, size);
    }
};

bool change(int x, int y, vector<pair<int, int>> &c, segtree &tree) {
    int l = -1, r = c.size();
    while (r - l > 1) {
        int m = (r + l) / 2;
        if (c[m].first > x) {
            r = m;
        } else {
            l = m;
        }
    }
    int ans = r + get<0>(tree.mim(r, c.size(), y));
    if (ans >= c.size()) {
        return true;
    } else {
        return false;
    }
}

void dfs(int x, int ma, int p, int ass, vector<vector<pair<int, int>>> &a, vector<bool> &used, vector<int> &dist, vector<int> &prof) {
    used[x] = 0;
    ma = max(ma, prof[x]);
    if (dist[x] == -1 || dist[x] > ass) {
        dist[x] = ass;
    }
    for (int i = 0; i < a[x].size(); i++) {
        int nv = a[x][i].first, w = a[x][i].second;
        if (used[nv]) {
            int y = 0;
            if (w > p) {
                y = (w - p - 1) / ma + 1;
            }
            dfs(nv, ma, p + ma * y - w, ass + y, a, used, dist, prof);
        }
    }
    used[x] = 1;
}

void solve() {
    int n, m, p, g;
    cin >> n >> m >> p >> g;
    vector<int> prof(n);
    for (int i = 0; i < n; i++) {
        cin >> prof[i];
    }
    if (g == 1) {
        set<pair<int, int>> q;
        vector<vector<pair<int, int>>> a(n);
        for (int i = 0; i < m; i++) {
            int x, y, w;
            cin >> x >> y >> w;
            a[x - 1].push_back({y - 1, w});
        }
        vector<int> dist(n, -1);
        dist[0] = 0;
        q.insert({0, 0});
        while (!q.empty()) {
            int v = q.begin()->second;
            q.erase(q.begin());
            for (int i = 0; i < a[v].size(); i++) {
                int nv = a[v][i].first, w = a[v][i].second;
                if (dist[nv] > dist[v] + w || dist[nv] == -1) {
                    q.erase({dist[nv], nv});
                    dist[nv] = dist[v] + w;
                    q.insert({dist[nv], nv});
                }
            }
        }
        if (dist[n - 1] == -1) {
            cout << -1;
        } else {
            cout << max(0ll, dist[n - 1] - p);
        }
    } else if (g == 2) {
        int ma = 0;
        int ans = 0;
        for (int i = 0; i < m; i++) {
            ma = max(ma, prof[i]);
            int x, y, w;
            cin >> x >> y >> w;
            if (p < w) {
                int x = (w - p - 1) / ma + 1;
                ans += x;
                p += x * ma;
            }
            p -= w;
        }
        cout << ans;
    } else {
        vector<int> dist(n, -1);
        vector<bool> used(n, 1);
        vector<vector<pair<int, int>>> a(n);
        for (int i = 0; i < m; i++) {
            int x, y, w;
            cin >> x >> y >> w;
            a[x - 1].push_back({y - 1, w});
        }
        dist[0] = 0;
        dfs(0, 0, p, 0, a, used, dist, prof);
        cout << dist[n - 1];
    }
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    int t = 1;
    while (t-->0) {
        solve();
    }
}
