#include <algorithm>
#include <iostream>
#include <set>
#include <vector>

using namespace std;

constexpr long long INF = 100000000000000000ll;

vector<vector<pair<int, int>>> gr;
vector<int> w;

namespace Solveg1 {
    void solve(int n, int p) {
        vector<long long> dists(n, INF);
        dists[0] = 0;
        set<pair<long long, int>> q;
        q.insert({0, 0});
        while (!q.empty()) {
            auto [cc, cv] = *q.begin();
            q.erase(q.begin());
            for (auto [to, c] : gr[cv]) {
                if (dists[to] > cc + c) {
                    q.erase({dists[to], to});
                    dists[to] = cc + c;
                    q.insert({dists[to], to});
                }
            }
        }

        long long cdst = dists[n - 1];
        if (cdst == INF) {
            cout << "-1\n";
        } else {
            long long ck = max(0ll, cdst - p);
            cout << ck << "\n";
        }
    }
} // namespace Solveg1

namespace Solveg2 {
    void solve(int n, int p) {
        vector<long long> a(n);
        for (int i = 0; i < n - 1; i++) {
            a[i] = gr[i][0].second;
        }
        vector<long long> sumpaths(n, 0);
        for (int i = n - 2; i >= 0; i--) {
            sumpaths[i] = sumpaths[i + 1] + a[i];
        }

        vector<int> incr;
        incr.reserve(n);
        for (int i = 0; i < n; i++) {
            if (incr.empty() || w[i] > w[incr.back()]) {
                incr.emplace_back(i);
            }
        }
        if (incr.back() != n - 1) {
            incr.emplace_back(n - 1);
        }

        long long res = 0;
        int cv = 0;
        int cur = 0;
        while (cv != n - 1) {
            long long csum = sumpaths[incr[cur]] - sumpaths[incr[cur + 1]];
            long long ck = max(0ll, (csum - p + w[cv] - 1) / w[cv]);
            res += ck;
            p += ck * w[cv];
            p -= csum;
            cur++;
            cv = incr[cur];
        }
        cout << res << "\n";
    }
} // namespace Solveg2

namespace Solveg3 {
    long long dfs(int v, long long maxc, long long cp, int n, multiset<int>& seen) {
        if (seen.count(v) > 2) {
            return INF;
        }
        if (v == n - 1) {
            return 0;
        }
        seen.insert(v);
        maxc = max(maxc, (long long)w[v]);
        long long bestres = INF;
        for (auto [to, cw] : gr[v]) {
            long long cadd = max(0ll, (cw - cp + maxc - 1) / maxc);
            bestres = min(bestres, dfs(to, maxc, cp - cw + cadd * maxc, n, seen) + cadd);
        }
        seen.erase(v);
        return bestres;
    }
    void solve(int n, int p) {
        for (int i = 0; i < n; i++) {
            sort(gr[i].begin(), gr[i].end());
            gr[i].resize(unique(gr[i].begin(), gr[i].end(), [](const pair<int, int>& p1, const pair<int, int>& p2) {
                return p1.first == p2.first;
            }) - gr[i].begin());
        }

        multiset<int> seen;
        long long bestres = dfs(0, 0, p, n, seen);
        if (bestres == INF) {
            cout << "-1\n";
        } else {
            cout << bestres << "\n";
        }
    }
} // namespace Solveg3

int main() {
    int n, m, p, g;
    cin >> n >> m >> p >> g;
    w.resize(n);
    for (int i = 0; i < n; i++) {
        cin >> w[i];
    }
    gr.resize(n);
    for (int i = 0; i < m; i++) {
        int a, b, c;
        cin >> a >> b >> c;
        a--;
        b--;
        gr[a].emplace_back(b, c);
    }
    if (g == 1) {
        Solveg1::solve(n, p);
    } else if (g == 2) {
        Solveg2::solve(n, p);
    } else {
        Solveg3::solve(n, p);
    }
    return 0;
}
