#include <iostream>
#include <set>
#include <map>
#include <vector>
#include <string>
#include <tuple>

using namespace std;

#define range(i, n) for (int i = 0; i < n; i++)
#define loop(n) range(_, n)

#ifdef LOCAL
#define debug(x) cerr << #x << ": " << x << endl;
#define endline endl
#else
#define debug(x) ;
#define endline "\n"
#endif

const int INF = 2e9;
const long long INFLL = 1e18;


template<typename T>
istream &operator>>(istream &in, pair<T, T> &p) {
    in >> p.first >> p.second;
    return in;
}

template<typename T>
istream &operator>>(istream &in, vector<T> &v) {
    for (T &value: v) {
        in >> value;
    }
    return in;
}

template<typename T>
ostream &operator<<(ostream &out, vector<T> &v) {
    for (T &value: v) {
        out << value << " ";
    }
    return out;
}


pair<long long, long long> bested(int w, pair<long long, long long> p1, pair<long long, long long> p2) {
    if (p1.first < p2.first) {
        swap(p1, p2);
    }
    if (p1.second > (p1.first - p2.first) * w + p2.second) {
        return p1;
    } else {
        return p2;
    }
}

int main() {
#ifndef LOCAL
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif
    int n, m, group;
    long long p;
    cin >> n >> m >> p >> group;
    vector<int> w(n);
    cin >> w;
    vector<vector<pair<int, int>>> g(n);
    vector<tuple<int, int, int>> edges(m);
    range(i, m) {
        int a, b, s;
        cin >> a >> b >> s;
        a--;
        b--;
        g[a].emplace_back(b, s);
        edges[i] = {a, b, s};
    }
    if (group == 1) {
        set<pair<long long, int>> q;
        vector<long long> min_dist(n, INFLL);
        q.insert({0, 0});
        min_dist[0] = 0;
        while (!q.empty()) {
            auto [d, v] = *q.begin();
            q.erase(q.begin());
            for (auto [u, s]: g[v]) {
                if (min_dist[u] > d + s) {
                    q.erase({min_dist[u], u});
                    min_dist[u] = d + s;
                    q.insert({min_dist[u], u});
                }
            }
        }
        if (min_dist[n - 1] == INFLL) {
            cout << -1 << endline;
        } else {
            cout << max(0LL, min_dist[n - 1] - p) << endline;
        }
    } else if (group == 2) {
        long long ans = 0;
        int max_w = 0;
        range(i, n - 1) {
            max_w = max(max_w, w[i]);
            int s = g[i][0].second;
            long long cnt = max(0LL, (s - p + max_w - 1) / max_w);
            ans += cnt;
            p += cnt * max_w;
            p -= s;
        }
        cout << ans << endline;
    } else {
        set<int> s_;
        for (int w0: w) {
            s_.insert(w0);
        }
        map<int, int> zip;
        vector<int> unzip(s_.size());
        int _i = 0;
        for (int w0: s_){
            zip[w0] = _i;
            unzip[_i] = w0;
            _i++;
        }
        vector<vector<pair<long long, long long>>> data(n, vector<pair<long long, long long>>(zip.size(), {INFLL, -1}));
        data[0][zip[w[0]]] = {0, p};
        long long best = INFLL;
        loop(2 * n) {
            for (auto [a, b, s]: edges) {
                range(i, s_.size()) {
                    int w0 = unzip[i];
                    auto [cnt0, p0] = data[a][i];
                    int w1 = zip[w[b]];
                    int i1 = zip[max(w0, w1)];
                    long long cnt1 = min(0LL, s - p0 + w0 - 1) / w0;
                    if (b == n - 1) {
                        best = min(best, cnt0 + cnt1);
                    } else {
                        data[b][i1] = bested(w1, data[b][i1], {cnt0 + cnt1, p0 + cnt1 * w0 - s});
                    }
                }
            }
        }
        if (best == INFLL) {
            cout << -1 << endline;
        } else {
            cout << best << endline;
        }
    }
}