#include <algorithm>
#include <iostream>
#include <iomanip>
#include <numeric>
#include <vector>
#include <stack>
#include <set>

typedef long long llong;
const int MAXN = 200000 + 10;

int n;
int perm[MAXN];
std::vector <int> songs[MAXN];
std::vector <std::pair <int,int>> nums;

struct BIT
{
    int tree[MAXN];
    void update(int pos, int value)
    {
        pos = n + 1 - pos;
        for (int idx = pos ; idx <= n ; idx += idx & (-idx))
        {
            tree[idx] = std::max(tree[idx], value);
        }
    }

    int query(int pos)
    {
        int ans = 0;
        pos = n + 1 - pos;
        for (int idx = pos ; idx >= 1 ; idx -= idx & (-idx))
        {
            ans = std::max(ans, tree[idx]);
        }

        return ans;
    }
};

BIT fenwick;
int dp[MAXN];
bool vis[MAXN];
int inPerm[MAXN];
void solve()
{
    std::iota(perm + 1, perm + 1 + n, 1);
    std::sort(perm + 1, perm + 1 + n, [&](const int &a, const int &b)
    {
        return songs[a].back() < songs[b].back();
    });
    
    for (int i = 1 ; i <= n ; ++i)
    {
        inPerm[perm[i]] = i;
    }

    std::sort(nums.begin(), nums.end(), [&](const auto &a, const auto &b)
    {
        return a.first > b.first || (a.first == b.first && inPerm[a.second] < inPerm[b.second]);
    });

    for (auto &[value, playlist] : nums)
    {
        playlist = inPerm[playlist];
        if (!vis[playlist])
        {
            dp[playlist] = fenwick.query(playlist + 1);
            vis[playlist] = true;
        }

        dp[playlist]++;
        fenwick.update(playlist, dp[playlist]);
    }

    int ans = 0;
    for (int i = 1 ; i <= n ; ++i)
    {
        ans = std::max(ans, dp[i]);
    }

    std::cout << ans << '\n';
}

void input()
{
    std::cin >> n;
    for (int i = 1 ; i <= n ; ++i)
    {
        int k, curr;
        std::cin >> k;
        for (int j = 1 ; j <= k ; ++j)
        {
            std::cin >> curr;
            if (songs[i].empty() || curr > songs[i].back())
            {
                songs[i].push_back(curr);
                nums.push_back({curr, i});
            }
        }
    }
}

void fastIO()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIO();
    input();
    solve();

    return 0;
}