#include <iostream>
#include <vector>
#include <algorithm>
#include <map>
#include <set>
#include <unordered_map>
#include <iomanip>
#include <bitset>
#include <math.h>
#include <cmath>
#include <random>
using namespace std;
typedef long long ll;
typedef long double ld;
mt19937 rnd(time(NULL));

struct treap {
    struct node {
        ll l, r, val, pos, p, pr;
        node(ll _val, ll _pos) {
            val = _val;
            pos = _pos;
            pr = rnd();
            p = 0;
            l = -1;
            r = -1;
        }
    };
    vector<node> t;
    ll root = -1;

    void push(ll v) {
        if (v == -1 || !t[v].p) return;
        if (t[v].l != -1) {
            t[t[v].l].pos += t[v].p;
            t[t[v].l].p += t[v].p;
        }
        if (t[v].r != -1) {
            t[t[v].r].pos += t[v].p;
            t[t[v].r].p += t[v].p;
        }
        t[v].p = 0;
    }

    pair<ll, ll> split(ll v, ll x) {
        push(v);
        if (v == -1) return { -1, -1 };
        if (t[v].pos >= x) {
            pair<ll, ll> res = split(t[v].l, x);
            t[v].l = res.second;
            res.second = v;
            return res;
        }
        else {
            pair<ll, ll> res = split(t[v].r, x);
            t[v].r = res.first;
            res.first = v;
            return res;
        }
    }
    ll merge(ll l, ll r) {
        push(l);
        push(r);
        if (l == -1) return r;
        if (r == -1) return r;
        if (t[l].pr > t[r].pr) {
            t[l].r = merge(t[l].r, r);
            return l;
        }
        else {
            t[r].l = merge(l, t[r].l);
            return r;
        }
    }
    void insert(ll val, ll pos) {
        pair<ll, ll> res = split(root, pos);
        t.push_back(node(val, pos));
        root = merge(res.first, merge(t.size() - 1, res.second));
    }
    void erase(ll pos) {
        pair<ll, ll> res = split(root, pos);
        pair<ll, ll> res2 = split(res.second, pos + 1);
        root = merge(res.first, res2.second);
    }
};

struct dsu {
    vector<ll> p, r;
    void init(ll n) {
        p.resize(n);
        r.assign(n, 0);
        for (int i = 0; i < n; ++i) p[i] = i;
    }
    void un(ll a, ll b) {
        a = get(a);
        b = get(b);
        if (a != b) {
            if (r[a] == r[b]) ++r[a];
            if (r[a] < r[b]) p[a] = b;
            else p[b] = a;
        }
    }
    ll get(ll a) {
        if (a != p[a]) p[a] = get(p[a]);
        return p[a];
    }
};

struct segtree {
    vector<ll> tree;
    ll size = 1;
    void init(ll n) {
        while (size < n) size *= 2;
        tree.assign(size * 2 - 1, 0);
    }
    void build(vector<ll>& a, ll x, ll lx, ll rx) {
        if (rx - lx == 1) {
            if (lx < a.size()) tree[x] = a[lx];
            return;
        }
        ll m = (lx + rx) / 2;
        build(a, x * 2 + 1, lx, m);
        build(a, x * 2 + 2, m, rx);
        tree[x] = tree[x * 2 + 1] + tree[x * 2 + 2];
    }
    void build(vector<ll>& a) {
        build(a, 0, 0, size);
    }
    void upd(ll ind, ll val, ll x, ll lx, ll rx) {
        if (rx - lx == 1) {
            tree[x] = val;
            return;
        }
        ll m = (lx + rx) / 2;
        if (ind < m) upd(ind, val, x * 2 + 1, lx, m);
        else upd(ind, val, x * 2 + 2, m, rx);
        tree[x] = tree[x * 2 + 1] + tree[x * 2 + 2];
    }
    void upd(ll ind, ll val) {
        upd(ind, val, 0, 0, size);
    }
    ll qry(ll l, ll r, ll x, ll lx, ll rx) {
        if (l >= rx || r <= lx) return 0;
        if (l <= lx && rx <= r) return tree[x];
        ll m = (lx + rx) / 2;
        return qry(l, r, x * 2 + 1, lx, m) + qry(l, r, x * 2 + 2, m, rx);
    }
    ll qry(ll l, ll r) {
        return qry(l, r, 0, 0, size);
    }
};

struct point {
    ll x, y;
};

point gv(point a, point b) {
    return { a.x - b.x, a.y - b.y };
}

ll cross(point a, point b) {
    return a.x * b.y - a.y * b.x;
}

ll dot(point a, point b) {
    return a.x * b.x + a.y * b.y;
}

vector<vector<pair<ll, ll>>> g;
vector<ll> dist;

void dfs(ll v) {
    for (auto to : g[v]) {
        dist[to.first] = max(dist[to.first], dist[v] + to.second);
        dfs(to.first);
    }
}

int main() {
    ios::sync_with_stdio(0); cin.tie(0); cout.tie(0);
    ll n;
    cin >> n;
    vector<vector<ll>> a(n);
    for (int i = 0; i < n; ++i) {
        ll k;
        cin >> k;
        a[i].resize(k);
        for (int j = 0; j < k; ++j) {
            cin >> a[i][j];
        }
    }
    vector<ll> mxel(n);
    vector<vector<ll>> p(n);
    for (int i = 0; i < n; ++i) {
        ll now = 0;
        for (ll w : a[i]) {
            if (w > now) {
                now = w;
                p[i].push_back(w);
            }
        }
        mxel[i] = now;
    }
    g.resize(n);
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < n; ++j) {
            if (mxel[i] >= mxel[j]) continue;
            ll ct = p[j].end() - upper_bound(p[j].begin(), p[j].end(), mxel[i]);
            g[i].push_back({ j, ct });
        }
    }
    ll ans = 0;
    for (int i = 0; i < n; ++i) {
        dist.assign(n, 0);
        dist[i] = p[i].size();
        dfs(i);
        for (ll w : dist) ans = max(ans, w);
    }
    cout << ans;
}
