#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <map>
#include <math.h>
#include <stack>
#include <random>
#include <set>

using namespace std;

mt19937_64 rnd(17112004);

const int inf = (1 << 31) - 1;

struct Node {
    int val, pr, sz;
    Node* l;
    Node* r;

    Node(int x) {
        val = x;
        pr = rnd();
        sz = 1;
        l = r = nullptr;
    }
};

int getSz(Node* t) {
    if (t == nullptr) {
        return 0;
    }
    return t->sz;
}

void upd(Node* t) {
    if (t == nullptr) {
        return;
    }
    t->sz = getSz(t->l) + getSz(t->r) + 1;
}

pair<Node* , Node*> split(Node* t, int x) {
    if (t == nullptr) {
        return {nullptr, nullptr};
    }
    if (getSz(t->l) + 1 < x) {
        auto n = split(t->r, x - getSz(t->l) - 1);
        t->r = n.first;
        upd(t);
        return {t, n.second};
    } else if (getSz(t->l) + 1 == x) {
        Node* n = t->r;
        t->r = nullptr;
        upd(t);
        return {t, n};
    } else {
        auto n = split(t->l, x);
        t->l = n.second;
        upd(t);
        return {n.first, t};
    }
}

Node* merge(Node* l, Node* r) {
    if (l == nullptr) {
        return r;
    }
    if (r == nullptr) {
        return l;
    }
    if (l->pr > r->pr) {
        auto n = merge(l->r, r);
        l->r = n;
        upd(l);
        return l;
    } else {
        auto n = merge(l, r->l);
        r->l = n;
        upd(r);
        return r;
    }
}

void sw(Node* &t, int m) {
    auto n = split(t, m);
    t = merge(n.second, n.first);
}

void print(Node* t) {
    if (t == nullptr) {
        return;
    }
    print(t->l);
    cout << t->val << " ";
    print(t->r);
}

struct segtree {
    vector<vector<int>> tree;
    int size = 1;

    void build(int x, int lx, int rx, vector<int> &a) {
        if (rx - lx == 1) {
            if (lx < a.size()) {
                tree[x] = {a[lx]};
            }
            return;
        }
        int m = (rx + lx) / 2;
        build(x * 2 + 1, lx, m, a);
        build(x * 2 + 2, m, rx, a);
        int i = 0, j = 0;
        int y = x * 2 + 1, z = x * 2 + 2;
        tree[x] = vector<int> (tree[y].size() + tree[z].size());
        while (i < tree[y].size() && j < tree[z].size()) {
            if (tree[y][i] < tree[z][j]) {
                tree[x][i + j] = tree[y][i];
                i++;
            } else {
                tree[x][i + j] = tree[z][j];
                j++;
            }
        }
        while (i < tree[y].size()) {
            tree[x][i + j] = tree[y][i];
            i++;
        }
        while (j < tree[z].size()) {
            tree[x][i + j] = tree[z][j];
            j++;
        }
    }

    segtree(int n, vector<int> &a) {
        while (size < n) size <<= 1;
        tree = vector<vector<int>> (size * 2 - 1, vector<int>(0));
        build(0, 0, size, a);
    }

    int mim(int l, int r, int k, int x, int lx, int rx) {
        if (r <= lx || rx <= l) {
            return -1;
        }
        if (l <= lx && rx <= r) {
            int ll = -1, rr = tree[x].size();
            while (rr - ll > 1) {
                int m = (rr + ll);
                if (tree[x][m] > k) {
                    rr = m;
                } else {
                    ll = m;
                }
            }
            if (ll == -1) {
                return -1;
            } else {
                return tree[x][ll];
            }
        }
        int m = (rx + lx) / 2;
        int x1 = mim(l, r, k, x * 2 + 1, lx, m);
        int x2 = mim(l, r, k, x * 2 + 2, m, rx);
        return max(x1, x2);
    }

    int mim(int l, int r, int k) {
        return mim(l, r, k, 0, 0, size);
    }

    int mam(int l, int r, int k, int x, int lx, int rx) {
        if (r <= lx || rx <= l) {
            return inf;
        }
        if (l <= lx && rx <= r) {
            int ll = -1, rr = tree[x].size();
            while (rr - ll > 1) {
                int m = (rr + ll) / 2;
                if (tree[x][m] < k) {
                    ll = m;
                } else {
                    rr = m;
                }
            }
            if (rr == tree[x].size()) {
                return inf;
            } else {
                return tree[x][rr];
            }
        }
        int m = (rx + lx) / 2;
        int x1 = mam(l, r, k, x * 2 + 1, lx, m);
        int x2 = mam(l, r, k, x * 2 + 2, m, rx);
        return min(x1, x2);
    }

    int mam(int l, int r, int k) {
        return mam(l, r, k, 0, 0, size);
    }
};

bool change(int y, int n, int x, vector<int> &c, vector<int> &d) {
    int l = -1, r = n;
    while (r - l > 1) {
        int m = (r + l) / 2;
        if (c[m] > x) {
            r = m;
        } else {
            l = m;
        }
    }
    int ass = r;
    l = -1;
    r = n;
    while (r - l > 1) {
        int m = (r + l) / 2;
        if (d[m] > y) {
            r = m;
        } else {
            l = m;
        }
    }
    if (ass + r >= n) {
        return true;
    }
    return false;
}

void solve() {
    int n;
    cin >> n;
    vector<int> a(n), b(n);
    set<pair<int, int>> s1, s2;
    for (int i = 0; i < n; i++) {
        cin >> a[i] >> b[i];
        if (a[i] > b[i]) {
            s1.insert({-a[i], i});
        } else {
            s2.insert({-b[i], i});
        }
    }
    if (s1.size() == 0) {
        int i = s2.begin()->second;
        s2.erase(s2.begin());
        s1.insert({-b[i], i});
    }
    if (s2.empty()) {
        int i = s1.begin()->second;
        s1.erase(s1.begin());
        s2.insert({-a[i], i});
    }
    int ans = inf;
    for (int i = 0; i < n * 10; i++) {
        ans = min(ans, abs(s1.begin()->first - s2.begin()->first));
        if (s1.begin()->first < s2.begin()->first) {
            int j = s1.begin()->second;
            s1.erase(s1.begin());
            s2.insert({-b[j], j});
        } else {
            int j = s2.begin()->second;
            s2.erase(s2.begin());
            s1.insert({-a[j], j});
        }
    }
    cout << ans;
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
    int t = 1;
    while (t-->0) {
        solve();
    }
}
