#include <iostream>
#include <vector>
#include <algorithm>
#include <unordered_map>
#include <set>
#include <random>
#include <time.h>

using namespace std;
mt19937 rnd(time(0));

typedef long long ll;
typedef vector<int> vi;
typedef vector<vector<pair<int, int>>> graph;

const int INF = 1e9;

vector<int> w;
vector<vector<pair<int, int>>> g;

struct Treap {
	Treap* l;
	Treap* r;
	int x, y;
	int cnt;
	Treap(int x) : x(x), cnt(1), y(rnd()), l(nullptr), r(nullptr) {}
};

void set_l(Treap* v, Treap* l) {
	v->l = l;
}
void set_r(Treap* v, Treap* r) {
	v->r = r;
}

Treap* merge(Treap* l, Treap* r) {
	if (l == nullptr || r == nullptr)
		return l == nullptr ? r : l;
	if (l->y < r->y) {
		set_l(r, merge(l, r->l));
		return r;
	}
	else {
		set_r(l, merge(l->r, r));
		return l;
	}
}
pair<Treap*, Treap*> split(Treap* v, int x) {
	if (v == nullptr)
		return { nullptr, nullptr };
	if (x <= v->x) {
		auto [l, r] = split(v->l, x);
		v->l = r;
		return { l, v };
	}
	else {
		auto [l, r] = split(v->r, x);
		v->r = l;
		return { v, r };
	}
}

Treap* insert(Treap* v, int x) {
	auto [l, t] = split(v, x);
	auto [mid, r] = split(t, x + 1);
	if (mid == nullptr)
		mid = new Treap(x);
	else
		++mid->cnt;
	return merge(merge(l, mid), r);
}
void show(Treap* v) {
	if (v == nullptr)
		return;
	show(v->l);
	cout << v->x << " " << v->cnt << "    ";
	show(v->r);
}
Treap* erase(Treap* v, int x) {
	auto [l, t] = split(v, x);
	auto [mid, r] = split(t, x + 1);
	//cout << "l: "; show(l);
	//cout << "\nr: "; show(r);
	if (mid == nullptr)
		cout << "A";
	--mid->cnt;
	if (mid->cnt == 0)
		mid = nullptr;
	return merge(merge(l, mid), r);
}

int getr(Treap* v) {
	if (v == nullptr)
		return -INF;
	if (v->r == nullptr)
		return v->x;
	return getr(v->r);
}
int getl(Treap* v) {
	if (v == nullptr)
		return -INF;
	if (v->l == nullptr)
		return v->x;
	return getl(v->l);
}

Treap* findlr(Treap* v, int x, int& yl, int& yr) {
	auto [l, r] = split(v, x);
	yl = getr(l);
	yr = getl(r);
	return merge(l, r);
}

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);
	cout.tie(0);

	Treap* head = nullptr;
	int n;
	cin >> n;
	vector<pair<int, int>> a(n);
	for (int i = 0; i < n; ++i) {
		cin >> a[i].first >> a[i].second;
		head = insert(head, a[i].second);
	}
	sort(a.begin(), a.end());
	reverse(a.begin(), a.end());
	int mx = 0;
	int ans = INF;
	for (int i = 0; i < n; ++i) {
		//show(head);
		//cout << "\n";
		//head = erase(head, a[i].second);
		int x = a[i].first;
		int yl, yr;
		findlr(head, x, yl, yr);
		yl = max(yl, mx);
		yr = max(yr, mx);
		int cur = min(min(abs(x - yl), abs(x - yr)), abs(x - mx));
		ans = min(ans, cur);
		//cout << "   " << i << " " << cur << "\n";
		mx = max(mx, a[i].second);
	}
	cout << ans << "\n";
}