#include <iostream>
#include <vector>
#include <set>
#include <map>
#include <queue>
#include <unordered_set>
#include <unordered_map>
#include <algorithm>
#include <numeric>
#include <cmath>

#pragma GCC optimize("O3,unroll-loops")

using namespace std;

using ll = long long;
using ld = long double;

const int INT_INF = 2e9 + 10;
const ll LL_INF = 2e18 + 10;

template<typename T> istream& operator >> (istream& in, vector<T>& a) {
    for (auto& i : a) {
        in >> i;
    }
    return in;
}

struct Elem {
    int a, b;
    Elem() : a(0), b(0) {}
    Elem(int _a, int _b) : a(_a), b(_b) {}
};

vector<int> all;
unordered_map<int, int> compress;
vector<int> decompress;

struct Segment_tree_min {
    int n;
    vector<int> tree;
    Segment_tree_min() {
        n = all.size();
        tree.resize(4 * n, n);
    }
    int _get(int u, int l, int r, int lq) {
        if (r <= lq) {
            return n;
        }
        if (l >= lq) {
            return tree[u];
        }
        int m = (l + r) / 2;
        return min(_get(2 * u + 1, l, m, lq), _get(2 * u + 2, m, r, lq));
    }
    int get(int l) {
        int res = _get(0, 0, n, compress[l]);
        if (res != n) {
            return decompress[res];
        }
        return INT_INF;
    }
    void _add(int u, int l, int r, int i) {
        if (l > i || r <= i) {
            return;
        }
        if (l + 1 == r) {
            tree[u] = i;
            return;
        }
        int m = (l + r) / 2;
        int v1 = 2 * u + 1;
        int v2 = 2 * u + 2;
        _add(v1, l, m, i);
        _add(v2, m, r, i);
        tree[u] = min(tree[v1], tree[v2]);
    }
    void add(int i) {
        _add(0, 0, n, compress[i]);
    }
};

struct Segment_tree_max {
    int n;
    vector<int> tree;
    Segment_tree_max() {
        n = all.size();
        tree.resize(4 * n, -1);
    }
    int _get(int u, int l, int r, int rq) {
        if (l >= rq) {
            return -1;
        }
        if (r <= rq) {
            return tree[u];
        }
        int m = (l + r) / 2;
        return max(_get(2 * u + 1, l, m, rq), _get(2 * u + 2, m, r, rq));
    }
    int get(int r) {
        int res = _get(0, 0, n, compress[r]);
        if (res != -1) {
            return decompress[res];
        }
        return -INT_INF;
    }
    void _add(int u, int l, int r, int i) {
        if (l > i || r <= i) {
            return;
        }
        if (l + 1 == r) {
            tree[u] = i;
            return;
        }
        int m = (l + r) / 2;
        int v1 = 2 * u + 1;
        int v2 = 2 * u + 2;
        _add(v1, l, m, i);
        _add(v2, m, r, i);
        tree[u] = max(tree[v1], tree[v2]);
    }
    void add(int i) {
        _add(0, 0, n, compress[i]);
    }
};

void do_compression() {
    sort(all.begin(), all.end());
    all.resize(unique(all.begin(), all.end()) - all.begin());
    for (int i = 0; i < all.size(); i++) {
        compress[all[i]] = i;
        decompress.push_back(all[i]);
    }
}

int main() {
    ios_base::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);
    int n;
    cin >> n;
    vector<int> a(n), b(n);
    vector<Elem> p(n);
    for (int i = 0; i < n; i++) {
        cin >> a[i] >> b[i];
        p[i] = Elem(a[i], b[i]);
        all.push_back(a[i]);
        all.push_back(b[i]);
    }
    do_compression();
    vector<vector<int>> list(all.size());
    for (auto para : p) {
        list[compress[para.a]].push_back(para.b);
    }
    //sort(p.begin(), p.end(), [](const Elem& _a, const Elem& _b) { return _a.a < _b.a; });
    sort(a.begin(), a.end());
    a.resize(unique(a.begin(), a.end()) - a.begin());
    int ans = INT_INF;
    int start_max_b = -INT_INF;
    Segment_tree_min tree_min;
    Segment_tree_max tree_max;
    vector<int> ans_min(a.size());
    vector<int> ans_max(a.size());
    for (int i = 0; i < a.size(); i++) {
        ans_min[i] = tree_min.get(a[i]);
        ans_max[i] = tree_max.get(a[i]);
        for (auto v : list[compress[a[i]]]) {
            tree_min.add(v);
            tree_max.add(v);
        }
    }
    for (int i = a.size() - 1; i >= 0; i--) {
        int max_a = a[i];
        int max_b = start_max_b;
        if (max_b >= max_a) {
            ans = min(ans, max_b - max_a);
            break;
        }
        int min_b = INT_INF;
        if (i != 0) {
            max_b = max(max_b, ans_max[i]);
            min_b = min(min_b, ans_min[i]);
        }
        if (list[compress[a[i]]].size() > 1) {
            for (auto j : list[compress[a[i]]]) {
                if (j >= max_a) {
                    min_b = min(min_b, j);
                } else {
                    max_b = max(max_b, j);
                }
            }
        }
        ans = min(ans, min(max_a - max_b, min_b - max_a));
        for (auto x : list[compress[a[i]]]) {
            start_max_b = max(start_max_b, x);
        }
    }
    cout << ans;
}
