use std::{
    collections::HashMap,
    io::{self, prelude::*},
    mem::MaybeUninit,
};

static mut MEM: MaybeUninit<HashMap<usize, usize>> = MaybeUninit::uninit();

fn ask<R: BufRead, W: Write>(scan: &mut Scanner<R>, w: &mut W, p: usize) -> usize {
    *unsafe { MEM.assume_init_mut().entry(p) }.or_insert_with(|| {
        writeln!(w, "? {p}").expect("Failed write");
        scan.token()
    })
}

fn search<R: BufRead, W: Write>(
    scan: &mut Scanner<R>,
    w: &mut W,
    ans: &mut Vec<usize>,
    k: &mut usize,
    lo: usize,
    hi: usize,
) {
    if *k == 0 {
        return;
    }
    let (l, r) = (ask(scan, w, lo - 1), ask(scan, w, hi - 1));
    if r - l != (lo + hi - 1) * (hi - lo) / 2 {
        if hi - lo == 1 {
            ans.push(lo);
            *k -= 1;
        } else {
            let mid = (lo + hi) / 2;
            search(scan, w, ans, k, lo, mid);
            search(scan, w, ans, k, mid, hi);
        }
    }
}

fn solve<R: BufRead, W: Write>(scan: &mut Scanner<R>, w: &mut W) -> io::Result<()> {
    unsafe {
        MEM.write(HashMap::new());
        MEM.assume_init_mut().insert(0, 0);
    }
    let (n, mut k): (usize, usize) = (scan.token(), scan.token());
    let mut ans = vec![];
    search(scan, w, &mut ans, &mut k, 1, n + 1);

    writeln!(
        w,
        "! {}",
        ans.into_iter()
            .map(|x| x.to_string())
            .collect::<Vec<_>>()
            .join(" ")
    )
}

fn main() -> io::Result<()> {
    let mut scan = Scanner::new(io::stdin().lock());
    let mut w = io::stdout().lock();
    (0..scan.token()).try_for_each(|_| solve(&mut scan, &mut w))
}

struct Scanner<R> {
    reader: R,
    buf: Vec<u8>,
    buf_iter: std::str::SplitAsciiWhitespace<'static>,
}

impl<R: BufRead> Scanner<R> {
    fn new(reader: R) -> Self {
        Self {
            reader,
            buf: vec![],
            buf_iter: "".split_ascii_whitespace(),
        }
    }

    fn token<T: std::str::FromStr>(&mut self) -> T {
        loop {
            if let Some(token) = self.buf_iter.next() {
                return token.parse().ok().expect("Failed parse");
            }
            self.buf.clear();
            self.reader
                .read_until(b'\n', &mut self.buf)
                .expect("Failed read");
            self.buf_iter = unsafe {
                let slice = std::str::from_utf8_unchecked(&self.buf);
                std::mem::transmute(slice.split_ascii_whitespace())
            };
        }
    }
}
