#include <iostream>
#include <vector>
#include <unordered_map>
#include <algorithm>
#include <cassert>

using namespace std;

unordered_map<int, int64_t> query_cache;

vector<int> answer;

#ifdef STRESS
const int N = 10;
int arr[N + 1];
int K;
void generate_test() {
    // int new_arr[N + 1] = {0, 1, 0, 3, 4, 0, 0, 0, 8, 0, 10};
    // for (int i = 0; i <= N; i++) {
    //     arr[i] = new_arr[i];
    // }
    // K = 5;
    K = 0;
    cerr << "Removing coins: ";
    for (int i = 1; i <= N; i++) {
        arr[i] = i;
        if (rand() % 2 == 0) {
            K++;
            arr[i] = 0;
            cerr << i << " ";
        }
    }
    cerr << endl;
}
#endif

int64_t desired_sum(int l, int r) {
    return r * 1ll * (r +  1) / 2 - (l - 1) * 1ll * l / 2;
}

int64_t query(int p) {
#ifdef STRESS
    int64_t res = 0;
    for (int i = 0; i <= p; i++) {
        res += arr[i];
    }
    return res;
#endif
    if (p == 0) {
        return 0;
    }
    if (query_cache.find(p) == query_cache.end()) {
        cout << "? " << p << endl;
        int64_t x; cin >> x;
        if (x == -1) {
            exit(1);
        }
        query_cache[p] = x;
    }
    return query_cache[p];
}

int cherrypick(int l, int r, int64_t s) {
    //assert(r <= 2 * l);
    int64_t delta = desired_sum(l, r) - s;
    if (delta <= r) {
        return delta;
    }
    return -1;
}

void solve_cherrypick(int l, int r, int64_t sl, int64_t sr, vector<int> last_queries) {
    if (l > r) {
        return;
    }
    if (l == r) {
        if (sl == sr) {
            answer.push_back(l);
        }
        return;
    }
    if (sr - sl == desired_sum(l, r)) {
        return;
    }
    while (!last_queries.empty() && last_queries.back() < l) {
        last_queries.pop_back();
    }
    for (int i : last_queries) {
        int cherry = cherrypick(l, i, query(i) - sl);
        if (cherry > 0) {
            answer.push_back(cherry);
            solve_cherrypick(i + 1, r, query(i), sr, last_queries);
            return;
        }
    }
    int lo = l - 1;
    int hi = (last_queries.empty() ? r + 1 : last_queries[0]);
    last_queries = {};
    while (hi - lo > 1) {
        int mid = (lo + hi) / 2;
        int cherry = cherrypick(l, mid, query(mid) - sl);
        if (cherry > 0) {
            answer.push_back(cherry);
        }
        if (cherry >= 0) {
            solve_cherrypick(mid + 1, r, query(mid), sr, last_queries);
            // solve_cherrypick(mid + 1, r, query(mid), sr, {});
            return;
        }
        last_queries.push_back(mid);
        hi = mid;
    }
}

void solve(int l, int r, int64_t sl, int64_t sr) {
    if (l > r) {
        return;
    }
    if (l == r) {
        if (sl == sr) {
            answer.push_back(l);
        }
        return;
    }
    if (sr - sl == desired_sum(l, r)) {
        return;
    }
    int m = (l + r) / 2;
    int64_t sm = query(m - 1);
    solve_cherrypick(m, r, sm, sr, {});
    solve(l, m - 1, sl, sm);
}

int main() {
    srand(time(NULL));
    int t; cin >> t;
    while (t--) {
        query_cache.clear();
        answer.clear();
        int n, k;
#ifndef STRESS
        cin >> n >> k;
#else
        generate_test();
        n = N;
        k = K;
#endif
        solve(1, n, 0, query(n));
        sort(answer.begin(), answer.end());
        cout << "! ";
        for (int i : answer) {
#ifdef STRESS
            if (arr[i] > 0) {
                return 1;
            }
#endif
            cout << i << ' ';
        }
        cout << endl;
#ifdef STRESS
        assert(answer.size() == K);
#endif
        int result; cin >> result;
        if (result == -1) {
            return 1;
        }
    }
}
