// C++ includes used for precompiling -*- C++ -*-

// Copyright (C) 2003-2022 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 3, or (at your option)
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// Under Section 7 of GPL version 3, you are granted additional
// permissions described in the GCC Runtime Library Exception, version
// 3.1, as published by the Free Software Foundation.

// You should have received a copy of the GNU General Public License and
// a copy of the GCC Runtime Library Exception along with this program;
// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
// <http://www.gnu.org/licenses/>.

/** @file stdc++.h
 *  This is an implementation file for a precompiled header.
 */

 // 17.4.1.2 Headers

 // C
#ifndef _GLIBCXX_NO_ASSERT
#include <cassert>
#endif
#include <cctype>
#include <cerrno>
#include <cfloat>
#include <ciso646>
#include <climits>
#include <clocale>
#include <cmath>
#include <csetjmp>
#include <csignal>
#include <cstdarg>
#include <cstddef>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>
#include <cwchar>
#include <cwctype>

#if __cplusplus >= 201103L
#include <ccomplex>
#include <cfenv>
#include <cinttypes>
#include <cstdalign>
#include <cstdbool>
#include <cstdint>
#include <ctgmath>
#include <cuchar>
#endif

// C++
#include <algorithm>
#include <bitset>
#include <complex>
#include <deque>
#include <exception>
#include <fstream>
#include <functional>
#include <iomanip>
#include <ios>
#include <iosfwd>
#include <iostream>
#include <istream>
#include <iterator>
#include <limits>
#include <list>
#include <locale>
#include <map>
#include <memory>
#include <new>
#include <numeric>
#include <ostream>
#include <queue>
#include <set>
#include <sstream>
#include <stack>
#include <stdexcept>
#include <streambuf>
#include <string>
#include <typeinfo>
#include <utility>
#include <valarray>
#include <vector>

#if __cplusplus >= 201103L
#include <array>
#include <atomic>
#include <chrono>
#include <codecvt>
#include <condition_variable>
#include <forward_list>
#include <future>
#include <initializer_list>
#include <mutex>
#include <random>
#include <ratio>
#include <regex>
#include <scoped_allocator>
#include <system_error>
#include <thread>
#include <tuple>
#include <typeindex>
#include <type_traits>
#include <unordered_map>
#include <unordered_set>
#endif

#if __cplusplus >= 201402L
#include <shared_mutex>
#endif

#if __cplusplus >= 201703L
#include <any>
#include <charconv>
// #include <execution>
#include <filesystem>
#include <optional>
#include <memory_resource>
#include <string_view>
#include <variant>
#endif

#if __cplusplus >= 202002L
#include <barrier>
#include <bit>
#include <compare>
#include <concepts>
#if __cpp_impl_coroutine
# include <coroutine>
#endif
#include <latch>
#include <numbers>
#include <ranges>
#include <span>
#include <stop_token>
#include <semaphore>
#include <source_location>
#include <syncstream>
#include <version>
#endif

#if __cplusplus > 202002L
#include <expected>
#include <spanstream>
#if __has_include(<stacktrace>)
# include <stacktrace>
#endif
#include <stdatomic.h>
#endif

using namespace std;

#ifdef OG_Matveychick1
bool local = 1;
#else
bool local = 0;
#endif

#define for0(i,n) for(int i=0;i<n;i++)
#define for1(i,n) for(int i=1;i<n;i++)
#define fi first
#define se second
#define con continue
#define ret return
#define rfor(i,n) for (int i=n-1;i>=0;i--)
#define be(x) x.begin()
#define en(x) x.end()
#define all(x) be(x),en(x)
#define pb push_back
#define pob pop_back
#define ins insert
#define bs binary_search
#define lb lower_bound
#define ub upper_bound
#define sz(x) x.size()
#define ba(x) x.back()



typedef long long ll;
typedef vector<ll> vll;
typedef set<ll> sll;
typedef string str;
typedef vector<string> vs;
typedef pair<ll, ll> pll;
typedef vector<pll> vpll;
typedef vector<vll> vvll;
typedef long double ld;
typedef vector<vpll> vvpll;


void solve();

ll rnd(ll l, ll r)
{
	ret rand() % (r - l + 1) + l;
}

ll bin_pow(ll n, ll x, ll mod)
{
	ll r = 1;
	ll um = n;
	while (x)
	{
		if (x & 1)
		{
			r *= um;
			r %= mod;
		}
		x >>= 1;
		um *= um;
		um %= mod;
	}
	ret r;
}

ll mtf(ll a, ll b, ll mod)
{
	ret a* bin_pow(b, mod - 2, mod) % mod;
}

int main()
{
	srand(time(0));
	if (local)
	{
		//freopen("input.txt", "r", stdin);
		//freopen("output.txt", "w", stdout);
	}
	int t = 1;
	cin >> t;
	while (t--)
	{
		solve();
	}
}

ll zapr(ll x)
{
	cout << "? " << x << endl;
	ll r;
	cin >> r;
	ret r;
}

void solve()
{
	ll n, k;
	cin >> n >> k;
	if (n == 1)
	{
		cout << "! 1" << endl;
		ll x;
		cin >> x;
		ret;
	}
	map<ll, ll> sum;
	vll ans;
	vpll otr;
	ll x = zapr(n);
	sum[n] = x;
	sum[0] = 0;
	if (x != n * (n + 1) / 2)
	{
		otr.pb({ 0,n });
	}
	ll it = 0;
	while (sz(otr) != 0)
	{
		pll z = otr.back();
		if (z.second - z.first <= 2)
		{
			otr.pob();
			if (sum[z.se] == z.se * (z.se + 1) / 2 - z.se - z.se + 1)
			{
				ans.pb(z.se - 1);
				ans.pb(z.se);
				it = z.second;
				for (auto &x : sum)
				{
					if (x.first >= z.second - 1)
					{
						x.second += z.second - 1;
					}
					if (x.first >= z.se)
					{
						x.se += z.second;
					}
				}
			}
			else
			{
				ans.pb(z.se * (z.se + 1) / 2 - sum[z.se]);
				it = z.se * (z.se + 1) / 2 - sum[z.se];
				ll pl = z.se * (z.se + 1) / 2 - sum[z.se];
				for (auto& x : sum)
				{
					if (x.first >= pl)
					{
						x.second += pl;
					}
				}
			}
			for0(i, sz(otr))
			{
				if (otr[i].second <= it || sum[otr[i].second] == otr[i].se * (otr[i].se + 1) / 2)
				{
					it = max(it, otr[i].second);
					otr.erase(be(otr) + i);
					i--;
				}
				else
				{
					otr[i].fi = max(otr[i].first, it);
				}
			}
			for0(i, sz(otr))
			{
				if (otr[i].second <= it || sum[otr[i].second] == otr[i].se * (otr[i].se + 1) / 2)
				{
					it = max(it, otr[i].second);
					otr.erase(be(otr) + i);
					i--;
				}
				else
				{
					otr[i].fi = max(otr[i].first, it);
				}
			}
			con;
		}
		ll m = (z.first + z.se) / 2;
		ll x;
		if (sum.find(m) == en(sum))
		{
			x = zapr(m);
			for0(i, sz(ans))
			{
				if (ans[i] <= m)
				{
					x += ans[i];
				}
				else
				{
					break;
				}
			}
		}
		else
		{
			x = sum[m];
		}
		if (x == -1)
		{
			exit(0);
		}
		sum[m] = x;
		if (z.second - z.first == 4)
		{
			if (x != m * (m + 1) / 2)
			{
				if (x == m * (m + 1) / 2 - m - m + 1)
				{
					ans.pb(m - 1);
					ans.pb(m);
					it = m;
					for (auto& x : sum)
					{
						if (x.first >= m - 1)
						{
							x.second += m - 1;
						}
						if (x.first >= m)
						{
							x.se += m;
						}
					}
				}
				else
				{
					ans.pb(m * (m + 1) / 2 - x);
					it = m * (m + 1) / 2 - x;
					ll pl = m * (m + 1) / 2 - x;
					for (auto& x : sum)
					{
						if (x.first >= pl)
						{
							x.second += pl;
						}
					}
				}
			}
			else
			{
				if (sum[z.se] == z.se * (z.se + 1) / 2 - z.se - z.se + 1)
				{
					ans.pb(z.se - 1);
					ans.pb(z.se);
					it = z.second;
					for (auto& x : sum)
					{
						if (x.first >= z.second - 1)
						{
							x.second += z.second - 1;
						}
						if (x.first >= z.se)
						{
							x.se += z.second;
						}
					}
				}
				else
				{
					ans.pb(z.se* (z.se + 1) / 2 - sum[z.se]);
					it = z.se * (z.se + 1) / 2 - sum[z.se];
					ll pl = z.se * (z.se + 1) / 2 - sum[z.se];
					for (auto& x : sum)
					{
						if (x.first >= pl)
						{
							x.second += pl;
						}
					}
				}
			}
			for0(i, sz(otr))
			{
				if (otr[i].second <= it || sum[otr[i].second] == otr[i].se * (otr[i].se + 1) / 2)
				{
					it = max(it, otr[i].second);
					otr.erase(be(otr) + i);
					i--;
				}
				else
				{
					otr[i].fi = max(otr[i].first, it);
				}
			}
			for0(i, sz(otr))
			{
				if (otr[i].second <= it || sum[otr[i].second] == otr[i].se * (otr[i].se + 1) / 2)
				{
					it = max(it, otr[i].second);
					otr.erase(be(otr) + i);
					i--;
				}
				else
				{
					otr[i].fi = max(otr[i].first, it);
				}
			}
		}
		else if (z.second - z.first == 3)
		{
			if (x != m * (m + 1) / 2)
			{
				ans.pb(m);
				it = m;
				ll pl = m;
				for (auto& x : sum)
				{
					if (x.first >= pl)
					{
						x.second += pl;
					}
				}
			}
			else
			{
				if (sum[z.se] == z.se * (z.se + 1) / 2 - z.se - z.se + 1)
				{
					ans.pb(z.se - 1);
					ans.pb(z.se);
					it = z.second;
					for (auto& x : sum)
					{
						if (x.first >= z.second - 1)
						{
							x.second += z.second - 1;
						}
						if (x.first >= z.se)
						{
							x.se += z.second;
						}
					}
				}
				else
				{
					ans.pb(z.se * (z.se + 1) / 2 - sum[z.se]);
					it = z.se * (z.se + 1) / 2 - sum[z.se];
					ll pl = z.se * (z.se + 1) / 2 - sum[z.se];
					for (auto& x : sum)
					{
						if (x.first >= pl)
						{
							x.second += pl;
						}
					}
				}
			}
			for0(i, sz(otr))
			{
				if (otr[i].second <= it || sum[otr[i].second] == otr[i].se * (otr[i].se + 1) / 2)
				{
					it = max(it, otr[i].second);
					otr.erase(be(otr) + i);
					i--;
				}
				else
				{
					otr[i].fi = max(otr[i].first, it);
				}
			}
			for0(i, sz(otr))
			{
				if (otr[i].second <= it || sum[otr[i].second] == otr[i].se * (otr[i].se + 1) / 2)
				{
					it = max(it, otr[i].second);
					otr.erase(be(otr) + i);
					i--;
				}
				else
				{
					otr[i].fi = max(otr[i].first, it);
				}
			}
		}
		else if (x != m * (m + 1) / 2)
		{
			otr.pb({ z.first,m });
		}
		else
		{
			it = m;
			otr.pb({ m,z.se });
			for0(i, sz(otr))
			{
				if (otr[i].second <= it || sum[otr[i].second] == otr[i].se * (otr[i].se + 1) / 2)
				{
					it = max(it, otr[i].second);
					otr.erase(be(otr) + i);
					i--;
				}
				else
				{
					otr[i].fi = max(otr[i].first, it);
				}
			}
			for0(i, sz(otr))
			{
				if (otr[i].second <= it || sum[otr[i].second] == otr[i].se * (otr[i].se + 1) / 2)
				{
					it = max(it, otr[i].second);
					otr.erase(be(otr) + i);
					i--;
				}
				else
				{
					otr[i].fi = max(otr[i].first, it);
				}
			}
		}
	}
	cout << "! ";
	for0(i, sz(ans))
	{
		cout << ans[i] << " ";
	}
	cout << endl;
	ll y;
	cin >> y;
	if (y == -1)
	{
		exit(0);
	}
}