#pragma GCC optimize("Ofast")
#ifndef LOCAL
#   include <bits/stdc++.h>
#   include <ext/pb_ds/assoc_container.hpp>
#   define debug(...)
#endif
using namespace std;

constexpr int ord(const char &x) {
    return static_cast<int>(x - 'a');
}

namespace st {
    struct node {
        int val = 0;
        int left = -1;
        int right = -1;
    };
    int sz = 0;
    vector<node> tree;

    int new_node() {
        tree.emplace_back();
        return sz++;
    }

    int get_val(int x) {
        if (x == -1) {
            return 0;
        } else {
            return tree[x].val;
        }
    }

    int set(int i, int x, int lx = 1, int rx = 500500) {
        int y = x;
        x = new_node();
        if (rx - lx == 1) {
            tree[x].val = 1;
            return x;
        }
        if (y != -1) {
            tree[x].left = tree[y].left;
            tree[x].right = tree[y].right;
        }
        if (i < (lx + rx) / 2) {
            tree[x].left = set(i, tree[x].left, lx, (lx + rx) / 2);
        } else {
            tree[x].right = set(i, tree[x].right, (lx + rx) / 2, rx);
        }
        tree[x].val = get_val(tree[x].left) + get_val(tree[x].right);
        return x;
    }

    int get(int l, int r, int x, int lx = 1, int rx = 500500) {
        if (x == -1 || l >= rx || lx >= r) {
            return 0;
        }
        if (l <= lx && rx <= r) {
            return tree[x].val;
        }
        return get(l, r, tree[x].left, lx, (lx + rx) / 2) + get(l, r, tree[x].right, (lx + rx) / 2, rx);
    }
}

struct aho {
    struct node {
        int nx[26]{};
        int suff = -1;
        int p = -1;
        int chr = -1;
        int len = 0;
        bool cnt = false;
        int st_node = -1;
        node() {
            memset(nx, -1, sizeof nx);
        }
    };

    vector<node> tree = { node() };
    int sz = 1;

    int new_node() {
        tree.emplace_back();
        return sz++;
    }

    void add_string(string &s) {
        int u = 0;
        for (const char &x : s) {
            if (tree[u].nx[ord(x)] == -1) {
                int v = new_node();
                tree[u].nx[ord(x)] = v;
                tree[v].p = u;
                tree[v].chr = ord(x);
                tree[v].len = tree[u].len + 1;
            }
            u = tree[u].nx[ord(x)];
        }
        tree[u].cnt = true;
    }

    void build() {
        for (int &v : tree[0].nx) {
            if (v == -1) {
                v = 0;
            }
        }
        vector<int> A(sz - 1);
        iota(A.begin(), A.end(), 1);
        sort(A.begin(), A.end(), [&](int i, int j) {
            return tree[i].len < tree[j].len;
        });
        for (int u : A) {
            int v = tree[tree[u].p].suff;
            if (v == -1) {
                tree[u].suff = 0;
                for (int i = 0; i < 26; i++) {
                    if (tree[u].nx[i] == -1) {
                        tree[u].nx[i] = tree[0].nx[i];
                    }
                }
            } else {
                tree[u].suff = tree[v].nx[tree[u].chr];
                tree[u].st_node = tree[tree[u].suff].st_node;
                for (int i = 0; i < 26; i++) {
                    if (tree[u].nx[i] == -1) {
                        tree[u].nx[i] = tree[tree[u].suff].nx[i];
                    }
                }
            }
            if (tree[u].cnt) {
                tree[u].st_node = st::set(tree[u].len, tree[u].st_node);
            }
        }
    }
};

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
    int n, m;
    cin >> n >> m;
    string t;
    cin >> t;
    aho A, B;
    for (int i = 0; i < n; i++) {
        string s;
        cin >> s;
        A.add_string(s);
        reverse(s.begin(), s.end());
        B.add_string(s);
    }
    A.build();
    B.build();

    vector<int> stateA(t.size());
    vector<int> stateB(t.size());
    for (int u = 0, i = 0; i < (int)t.size(); i++) {
        int x = ord(t[i]);
        u = A.tree[u].nx[x];
        stateA[i] = u;
    }

    for (int u = 0, i = (int)t.size() - 1; i >= 0; i--) {
        int x = ord(t[i]);
        u = B.tree[u].nx[x];
        stateB[i] = u;
    }

    vector<int> ans(m);
    vector<tuple<int, int, int>> Q;
    for (int i = 0; i < m; i++) {
        int l, r;
        cin >> l >> r;
        l--, r--;
        Q.emplace_back(l, r, i);
    }

    sort(Q.begin(), Q.end(), [](auto A, auto B) {
        static const int k = 600;
        return tuple(get<0>(A) / k, get<1>(A)) < tuple(get<0>(B) / k, get<1>(B));
    });

    int r = -1;
    int l = 0;
    int cnt = 0;

    for (auto &[L, R, i] : Q) {
        while (r < R) {
            r++;
            cnt += st::get(1, r - l + 2, A.tree[stateA[r]].st_node);
        }
        while (l > L) {
            l--;
            cnt += st::get(1, r - l + 2, B.tree[stateB[l]].st_node);
        }
        while (r > R) {
            cnt -= st::get(1, r - l + 2, A.tree[stateA[r]].st_node);
            r--;
        }
        while (l < L) {
            cnt -= st::get(1, r - l + 2, B.tree[stateB[l]].st_node);
            l++;
        }
        ans[i] = cnt;
    }

    for (int i = 0; i < m; i++) {
        cout << ans[i] << " \n"[i == m - 1];
    }
    return 0;
}
