#include <iostream>
#include <vector>
#include <map>
#include <unordered_map>

#if !defined LOCAL && !defined DEBUG
#define USE_FAST_IO
#endif

#ifdef USE_FAST_IO
#define endl '\n'
#endif

using namespace std;

const int NULL_NODE = 0;

struct SuffixAutomaton {
    struct Node {
        map<char, int> go;
        int link;

        int length;
        int terminal_cnt;
        int sum_terminal_cnt;

        Node() {
            link = NULL_NODE;
            length = 0;
            terminal_cnt = 0;
            sum_terminal_cnt = -1;
        }
    };

    using vertex_t = int;

    vector<Node> nodes;
    vertex_t root, last;

    vertex_t new_node() {
        nodes.push_back(Node());
        return (int) nodes.size() - 1;
    }

    void add_char(char c) {
        if (nodes[last].go[c] != NULL_NODE) {
            vertex_t q = nodes[last].go[c];
            if (nodes[last].length + 1 == nodes[q].length) {
                last = q;
            } else {
                vertex_t clone = new_node();
                nodes[clone] = nodes[q];
                nodes[q].link = clone;
                nodes[clone].terminal_cnt = 0;
                nodes[clone].length = nodes[last].length + 1;
                while (last != NULL_NODE && nodes[last].go[c] == q) {
                    nodes[last].go[c] = clone;
                    last = nodes[last].link;
                }
                last = clone;
            }
            return;
        }
        vertex_t new_last = new_node();
        nodes[new_last].length = nodes[last].length + 1;
        vertex_t cur = last;
        while (cur != NULL_NODE && nodes[cur].go[c] == NULL_NODE) {
            nodes[cur].go[c] = new_last;
            cur = nodes[cur].link;
        }
        if (cur == NULL_NODE) {
            nodes[new_last].link = root;
            last = new_last;
            return;
        }
        vertex_t p = cur;
        vertex_t q = nodes[p].go[c];
        if (nodes[p].length + 1 == nodes[q].length) {
            nodes[new_last].link = q;
            last = new_last;
            return;
        }
        vertex_t clone = new_node();
        nodes[clone] = nodes[q];
        nodes[q].link = clone;
        nodes[clone].length = nodes[p].length + 1;
        nodes[clone].terminal_cnt = 0;
        while (p != NULL_NODE && nodes[p].go[c] == q) {
            nodes[p].go[c] = clone;
            p = nodes[p].link;
        }
        nodes[new_last].link = clone;
        last = new_last;
    }

    int get_sum_terminal_cnt(vertex_t v) {
        if (v == NULL_NODE) return 0;
        if (nodes[v].sum_terminal_cnt == -1) {
            nodes[v].sum_terminal_cnt = nodes[v].terminal_cnt + get_sum_terminal_cnt(nodes[v].link);
        }
        return nodes[v].sum_terminal_cnt;
    }

    int get_shortest(vertex_t v) {
        if (nodes[v].link == NULL_NODE) {
            return 0;
        }
        return nodes[nodes[v].link].length + 1;
    }

    SuffixAutomaton() {
        nodes = {Node()};
        root = last = new_node();
    }
};

using vertex_t = SuffixAutomaton::vertex_t;

struct Fenwick2D {
    unordered_map<int64_t, int> tree;
    int n;

    inline int64_t upair(int x, int y) {
        return (((int64_t) x) << 30) ^ y;
    }

    void _update(int x, int y, int value) {
        for (int cx = x; cx <= n; cx += cx & -cx) {
            for (int cy = y; cy <= n; cy += cy & -cy) {
                tree[upair(cx, cy)] += value;
            }
        }
    }
    
    void update(int x, int y, int value) {
        // cerr << x << ' ' << y << ' ' << value << endl;
        _update(x + 1, y + 1, value);
    }

    inline int64_t get(int x, int y) {
        int64_t res = 0;
        for (int cx = x; cx > 0; cx -= cx & -cx) {
            for (int cy = y; cy > 0; cy -= cy & -cy) {
                res += tree[upair(cx, cy)];
            }
        }
        return res;
    }

    Fenwick2D(int n_) : n(n_) {

    }
};

int main() {
#ifdef USE_FAST_IO
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif
    int n, m; cin >> n >> m;
    SuffixAutomaton sa = SuffixAutomaton();
    string t; cin >> t;
    vector<int> t_prefixes;
    // sa.last = sa.root;
    // for (char c : t) {
    //     sa.add_char(c);
    //     t_prefixes.push_back(sa.last);
    // }
    
    for (int i = 0; i < n; i++) {
        string s; cin >> s;
        sa.last = sa.root;
        for (char c : s) {
            sa.add_char(c);
        }
        sa.nodes[sa.last].terminal_cnt++;
    }
    n = t.length();

    Fenwick2D tree(n + 228);

    vector<vector<int>> pre_answers(n, vector<int>(n));
    int ptr = 0;
    vertex_t cur = sa.root;
    for (int i = 0; i < n; i++) {
        ptr = max(i, ptr);
        while (ptr < n && sa.nodes[cur].go[t[ptr]] != NULL_NODE) {
            // cerr << "GO " << i << ' ' << ptr << endl;
            cur = sa.nodes[cur].go[t[ptr]];
            ptr++;
            vertex_t z = cur;
            if (sa.nodes[z].length <= ptr - i) {
                tree.update(ptr - sa.nodes[z].length, ptr - 1, sa.nodes[z].terminal_cnt);
            }
            z = sa.nodes[z].link;
            while (z != NULL_NODE) {
                tree.update(ptr - sa.nodes[z].length, ptr - 1, sa.nodes[z].terminal_cnt);
                z = sa.nodes[z].link;
            }
        }
        while (cur != sa.root && sa.get_shortest(cur) >= ptr - i) {
            cur = sa.nodes[cur].link;
        }
    }
    while (m--) {
        int l, r; cin >> l >> r; l--; r--;
        cout << tree.get(n, r + 1) - tree.get(l, r + 1) << ' ';
    }
    cout << endl;
}

