#define _CRT_SECURE_NO_WARNINGS

#include <iostream>
#include <iomanip>
#include <algorithm>
#include <vector>
#include <string>
#include <cmath>
#include <set>
#include <map>
#include <queue>
#include <unordered_set>
#include <unordered_map>

using namespace std;
typedef long long ll;
typedef long double ld;
//random_device rd;
//mt19937 gen(rd);
const int SZ = 1e5 + 1, inf = 2e9;
const long long INF = 2e18;

struct Pt {
    ld x, y;
    Pt(ld x = 0, ld y = 0) : x(x), y(y) {}
};

Pt operator+(Pt a, Pt b) {
    return Pt(a.x + b.x, a.y + b.y);
}
Pt operator-(Pt a, Pt b) {
    return Pt(a.x - b.x, a.y - b.y);
}
ld operator*(Pt a, Pt b) {
    return a.x * b.x + a.y * b.y;
}
ld operator%(Pt a, Pt b) {
    return a.x * b.y - a.y * b.x;
}
ld get_ungle(Pt a, Pt b) {
    return atan2(a % b, a * b);
}

struct segment_tree {
    int sz;
    vector<ll> tree;

    segment_tree(int n) {
        int k = 1;
        while (k < n)
            k <<= 1;
        sz = k;
        tree.assign(sz * 2 - 1, 0);
    }

    ll on_segment(int vert, int left, int right, int qleft, int qright) {
        if (qleft <= left && right <= qright) {
            //in
            return tree[vert];
        }
        if (right <= qleft || qright <= left) {
            //not in
            return 0;
        }
        int midle = (left + right) / 2;
        ll a = on_segment(vert * 2 + 1, left, midle, qleft, qright);
        ll b = on_segment(vert * 2 + 2, midle, right, qleft, qright);
        return a + b;
    }

    void add(int vert, int left, int right, int pos) {
        if (right <= pos || pos < left) {
            //not in
            return;
        }
        if (right - left == 1) {
            tree[vert]++;
            return;
        }
        int midle = (left + right) / 2;
        add(vert * 2 + 1, left, midle, pos);
        add(vert * 2 + 2, midle, right, pos);
        tree[vert] = tree[vert * 2 + 1] + tree[vert * 2 + 2];
        return;
    }
};

struct axo_Node {
    bool term;
    int ch, rnk;
    axo_Node* suf;
    axo_Node* pr;
    axo_Node* go[26];
    axo_Node* to[26];
    axo_Node(axo_Node* p = 0, int c = -1, int r = 0) {
        suf = 0;
        pr = p;
        term = 0;
        ch = c;
        rnk = r;
        for (int i = 0; i < 26; i++) {
            go[i] = 0;
            to[i] = 0;
        }
    }
};

struct Axo_Korasik {
    axo_Node* root;
    Axo_Korasik(vector<string> s) {
        root = new axo_Node();
        root->pr = root;
        for (auto& x : s)
            add_build(root, x);
        build(root);
    }

    void add_build(axo_Node* vert, string& s, int pos = 0) {
        if (pos == s.size()) {
            vert->term = 1;
            return;
        }
        if (vert->to[s[pos] - 'a'] == 0) {
            vert->to[s[pos] - 'a'] = new axo_Node(vert, s[pos] - 'a', vert->rnk + 1);
        }
        add_build(vert->to[s[pos] - 'a'], s, pos + 1);
    }

    void build(axo_Node* vert, int rnk = 0) {
        if (rnk <= 1) {
            vert->suf = root;
        }
        else {
            vert->suf = vert->pr->suf->go[vert->ch];
        }
        if (rnk == 0) {
            for (int i = 0; i < 26; i++) {
                vert->go[i] = root;
                if (vert->to[i] != 0)
                    vert->go[i] = vert->to[i];
            }
        }
        else {
            for (int i = 0; i < 26; i++) {
                vert->go[i] = vert->to[i];
                if (vert->to[i] == 0)
                    vert->go[i] = vert->suf->go[i];
            }
        }
        for (int i = 0; i < 26; i++) {
            if (vert->to[i] != 0)
                build(vert->to[i], rnk + 1);
        }
    }
};

void get_ans(axo_Node* vert, segment_tree& tr, Axo_Korasik& a, int pos) {
    axo_Node* nd = vert;
    while (nd != a.root) {
        //
        if (nd->term)
            tr.add(0, 0, tr.sz, pos - nd->rnk + 1);
        nd = nd->suf;
    }
    return;
}

int main()
{
#ifdef _DEBUG
    freopen("input.txt", "r", stdin);
#endif
    //ios_base::sync_with_stdio(false);
    //cin.tie(0); cout.tie(0);
    int n, m;
    string t;
    cin >> n >> m;
    cin >> t;
    vector<string> str(n);
    for (int i = 0; i < n; i++)
        cin >> str[i];
    vector<vector<pair<int, int>>> quest(t.size());
    vector<int> ans(m);
    for (int i = 0; i < m; i++) {
        int l, r;
        cin >> l >> r;
        l--; r--;
        quest[r].push_back({ l, i });
    }
    Axo_Korasik axo = Axo_Korasik(str);
    segment_tree tr = segment_tree(t.size());
    axo_Node* nd = axo.root;
    for (int i = 0; i < t.size(); i++) {
        nd = nd->go[t[i] - 'a'];
        get_ans(nd, tr, axo, i);
        for (auto p : quest[i])
            ans[p.second] = tr.on_segment(0, 0, tr.sz, p.first, i + 1);
    }

    for (auto x : ans)
        cout << x << ' ';
}