﻿#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>

#include <algorithm>
#include <cmath>

#include <chrono>
#include <random>

#include <unordered_map>
#include <unordered_set>
#include <functional>
#include <vector>
#include <string>
#include <deque>
#include <queue>
#include <stack>
#include <array>
#include <list>
#include <set>
#include <map>

using namespace std;
using namespace chrono;


using ll = long long;
using ull = unsigned long long;
using ld = long double;

constexpr ll INF = static_cast<ll>(1e9 + 10);
constexpr ll MOD = static_cast<ll>(1e9 + 7);
constexpr ld EPS = 1e-9L;


#ifndef MSC_VER
ull __lg(ull x) {
	ull res = 0;
	for (; x > 1; x /= 2)
		++res;
	return res;
}
#endif


mt19937 rnd(0);


void solve(istream& cin, ostream& cout);


int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);


	ll _ = 1;
	// cin >> _;
	while (_--)
		solve(cin, cout);
}


class AhoKorasik {
public:
	struct node {
		array<node*, 26> childs;
		node* suf_pred = nullptr;
		node* pred = nullptr;
		node* suff_term = nullptr;
		bool term = false;
		char c = 0;
		size_t len = 0;


		node() {
			childs.fill(nullptr);
		}

		node(const char c, const size_t len, node* pred) :c(c), len(len), pred(pred) {
			childs.fill(nullptr);
		}


		void addString(const string& s, const size_t i) {
			if (s.size() == i) {
				term = true;
				return;
			}
			if (childs[s[i] - 'a'] == nullptr)
				childs[s[i] - 'a'] = new node(s[i], len + 1, this);
			childs[s[i] - 'a']->addString(s, i + 1);
		}


		node* next(const char c) const {
			return childs[c - 'a'];
		}

		const ll cntTerm(const size_t max_len) const {
			ll res = 0;
			if (term && len <= max_len)
				++res;
			for (node* buf = this->suff_term; buf != nullptr; buf = buf->suff_term) {
				if (buf->term && buf->len <= max_len)
					++res;
			}
			return res;
		}
	} root;


	AhoKorasik() = default;


	void addString(const string& s) {
		root.addString(s, 0);
	}

	void init() {
		root.suf_pred = &root;
		queue<node*> q;
		for (auto& t : root.childs) {
			if (t == nullptr)
				t = &root;
			else
				q.push(t);
		}

		while (!q.empty()) {
			node* v = q.front();
			q.pop();

			v->suf_pred = v->pred->suf_pred->childs[v->c - 'a'];
			if (v->suf_pred == v)
				v->suf_pred = &root;
			if (v->suf_pred->term)
				v->suff_term = v->suf_pred;
			else
				v->suff_term = v->suf_pred->suff_term;

			for (ll i = 0; i < v->childs.size(); ++i) {
				if (v->childs[i] == nullptr)
					v->childs[i] = v->suf_pred->childs[i];
				else
					q.push(v->childs[i]);
			}
		}
	}

	node* getRoot() {
		return &root;
	}
};


class SegTree {
private:
	vector<ll> v;
	size_t _size = 0;


	void write(const ll i, const ll first, const ll last, const ll L, const ll R) {
		if (first >= last)
			return;
		if (L <= first && last <= R) {
			++v[i];
			return;
		}
		if (first < R && L < last) {
			ll middle = (first + last) / 2;
			write(i * 2, first, middle, L, R);
			write(i * 2 + 1, middle, last, L, R);
		}
	}

public:
	SegTree() = default;
	SegTree(const size_t size) {
		for (_size = 1; _size < size;)
			_size *= 2;
		v.resize(_size * 2);
	}


	ll operator()(ll iter) const {
		ll res = 0;
		iter += _size;
		res += v[iter];
		for (iter /= 2; iter > 0; iter /= 2)
			res += v[iter];
		return res;
	}


	void add(const ll first, const ll last) {
		write(1, 0, _size, first, last);
	}
};


void solve(istream& cin, ostream& cout) {
	ll n, m;
	cin >> n >> m;
	string s, buf;
	cin >> s;

	AhoKorasik riba;
	for (ll i = 0; i < n; ++i) {
		cin >> buf;
		riba.addString(buf);
	}
	riba.init();


	vector<ll> res(m);
	vector<tuple<ll, ll, ll>> zp(m);
	for (ll i = 0; i < m; ++i) {
		auto& [r, l, id] = zp[i];
		cin >> l >> r;
		--l;
		id = i;
	}
	sort(zp.begin(), zp.end());

	
	SegTree seg_tree(n);
	auto root = riba.getRoot();

	ll i = 0;
	for (auto& [r, l, id] : zp) {
		for (; i < r; ++i) {
			root = root->next(s[i]);
			for (auto buf = root; buf != nullptr; buf = buf->suff_term) {
				if (buf->term)
					seg_tree.add(0, i + 2 - (ll)buf->len);
			}
		}

		res[id] = seg_tree(l);
	}


	for (auto& t : res)
		cout << t << ' ';
}