#pragma GCC optimize("O3")
#pragma GCC optimize("unroll-loops")
#pragma GCC optimize("inline")
#pragma GCC target("avx,avx2,bmi,bmi2,popcnt,clzero")

#include <algorithm>
#include <array>
#include <cstdio>
#include <string>
#include <vector>

using namespace std;

namespace Aho {
    struct Node {
        array<int, 26> go;
        int suf;
        int p;
        int c;
        int cntterm;
        int len;
        int nextterm;
        Node() {
            suf = 0;
            p = -1;
            c = -1;
            go.fill(-1);
            cntterm = 0;
            len = 0;
            nextterm = 0;
        }
    };

    vector<Node> trie;

    void init() {
        trie.emplace_back();
        trie[0].suf = -1;
        trie[0].nextterm = -1;
    }

    void add(const string& s) {
        int cv = 0;
        for (auto ch : s) {
            int el = ch - 'a';
            if (trie[cv].go[el] == -1) {
                trie[cv].go[el] = (int)trie.size();
                trie.emplace_back();
                trie.back().p = cv;
                trie.back().c = el;
                trie.back().len = trie[cv].len + 1;
            }
            cv = trie[cv].go[el];
        }
        trie[cv].cntterm++;
    }

    void bfs() {
        vector<int> q;
        q.push_back(0);
        for (int iter = 0; iter < (int)q.size(); iter++) {
            int cv = q[iter];
            for (int i = 0; i < 26; i++) {
                if (trie[cv].go[i] != -1) {
                    q.push_back(trie[cv].go[i]);
                }
            }
            if (cv != 0) {
                int cs = trie[trie[cv].p].suf;
                if (cs == -1) {
                    trie[cv].suf = 0;
                } else {
                    trie[cv].suf = trie[cs].go[trie[cv].c];
                }
                for (int i = 0; i < 26; i++) {
                    if (trie[cv].go[i] == -1) {
                        trie[cv].go[i] = trie[trie[cv].suf].go[i];
                    }
                }
                // trie[cv].cntterm += trie[trie[cv].suf].cntterm;
                trie[cv].nextterm = trie[trie[cv].suf].nextterm;
                if (trie[trie[cv].suf].cntterm > 0) {
                    trie[cv].nextterm = trie[cv].suf;
                }
            } else {
                for (int i = 0; i < 26; i++) {
                    if (trie[0].go[i] == -1) {
                        trie[0].go[i] = 0;
                    }
                }
            }
        }
        q.clear();
    }

    int curv = 0;

    void getch(char c) {
        curv = trie[curv].go[c - 'a'];
    }

    int countterm() {
        return trie[curv].cntterm;
    }
} // namespace Aho

struct Ev {
    int x;
    int y1;
    int y2;
    int type;
    int cnt;
    Ev() {
        x = -1;
        y1 = y2 = -1;
        type = -1;
        cnt = 0;
    }
    Ev(int _x, int _y1, int _y2, int _type, int _cnt) {
        x = _x;
        y1 = _y1;
        y2 = _y2;
        type = _type;
        cnt = _cnt;
    }
};

constexpr int MAXK = 5001000;
int f[MAXK];

void add(int i, int x) {
    while (i < MAXK) {
        f[i] += x;
        i |= (i + 1);
    }
}

int getpref(int i) {
    int res = 0;
    while (i >= 0) {
        res += f[i];
        i = (i & (i + 1)) - 1;
    }
    return res;
}

int getsum(int l, int r) {
    return getpref(r - 1) - getpref(l - 1);
}

int main() {
    int n, m;
    scanf("%d %d\n", &n, &m);
    char* tc = new char[5000010];
    fgets(tc, 5000010, stdin);
    string t(tc);
    delete[] tc;
    t.pop_back();
    Aho::init();
    char* cts = new char[1000010];
    for (int i = 0; i < n; i++) {
        fgets(cts, 1000010, stdin);
        string cur(cts);
        cur.pop_back();
        Aho::add(cur);
    }
    delete[] cts;

    Aho::bfs();

    // vector<pair<pair<int, int>, int>> dots;
    vector<Ev> evs;

    for (int i = 0; i < (int)t.size(); i++) {
        Aho::getch(t[i]);
        int cv = Aho::curv;
        while (cv != -1) {
            if (Aho::trie[cv].cntterm != 0) {
                int l = i - Aho::trie[cv].len + 1;
                // dots.emplace_back(make_pair(make_pair(l, i), Aho::trie[cv].cntterm));
                evs.emplace_back(l, i, i, 1, Aho::trie[cv].cntterm);
            }
            cv = Aho::trie[cv].nextterm;
        }
    }

    // vector<pair<int, int>> qrs(m);

    for (int i = 0; i < m; i++) {
        int l, r;
        scanf("%d %d\n", &l, &r);
        l--;
        r--;
        evs.emplace_back(l, 0, r + 1, 0, i);
        evs.emplace_back((int)t.size() + 1, 0, r + 1, 2, i);
    }

    // for (auto [dot, cnt] : dots) {
    //     auto [x, y] = dot;
    //     evs.emplace_back(x, y, y, 1, cnt);
    // }

    sort(evs.begin(), evs.end(), [](const Ev& a, const Ev& b) {
        if (a.x != b.x) {
            return a.x < b.x;
        }
        return a.type < b.type;
    });

    vector<int> anss(m);

    for (int i = 0; i < (int)evs.size(); i++) {
        if (evs[i].type == 1) {
            add(evs[i].y1, evs[i].cnt);
        } else if (evs[i].type == 0) {
            anss[evs[i].cnt] -= getsum(evs[i].y1, evs[i].y2);
        } else {
            anss[evs[i].cnt] += getsum(evs[i].y1, evs[i].y2);
        }
    }
    for (int i = 0; i < m; i++) {
        printf("%d ", anss[i]);
    }
    printf("\n");
    return 0;
}
