#include <iostream>
#include <map>
#include <vector>
#include <set>
#include <random>
#include <bitset>
#include <algorithm>
#include <deque>
#include <queue>

using namespace std;

mt19937 rn;
vector<int> fen;

void add(int i, int x) {
    for (; i < fen.size(); i = i | (i + 1)) {
        fen[i] = max(x, fen[i]);
    }
}

int get(int r) {
    int res = 0;
    for (; r >= 0; r = (r & (r + 1)) - 1) {
        res = max(res, fen[r]);
    }
    return res;
}

struct line {
    int a, b, c;
};

struct point {
    int x, y;

    point operator-(point a) {
        return point{x - a.x, y - a.y};
    }

    int operator*(point a) {
        return x * a.y - y - a.x;
    }

    int operator%(point a) {
        return x * a.x + y * a.y;
    }

    point operator-(int d) {
        return point{x - d, y - d};
    }

    point operator+(int d) {
        return point{x + d, y + d};
    }
};

int lcp(vector<int> a, vector<int> b) {
    vector<vector<int>> dp(a.size() + 1, vector<int>(b.size() + 1));
    int n = a.size();
    int m = b.size();
    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < m; ++j) {
            if (a[i] == b[j]) {
                dp[i + 1][j + 1] = dp[i][j] + 1;
            } else {
                dp[i + 1][j + 1] = max(dp[i][j + 1], dp[i + 1][j]);
            }
        }
    }
    return dp[n][m];
}

#define int __int128

int qq = 179;

int get_hash(string s) {
    int h = 0;
    for (auto i: s) {
        h = h * qq + i - 'a' + 1;
    }
    return h;
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(NULL);
    long long n, m;
    cin >> n >> m;
    string t;
    cin >> t;
    set<int> c;
    vector<int> hashs(n);
    vector<int> ln(n);
    for (int i = 0; i < n; ++i) {
        string s;
        cin >> s;
        int h = get_hash(s);
        c.insert(h);
        hashs[i] = h;
        ln[i] = s.size();
    }
    if (n > 100) {
        vector<vector<long long>> dp(t.size(), vector<long long>(t.size()));
        for (int i = 0; i < t.size(); ++i) {
            int h = 0;
            for (int j = i; j < t.size(); ++j) {
                h = h * qq + t[j] - 'a' + 1;
                dp[i][j] = c.count(h);
            }
        }
        vector<vector<long long>> ans(t.size(), vector<long long>(t.size()));
        for (int i = 0; i < t.size(); ++i) {
            ans[i][i] = dp[i][i];
        }
        for (int ln = 1; ln < t.size(); ++ln) {
            for (int i = 0; i < t.size() - ln; ++i) {
                ans[i][i + ln] = ans[i + 1][i + ln] + ans[i][i + ln - 1] + dp[i][i + ln];
                if (ln > 1) {
                    ans[i][i + ln] -= ans[i + 1][i + ln - 1];
                }
            }
        }
        for (int i = 0; i < m; ++i) {
            long long l, r;
            cin >> l >> r;
            --l;
            --r;
            cout << ans[l][r] << " ";
        }
    } else {
        vector<int> hash(t.size() + 1);
        vector<int> step(t.size() + 1, 1);
        for (int i = 0; i < t.size(); ++i) {
            hash[i + 1] = hash[i] * qq + t[i] - 'a' + 1;
            step[i + 1] = step[i] * qq;
        }
        vector<vector<int>> ind(n);
        for (int i = 0; i < n; ++i) {
            for (int j = ln[i]; j <= t.size(); ++j) {
                if (hashs[i] == hash[j] - hash[j - ln[i]] * step[ln[i]]) {
                    ind[i].push_back(j - ln[i]);
                }
            }
        }
        for (int i = 0; i < m; ++i) {
            long long l, r;
            cin >> l >> r;
            --l;
            int ans = 0;
            for (int j = 0; j < n; ++j) {
                auto f = lower_bound(ind[j].begin(), ind[j].end(), l);
                auto s = upper_bound(ind[j].begin(), ind[j].end(), r - ln[j]);
                int res = f - s;
                ans += max(res, (int) 0);
            }
            cout << (long long) ans << " ";
        }
    }
}