#pragma GCC optimize("O3")

#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <cmath>
#include <cassert>
#include <queue>
#include <set>
#include <map>
#include <unordered_set>
#include <unordered_map>
#include <random>
#include <iomanip>
#include <fstream>

using namespace std;

#ifdef LOCAL
ifstream input("input");
#define cin input
#define debug(x) cerr << #x << "=" << (x) << endl
#else
#define debug(x) ;
#endif

#define endl '\n'
#define int long long
#define float long double

random_device rd;
#ifndef int
mt19937 rng(rd());
#else
mt19937_64 rng(rd());
#endif

template<class T> int ckmin(T &a, const T &b) {
	if (b < a) {
		a = b;
		return 1;
	}
	return 0;
}

template<class T> int ckmax(T &a, const T &b) {
	if (b > a) {
		a = b;
		return 1;
	}
	return 0;
}

int mpow(int a, int n, int mod) {
	int ans = 1;
	while (n) {
		if (n & 1) {
			ans *= a;
			ans %= mod;
		}
		a *= a;
		a %= mod;
		n >>= 1;
	}
	return ans;
}

struct Node {
	int cnt = 0;
	int go[26];
	int suf = -1;
	int dp[26];
	int h = 0;
};

vector<Node> vns;

int based() {
	vns.emplace_back();
	for (int i = 0; i < 26; ++i) {
		vns.back().go[i] = -1;
		vns.back().dp[i] = -1;
	}
	return (int) vns.size() - 1;
}

void print(string s) {
	for (char &c : s) {
		c += 'a';
	}
	debug(s);
}

void vns_insert(const string &s) {
	int u = 0;
	for (const char &c : s) {
		if (vns[u].go[(int) c] == -1) {
			vns[u].go[(int) c] = based();
		}
		u = vns[u].go[(int) c];
	}
	vns[u].cnt = 1;
}

void build_aho() {
	for (int i = 0; i < 26; ++i) {
		vns[0].dp[i] = vns[0].go[i];
	}
	queue<int> q;
	q.push(0);
	while (!q.empty()) {
		int u = q.front();
		q.pop();
		for (int i = 0; i < 26; ++i) {
			int v = vns[u].go[i];
			if (v == -1) {
				continue;
			}
			if (vns[vns[u].suf].dp[i] == -1) {
				vns[v].suf = 0;
			} else {
				vns[v].suf = vns[vns[u].suf].dp[i];
			}
			for (int j = 0; j < 26; ++j) {
				if (vns[v].go[j] == -1) {
					vns[v].dp[j] = vns[vns[v].suf].dp[j];
				} else {
					vns[v].dp[j] = vns[v].go[j];
				}
			}
			vns[v].h = vns[u].h + 1;
			q.push(v);
		}
	}
}

void solve() {
	vns.clear();
	int n, m;
	cin >> n >> m;
	string t;
	cin >> t;
	for (char &c : t) {
		c -= 'a';
	}
	vector<string> s(n);
	for (int i = 0; i < n; ++i) {
		cin >> s[i];
		for (char &c : s[i]) {
			c -= 'a';
		}
	}
	based();
	for (int i = 0; i < n; ++i) {
		vns_insert(s[i]);
	}
	build_aho();
	int tn = (int) t.size();
	vector<vector<int>> dp(tn, vector<int>(tn, 0));
	for (int i = 0, u = 0; i < tn; ++i) {
		u = vns[u].dp[(int) t[i]];
		for (int v = u; v != 0; v = vns[v].suf) {
			dp[i - vns[v].h + 1][i] += vns[v].cnt;
		}
	}
	for (int i = 0; i < m; ++i) {
		int l, r;
		cin >> l >> r;
		--l;
		--r;
		int ans = 0;
		for (int L = l; L <= r; ++L) {
			for (int R = L; R <= r; ++R) {
				ans += dp[L][R];
			}
		}
		cout << ans << " ";
	}
	cout << endl;
}

int32_t main() {
	int T = 1;
#ifdef LOCAL
	cin >> T;
#else
	ios_base::sync_with_stdio(false);
	cout.tie(nullptr);
	cin.tie(nullptr);
#endif
	for (int i = 1; i <= T; ++i) {
		cerr << "TEST #" << i << endl;
		solve();
	}
}
