#pragma GCC optimize("Ofast")
#pragma GCC target("avx2")

#include <bits/stdc++.h>

#include <ext/pb_ds/assoc_container.hpp>
#include <ext/pb_ds/tree_policy.hpp>

using namespace std;

#define ALL(a) (a).begin(), (a).end()
#define RALL(a) (a).rbegin(), (a).rend()

using i32 = int32_t;
using i64 = int64_t;
using f64 = double;
using f128 = long double;

template <typename K, typename V, typename Cmp = less<K>>
using ordered_map =
    __gnu_pbds::tree<K, V, Cmp, __gnu_pbds::rb_tree_tag,
                     __gnu_pbds::tree_order_statistics_node_update>;

template <typename T, typename Cmp = less<T>>
using ordered_set = ordered_map<T, __gnu_pbds::null_type, Cmp>;

template <typename T1, typename T2>
istream& operator>>(istream& is, pair<T1, T2>& p) {
  return is >> p.first >> p.second;
}
template <typename T1, typename T2>
ostream& operator<<(ostream& os, const pair<T1, T2>& p) {
  return os << p.first << " " << p.second;
}
template <typename T>
istream& operator>>(istream& is, vector<T>& v) {
  for (T& e : v) is >> e;
  return is;
}
template <typename T>
ostream& operator<<(ostream& os, const vector<T>& v) {
  for (const T& e : v) os << e << " ";
  return os;
}
template <typename T>
void chmax(T& a, const T& b) {
  if (a < b) a = b;
}
template <typename T>
void chmin(T& a, const T& b) {
  if (a > b) a = b;
}

void solve();

signed main() {
  ios_base::sync_with_stdio(false);
  cin.tie(nullptr);
  solve();
  return 0;
}

struct aho_node {
  char c = -1;
  array<i32, 26> next = {0};
  i32 p = 0, suff = 0, suff_end = 0, i = -1;
  bool end = false;
};

pair<vector<aho_node>, vector<i32>> make_corasick(const vector<string>& words) {
  vector<aho_node> corasick;
  vector<i32> ind(size(words));
  corasick.push_back({});
  // construct trie
  for (int i = 0; i < ssize(words); ++i) {
    const string& s = words[i];
    i32 cur = 0;
    for (char c : s) {
      if (corasick[cur].next[c - 'a'] == 0) {
        corasick[cur].next[c - 'a'] = (i32)ssize(corasick);
        corasick.push_back({.c = (char)(c - 'a'), .p = cur});
      }
      cur = corasick[cur].next[c - 'a'];
    }
    corasick[cur].end = true;
    corasick[cur].i = i;
    ind[i] = cur;
  }
  // suff links
  queue<i32> q;
  for (int i = 0; i < 26; ++i) {
    i32 u;
    if (u = corasick[0].next[i]; u == 0) continue;
    for (int j = 0; j < 26; ++j) {
      i32 v;
      if (v = corasick[u].next[j]; v == 0) continue;
      q.push(v);
    }
  }
  while (!q.empty()) {
    i32 u = q.front();
    q.pop();
    const char c = corasick[u].c;
    i32 p = corasick[u].p;
    while (p != 0 && corasick[corasick[p].suff].next[c] == 0) {
      p = corasick[p].suff;
    }
    corasick[u].suff = corasick[corasick[p].suff].next[c];
    corasick[u].suff_end = corasick[corasick[u].suff].suff_end;
    if (corasick[corasick[u].suff].end)
      corasick[u].suff_end = corasick[u].suff;
    for (int i = 0; i < 26; ++i) {
      i32 v;
      if (v = corasick[u].next[i]; v == 0) continue;
      q.push(v);
    }
  }
  return {corasick, ind};
}

vector<i64> count_prefix_ends(const string& text, const vector<string>& words) {
  i32 n = ssize(words);
  auto [cor, ind] = make_corasick(words);
  map<i32, i32> r_ind;
  for (int i = 0; i < ssize(words); ++i) {
    r_ind[ind[i]] = i;
  }
  vector<i64> prefix_ends(ssize(text));
  vector<i32> p(size(words)), depth(size(words));
  for (int i = 0; i < n; ++i) {
    p[i] = -1;
    if (i32 x = cor[ind[i]].suff_end; x != 0) {
      p[i] = r_ind[x];
    }
  }
  vector<vector<i32>> g(n);
  for (int i = 0; i < n; ++i) {
    if (p[i] != -1) g[p[i]].push_back(i);
  }
  auto dfs = [&](i32 u, i32 d, auto&& dfs) -> void {
    depth[u] = d;
    for (i32 v : g[u]) dfs(v, d + 1, dfs);
  };
  for (int i = 0; i < n; ++i) {
    if (p[i] == -1) dfs(i, 1, dfs);
  }

  i32 cur = 0;
  for (int i = 0; i < ssize(text); ++i) {
    char c = (char)(text[i] - 'a');
    while (cur != 0 && cor[cur].next[c] == 0) {
      cur = cor[cur].suff;
    }
    cur = cor[cur].next[c];
    i32 u;
    if (cor[cur].end)
      u = cur;
    else
      u = cor[cur].suff_end;
    if (u != 0) {
      prefix_ends[i] += depth[r_ind[u]];
    }
  }
  for (int i = 1; i < ssize(text); ++i) {
    prefix_ends[i] += prefix_ends[i - 1];
  }
  return prefix_ends;
}

void solve() {
  i32 n, m;
  string text;
  cin >> n >> m >> text;
  vector<string> words(n);
  cin >> words;

  auto pref = count_prefix_ends(text, words);
  reverse(ALL(text));
  for (auto& w : words) reverse(ALL(w));
  auto suff = count_prefix_ends(text, words);
  reverse(ALL(suff));
  for (int i = 0; i < ssize(text); ++i) {
    suff[i] = pref.back() - suff[i];
  }
  for (int i = 0; i < m; ++i) {
    i32 l, r;
    cin >> l >> r;
    --l, --r;
    cout << pref[r] - suff[l] << "\n";
  }
}
