#include <iostream>
#include <cmath>
#include <fstream>
#include <algorithm>
#include <vector>
#include <map>
#include <set>
#include <ext/pb_ds/assoc_container.hpp>
#include <numeric>
#include <iomanip>

using namespace __gnu_pbds;
using namespace std;

//#define int long long
#define all(x) (x).begin(), (x).end()
#define sz(x) (int) ((x).size())

template<typename T>
using ordered_set = tree<T, null_type, less<T>, rb_tree_tag, tree_order_statistics_node_update>;

#ifdef LOCAL
#include "algo/debug.h"
#else
#define debug(...) 42
#endif

typedef long long ll;

const int K = 300;
const int P = 239;
const int N = 1e6 + 5;
const ll mod = 1000000000000000003;

ll mul(ll a, ll b) {
  return (__int128) a * (__int128) b % mod;
}

ll sum(ll a, ll b) {
  a += b;
  if (a >= mod) {
    a -= mod;
  }
  return a;
}

ll sub(ll a, ll b) {
  a -= b;
  if (a < 0) {
    a += mod;
  }
  return a;
}

void solve() {
  int n, m;
  string t;
  cin >> n >> m >> t;

  vector<ll> deg(N);
  deg[0] = 1;
  for (int i = 1; i < N; i++) {
    deg[i] = mul(deg[i - 1], P);
  }

  int st = sz(t);
  vector<ll> hash(st + 1);
  for (int i = 0; i < st; i++) {
    hash[i + 1] = sum(mul(hash[i], P), t[i]);
  }

  gp_hash_table<ll, char> hashes;
  vector<pair<ll, int>> heavy; // [hash, len]
  for (int i = 0; i < n; i++) {
    string s;
    cin >> s;
    if (sz(s) > st) {
      continue;
    }
    ll hash = 0;
    for (int j = 0; j < sz(s); j++) {
      hash = sum(mul(hash, P), s[j]);
    }
    if (sz(s) <= K) { // light
      hashes[hash] = 1;
    } else { // heavy
      heavy.emplace_back(hash, sz(s));
    }
  }

  auto get_hash = [&] (int l, int r) {
    return sub(hash[r + 1], mul(hash[l], deg[r - l + 1]));
  };

  vector<vector<int>> pref(st, vector<int> (K + 1));
  for (int i = 0; i < st; i++) {
    ll cur_hash = 0;
    for (int len = 1; i + len <= st && len <= K; len++) {
      cur_hash = sum(mul(cur_hash, P), t[i + len - 1]);
      pref[i][len] = (hashes.find(cur_hash) != hashes.end());
    }
    for (int len = 1; len <= K; len++) {
      pref[i][len] += pref[i][len - 1];
    }
    if (i > 0) {
      for (int len = 1; len <= K; len++) {
        pref[i][len] += pref[i - 1][len];
      }
    }
  }

  vector<int> l(m), r(m);
  vector<ll> ans(m);
  for (int w = 0; w < m; w++) {
    cin >> l[w] >> r[w];
    --l[w], --r[w];
  }

  for (int i = 0; i < m; i++) {
    if (r[i] - K + 1 >= l[i]) {
      ans[i] += pref[r[i] - K + 1][K];
      if (l[i] > 0) {
        ans[i] -= pref[l[i] - 1][K];
      }
    }
    for (int j = max(l[i], r[i] - K + 2); j <= r[i]; j++) {
      ans[i] += pref[j][r[i] - j + 1] - (j > 0 ? pref[j - 1][r[i] - j + 1] : 0);
    }
  }

  vector<int> kok(st + 1);
  for (auto &[hash, len] : heavy) {
    for (int j = 0; j + len <= st; j++) {
      kok[j + 1] = kok[j] + (get_hash(j, j + len - 1) == hash);
    }
    for (int i = 0; i < m; i++) {
      int to = r[i] - len + 1;
      if (l[i] <= to) {
        ans[i] += kok[to + 1] - kok[l[i]];
      }
    }
  }

  for (auto &x : ans) {
    cout << x << ' ';
  }
  cout << '\n';
}

int32_t main() {
#ifdef LOCAL
  freopen("input.txt", "r", stdin);
#endif
  ios_base::sync_with_stdio(false);
  cin.tie(nullptr);
  solve();
  return 0;
}