use std::io::{self, prelude::*};

fn dfs(g: &[Vec<(usize, i32)>], xor: &mut [i32], odd: &mut [bool], used: &mut [bool], u: usize) {
    used[u] = true;
    for &(v, w) in &g[u] {
        if !used[v] { dfs(g, xor, odd, used, v); }
        odd[u] ^= odd[v];
        xor[u] ^= xor[v];
        if odd[v] {
            xor[u] ^= w;
        }
    }
}

fn calc_two_xors(g: &[Vec<(usize, i32)>], two_xors: &mut [Vec<i32>], used: &mut [bool], u: usize) {
    used[u] = true;
    for &(v, w) in &g[u] {
        if !used[v] {
            calc_two_xors(g, two_xors, used, v);
            for x in two_xors[v].iter().map(|&x| x ^ w).collect::<Vec<_>>() {
                two_xors[u].push(x);
            }
            two_xors[u].sort();
            two_xors[u].dedup();
            while two_xors[u].len() > 2 { two_xors[u].pop(); }
        }
    }
}

fn check_graph(g: &[Vec<(usize, i32)>], two_xors: &[Vec<i32>], xor: &mut [i32], used: &mut [u8], u: usize) -> bool {
    used[u] = 1;
    for &(v, w) in &g[u] {

        if used[v] == 1 {
            let cycle = xor[v] ^ xor[u] ^ w;
            if two_xors[v].iter().any(|&x| x ^ cycle != 0) {
                return true;
            }
        }

        if used[v] == 0 {
            xor[v] = xor[u] ^ w;
            if check_graph(g, two_xors, xor, used, v) {
                return true;
            }
        }
    }
    used[u] = 2;
    false
}

fn solve<R: BufRead, W: Write>(scan: &mut Scanner<R>, w: &mut W) -> io::Result<()> {
    let (n, m) = (scan.token(), scan.token());
    let mut g = vec![vec![]; n];
    for _ in 0..m {
        let (u, v, w) = (scan.token::<usize>() - 1, scan.token::<usize>() - 1, scan.token::<i32>());
        g[u].push((v, w));
    }

    let (a, b) = (scan.token::<usize>() - 1, scan.token::<usize>() - 1);

    let mut xor = vec![0; n];
    let mut odd = vec![false; n];
    odd[b] = true;
    dfs(&g, &mut xor, &mut odd, &mut vec![false; n], a);

    let mut two_xors = vec![vec![]; n];
    two_xors[b].push(0);
    calc_two_xors(&g, &mut two_xors, &mut vec![false; n], a);

    if check_graph(&g,  &two_xors, &mut vec![0; n], &mut vec![0; n], a) {
        return writeln!(w, "-1");
    }

    writeln!(w, "{}", xor[a])
}

fn main() -> io::Result<()> {
    let mut scan = Scanner::new(io::BufReader::new(io::stdin().lock()));
    let mut w = io::BufWriter::new(io::stdout().lock());
    (0..scan.token()).try_for_each(|_| solve(&mut scan, &mut w))
}

struct Scanner<R> {
    reader: R,
    buf: Vec<u8>,
    buf_iter: std::str::SplitAsciiWhitespace<'static>,
}

impl<R: BufRead> Scanner<R> {
    fn new(reader: R) -> Self {
        Self {
            reader,
            buf: vec![],
            buf_iter: "".split_ascii_whitespace(),
        }
    }

    fn token<T: std::str::FromStr>(&mut self) -> T {
        loop {
            if let Some(token) = self.buf_iter.next() {
                return token.parse().ok().expect("Failed parse");
            }
            self.buf.clear();
            self.reader
                .read_until(b'\n', &mut self.buf)
                .expect("Failed read");
            self.buf_iter = unsafe {
                let slice = std::str::from_utf8_unchecked(&self.buf);
                std::mem::transmute(slice.split_ascii_whitespace())
            };
        }
    }
}
