#include <iostream>
#include <iomanip>
#include <algorithm>
#include <vector>

using namespace std;

const int neutral = (1ll << 31) - 1;

vector<vector<pair<int, int>>> g;
vector<bool> used;

vector<int> topsort;

void dfs(int v) {
    used[v] = true;
    for (auto [to, w] : g[v]) {
        if (!used[to]) {
            dfs(to);
        }
    }
    topsort.push_back(v);
}

void end_solve() {
    g.clear();
    used.clear();
    topsort.clear();
}

void solve() {
    int n, m;
    cin >> n >> m;
    g.resize(n);
    vector<vector<pair<int, int>>> rg(n);
    for (int i = 0; i < m; ++i) {
        int v, u, w;
        cin >> v >> u >> w;
        g[v - 1].emplace_back(u - 1, w);
        rg[u - 1].emplace_back(v - 1, w);
    }
    int start, end;
    cin >> start >> end;
    start--;
    end--;

    used.assign(n, false);
    dfs(start);
    reverse(topsort.begin(), topsort.end());
    vector<int> pos(n);
    for (int i = 0; i < (int) topsort.size(); ++i) {
        pos[topsort[i]] = i;
    }

    //let's count the answer
    vector<int> path(n); //xor всех путей из a в i
    vector<int> cnt(n); //количество всех путей из a в i
    cnt[start] = 1;
    for (int i = 0; i <= pos[end]; ++i) {
        int v = topsort[i];
        for (auto [to, w] : g[v]) {
            if (pos[to] > pos[v]) {
                path[to] ^= path[v] ^ (w * (cnt[v] % 2));
                cnt[to] += cnt[v];
            }
        }
    }

    vector<int> pref_or(n, 0);
    vector<int> pref_and(n, neutral);
    pref_and[start] = 0;
    for (auto v : topsort) {
        for (auto [to, w] : g[v]) {
            if (pos[to] > pos[v]) {
                pref_or[to] |= pref_and[v] ^ w;
                pref_and[to] &= pref_or[v] ^ w;
            }
        }
    }

    vector<int> suf_or(n, 0);
    vector<int> suf_and(n, neutral);
    suf_and[end] = 0;
    for (int j = pos[end]; j >= 0; --j) {
        int v = topsort[j];
        for (auto [to, w] : rg[v]) {
            if (pos[to] < pos[v]) {
                suf_or[to] |= suf_and[v] ^ w;
                suf_and[to] &= suf_or[v] ^ w;
            }
        }
    }

    for (auto v : topsort) {
        for (auto [to, w] : g[v]) {
            if (pos[to] <= pos[v] && pos[to] <= pos[end]) {
                if ((path[v] ^ path[to] ^ w) > 0) {
                    cout << -1 << '\n';
                    end_solve();
                    return;
                } else {
                    if ((pref_or[to] | suf_or[v])) {
                        cout << -1 << '\n';
                        end_solve();
                        return;
                    }
                }
            }
        }
    }

    cout << path[end] << '\n';
    end_solve();
}

signed main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
#endif
    int t;
    cin >> t;
    for (int i = 0; i < t; ++i) {
        solve();
    }
}
