#include <iostream>
#include <vector>
#include <algorithm>

#if !defined LOCAL && !defined DEBUG
#define USE_FAST_IO
#endif

#ifdef USE_FAST_IO
#define endl '\n'
#endif

using namespace std;

struct Edge {
    int to, weight;
    explicit Edge() = default;
    Edge(int u, int w) : to(u), weight(w) {

    }
};

using Graph = vector<vector<Edge>>;

// bool has_cycle(int bit, const Graph &graph, int v, vector<int> &used, int p = 0) {
//     used[v] = 2 ^ p;
//     for (auto [u, w] : graph[v]) {
//         if ((w >> bit) & 1) {
//             if (used[u] >= 2 && used[u] ^ used[v] == 0) {
//                 return true;
//             }
//             if (used[u] == 0) {
//                 if (has_cycle(bit, graph, u, used, p ^ 1)) {
//                     return true;
//                 }
//             }
//         }
//     }
//     used[v] = 1;
//     return false;
// }

void fill_reachable(const Graph &graph, int v, vector<int> &used) {
    used[v] = 1;
    for (auto [u, w] : graph[v]) {
        if (!used[u]) {
            fill_reachable(graph, u, used);
        }
    }
}

void fill_topsort(const Graph &graph, int v, vector<int> &used, vector<int> &order) {
    used[v] = 1;
    for (auto [u, w] : graph[v]) {
        if (!used[u]) {
            fill_topsort(graph, u, used, order);
        }
    }
    order.push_back(v);
}

void fill_comps(const Graph &graph, int v, vector<int> &comps, vector<int> &weights_from_start, int timer, int W = 0) {
    comps[v] = timer;
    weights_from_start[v] = W;
    for (auto [u, w] : graph[v]) {
        if (comps[u] == -1) {
            fill_comps(graph, u, comps, weights_from_start, timer, W ^ w);
        }
    }
}

void solve() {
    int n, m; cin >> n >> m;
    Graph graph(n), rev_graph(n);
    bool loop_corrupt = false;
    bool topsorted = true;
    for (int i = 0; i < m; i++) {
        int v, u, w; cin >> v >> u >> w;
        v--; u--;
        if (v >= u) {
            topsorted = false;
        }
        if (v == u) {
            if (w != 0) {
                loop_corrupt = true;
            } else {
                continue;
            }
        }
        graph[v].emplace_back(u, w);
        rev_graph[u].emplace_back(v, w);
    }
    int a, b; cin >> a >> b; a--; b--;
    if (topsorted) {
        vector<int> sums(n), parity(n);
        vector<bool> rb(n);
        for (int v = n - 1; v >= 0; v--) {
            if (v == b) {
                rb[v] = true;
                parity[v] = 1;
            }
            for (auto [u, w] : graph[v]) {
                if (!rb[u]) continue;
                sums[v] ^= sums[u] ^ (w * parity[u]);
                parity[v] ^= parity[u];
                rb[v] = true;
            }
        }
        cout << sums[a] << endl;
        return;
    }
    vector<int> reachable_a(n, 0), reachable_b(n, 0);
    fill_reachable(graph, a, reachable_a);
    fill_reachable(rev_graph, b, reachable_b);
    vector<bool> deleted(n);
    for (int i = 0; i < n; i++) {
        if (!reachable_a[i] || !reachable_b[i]) {
            deleted[i] = true;
            graph[i].clear();
            rev_graph[i].clear();
        }
    }
    for (vector<Edge> &z : graph) {
        vector<Edge> new_z;
        for (auto [v, w] : z) {
            if (!deleted[v]) {
                new_z.push_back({v, w});
            }
        }
        z = new_z;
    }
    for (vector<Edge> &z : rev_graph) {
        vector<Edge> new_z;
        for (auto [v, w] : z) {
            if (!deleted[v]) {
                new_z.push_back({v, w});
            }
        }
        z = new_z;
    }

    if (loop_corrupt) {
        cout << -1 << endl;
        return;
    }
    vector<int> used(n, 0);
    // for (int bit = 0; bit < 30; bit++) {
    //     used.assign(n, 0);
    //     if (has_cycle(bit, graph, a, used)) {
    //         cout << -1 << endl;
    //         return;
    //     }
    // }
    used.assign(n, 0);
    vector<int> order;
    for (int i = 0; i < n; i++) {
        if (!used[i]) {
            fill_topsort(graph, i, used, order);
        }
    }
    reverse(order.begin(), order.end());
    vector<int> cmps(n, -1);
    vector<int> weights_from_start(n, -1);
    vector<int> starts;
    int timer = 0;
    for (int i : order) {
        if (cmps[i] == -1) {
            starts.push_back(i);
            fill_comps(rev_graph, i, cmps, weights_from_start, timer);
            timer++;
        }
    }
    Graph cond_graph(timer);
    for (int v = 0; v < n; v++) {
        for (auto [u, w] : graph[v]) {
            int a = cmps[v], b = cmps[u];
            if (a != b) {
                int new_w = weights_from_start[v] ^ w ^ weights_from_start[u];
                cond_graph[a].emplace_back(b, new_w);
            }
        }
    }
    vector<int> cond_used(timer, 0);
    vector<int> cond_order;
    for (int i = 0; i < timer; i++) {
        if (!cond_used[i]) {
            fill_topsort(cond_graph, i, cond_used, cond_order);
        }
    }
    vector<int> sums(timer), parity(timer);
    vector<bool> cond_rb(timer);
    for (int v : cond_order) {
        if (v == b) {
            cond_rb[v] = true;
            parity[v] = 1;
        }
        for (auto [u, w] : cond_graph[v]) {
            if (!cond_rb[u]) continue;
            sums[v] ^= sums[u] ^ (w * parity[u]);
            parity[v] ^= parity[u];
            cond_rb[v] = true;
        }
    }
    cout << (sums[cmps[a]] ^ (parity[a] * (weights_from_start[a] ^ weights_from_start[b]))) << endl;
}

int main() {
#ifdef USE_FAST_IO
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif
    int t; cin >> t;
    while (t--) {
        solve();
    }
}
