#include <iostream>
#include <vector>
#include <algorithm>

#if !defined LOCAL && !defined DEBUG
#define USE_FAST_IO
#endif

#ifdef USE_FAST_IO
#define endl '\n'
#endif

using namespace std;

struct Edge {
    int to, weight;
    explicit Edge() = default;
    Edge(int u, int w) : to(u), weight(w) {

    }
};

inline int ith_bit(int n, int bit) {
    return (n >> bit) & 1;
}

using Graph = vector<vector<Edge>>;

// bool has_cycle(int bit, const Graph &graph, int v, vector<int> &used, int p = 0) {
//     used[v] = 2 ^ p;
//     for (auto [u, w] : graph[v]) {
//         if ((w >> bit) & 1) {
//             if (used[u] >= 2 && used[u] ^ used[v] == 0) {
//                 return true;
//             }
//             if (used[u] == 0) {
//                 if (has_cycle(bit, graph, u, used, p ^ 1)) {
//                     return true;
//                 }
//             }
//         }
//     }
//     used[v] = 1;
//     return false;
// }

void fill_reachable(const Graph &graph, int v, vector<int> &used) {
    used[v] = 1;
    for (auto [u, w] : graph[v]) {
        if (!used[u]) {
            fill_reachable(graph, u, used);
        }
    }
}

void fill_topsort(const Graph &graph, int v, vector<int> &used, vector<int> &order) {
    used[v] = 1;
    for (auto [u, w] : graph[v]) {
        if (!used[u]) {
            fill_topsort(graph, u, used, order);
        }
    }
    order.push_back(v);
}

void fill_comps(const Graph &graph, int v, vector<int> &comps, int timer) {
    comps[v] = timer;
    for (auto [u, w] : graph[v]) {
        if (comps[u] != timer) {
            fill_comps(graph, u, comps, timer);
        }
    }
}

void solve() {
    int n, m; cin >> n >> m;
    Graph graph(n), rev_graph(n);
    bool topsorted = true;
    bool is_cycle = n == m;
    int max_bit = 0;
    int sum_weights = 0;
    for (int i = 0; i < m; i++) {
        int v, u, w; cin >> v >> u >> w;
        v--; u--;
        is_cycle = is_cycle && (v == i) && (u == ((i + 1) % n));
        if (w > 0) {
            max_bit = max(max_bit, 32 - __builtin_clz(w));
        }
        if (v >= u) {
            topsorted = false;
        }
        sum_weights ^= w;
        graph[v].emplace_back(u, w);
        rev_graph[u].emplace_back(v, w);
    }
    int a, b; cin >> a >> b; a--; b--;
    if (topsorted) {
        vector<int> sums(n), parity(n);
        vector<bool> rb(n);
        for (int v = n - 1; v >= 0; v--) {
            if (v == b) {
                rb[v] = true;
                parity[v] = 1;
            }
            for (auto [u, w] : graph[v]) {
                if (!rb[u]) continue;
                sums[v] ^= sums[u] ^ (w * parity[u]);
                parity[v] ^= parity[u];
                rb[v] = true;
            }
        }
        cout << sums[a] << endl;
        return;
    }
    if (is_cycle) {
        int w_ab = 0;
        for (int i = a; i != b; i = ((i + 1) % n)) {
            w_ab ^= graph[i][0].weight;
        }
        if (sum_weights != 0 || w_ab != 0) {
            cout << -1 << endl;
        } else {
            cout << 0 << endl;
        }
        return;
    }
    vector<int> reachable_a(n, 0), reachable_b(n, 0);
    fill_reachable(graph, a, reachable_a);
    fill_reachable(rev_graph, b, reachable_b);
    vector<bool> deleted(n);
    for (int i = 0; i < n; i++) {
        if (!reachable_a[i] || !reachable_b[i]) {
            deleted[i] = true;
            graph[i].clear();
            rev_graph[i].clear();
        }
    }
    for (vector<Edge> &z : graph) {
        vector<Edge> new_z;
        for (auto [v, w] : z) {
            if (!deleted[v]) {
                new_z.push_back({v, w});
            }
        }
        z = new_z;
    }
    for (vector<Edge> &z : rev_graph) {
        vector<Edge> new_z;
        for (auto [v, w] : z) {
            if (!deleted[v]) {
                new_z.push_back({v, w});
            }
        }
        z = new_z;
    }

    vector<int> used(n, 0);
    // for (int bit = 0; bit < 30; bit++) {
    //     used.assign(n, 0);
    //     if (has_cycle(bit, graph, a, used)) {
    //         cout << -1 << endl;
    //         return;
    //     }
    // }
    used.assign(n, 0);
    vector<int> order;
    for (int i = 0; i < n; i++) {
        if (!used[i]) {
            fill_topsort(graph, i, used, order);
        }
    }
    reverse(order.begin(), order.end());
    vector<int> cmps(n, -1);
    vector<int> starts;
    int timer = 0;
    for (int i : order) {
        if (cmps[i] == -1) {
            starts.push_back(i);
            fill_comps(rev_graph, i, cmps, timer);
            timer++;
        }
    }
    // for (int bit = 0; bit < max_bit; bit++) {
    //     vector<int> parts(n);
    //     vector<vector<int>> parities_from_start(timer);
    //     for (int start : starts) {
    //         fill_parts(bit, graph, start, parts);
    //     }
    //     for (int v = 0; v < n; v++) {
    //         for (auto [u, w] : graph[v]) {
    //             if (cmps[v] == cmps[u] && parts[v] == parts[u] && ith_bit(w, bit) != 0) {
    //                 cout << -1 << endl;
    //                 return 0;
    //             }
    //             if (cmps[v] == cmps[u] && parts[v] != parts[u]) {
    //                 assert(ith_bit(w, bit) == 1);
    //                 parities_from_start[cmps[v]] ^= 1;
    //             }
    //         }
    //     }
    //     Graph cond_graph(timer);
    //     for (int v = 0; v < n; v++) {
    //         for (auto [u, w] : graph[v]) {
    //             int a = cmps[v], b = cmps[u];
    //             if (a != b) {
    //                 int new_w = (parities_from_start[a] * (parts[v] ^ 1)) ^ 
    //                     ith_bit(w, bit) ^
    //                     (parities_from_start[b] * (parts[u] ^ 1));
    //                 cond_graph[a].emplace_back(b, new_w);
    //             }
    //         }
    //     }

    // }
    // vector<int> cond_used(timer, 0);
    // vector<int> cond_order;
    // for (int i = 0; i < timer; i++) {
    //     if (!cond_used[i]) {
    //         fill_topsort(cond_graph, i, cond_used, cond_order);
    //     }
    // }
    // vector<int> sums(timer), parity(timer);
    // vector<bool> cond_rb(timer);
    // for (int v : cond_order) {
    //     if (v == b) {
    //         cond_rb[v] = true;
    //         parity[v] = 1;
    //     }
    //     for (auto [u, w] : cond_graph[v]) {
    //         if (!cond_rb[u]) continue;
    //         sums[v] ^= sums[u] ^ (w * parity[u]);
    //         parity[v] ^= parity[u];
    //         cond_rb[v] = true;
    //     }
    // }
    // cout << (sums[cmps[a]] ^ (parity[a] * (weights_from_start[a] ^ weights_from_start[b]))) << endl;
}

int main() {
#ifdef USE_FAST_IO
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif
    int t; cin >> t;
    while (t--) {
        solve();
    }
}
