// C++ includes used for precompiling -*- C++ -*-

// Copyright (C) 2003-2022 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 3, or (at your option)
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// Under Section 7 of GPL version 3, you are granted additional
// permissions described in the GCC Runtime Library Exception, version
// 3.1, as published by the Free Software Foundation.

// You should have received a copy of the GNU General Public License and
// a copy of the GCC Runtime Library Exception along with this program;
// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
// <http://www.gnu.org/licenses/>.

/** @file stdc++.h
 *  This is an implementation file for a precompiled header.
 */

 // 17.4.1.2 Headers

 // C
#ifndef _GLIBCXX_NO_ASSERT
#include <cassert>
#endif
#include <cctype>
#include <cerrno>
#include <cfloat>
#include <ciso646>
#include <climits>
#include <clocale>
#include <cmath>
#include <csetjmp>
#include <csignal>
#include <cstdarg>
#include <cstddef>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>
#include <cwchar>
#include <cwctype>

#if __cplusplus >= 201103L
#include <ccomplex>
#include <cfenv>
#include <cinttypes>
#include <cstdalign>
#include <cstdbool>
#include <cstdint>
#include <ctgmath>
#include <cuchar>
#endif

// C++
#include <algorithm>
#include <bitset>
#include <complex>
#include <deque>
#include <exception>
#include <fstream>
#include <functional>
#include <iomanip>
#include <ios>
#include <iosfwd>
#include <iostream>
#include <istream>
#include <iterator>
#include <limits>
#include <list>
#include <locale>
#include <map>
#include <memory>
#include <new>
#include <numeric>
#include <ostream>
#include <queue>
#include <set>
#include <sstream>
#include <stack>
#include <stdexcept>
#include <streambuf>
#include <string>
#include <typeinfo>
#include <utility>
#include <valarray>
#include <vector>

#if __cplusplus >= 201103L
#include <array>
#include <atomic>
#include <chrono>
#include <codecvt>
#include <condition_variable>
#include <forward_list>
#include <future>
#include <initializer_list>
#include <mutex>
#include <random>
#include <ratio>
#include <regex>
#include <scoped_allocator>
#include <system_error>
#include <thread>
#include <tuple>
#include <typeindex>
#include <type_traits>
#include <unordered_map>
#include <unordered_set>
#endif

#if __cplusplus >= 201402L
#include <shared_mutex>
#endif

#if __cplusplus >= 201703L
#include <any>
#include <charconv>
// #include <execution>
#include <filesystem>
#include <optional>
#include <memory_resource>
#include <string_view>
#include <variant>
#endif

#if __cplusplus >= 202002L
#include <barrier>
#include <bit>
#include <compare>
#include <concepts>
#if __cpp_impl_coroutine
# include <coroutine>
#endif
#include <latch>
#include <numbers>
#include <ranges>
#include <span>
#include <stop_token>
#include <semaphore>
#include <source_location>
#include <syncstream>
#include <version>
#endif

#if __cplusplus > 202002L
#include <expected>
#include <spanstream>
#if __has_include(<stacktrace>)
# include <stacktrace>
#endif
#include <stdatomic.h>
#endif

using namespace std;

#ifdef OG_Matveychick1
bool local = 1;
#else
bool local = 0;
#endif

#define for0(i,n) for(int i=0;i<n;i++)
#define for1(i,n) for(int i=1;i<n;i++)
#define fi first
#define se second
#define con continue
#define ret return
#define rfor(i,n) for (int i=n-1;i>=0;i--)
#define be(x) x.begin()
#define en(x) x.end()
#define all(x) be(x),en(x)
#define pb push_back
#define pob pop_back
#define ins insert
#define bs binary_search
#define lb lower_bound
#define ub upper_bound



typedef long long ll;
typedef vector<ll> vll;
typedef set<ll> sll;
typedef string str;
typedef vector<string> vs;
typedef pair<ll, ll> pll;
typedef vector<pll> vpll;
typedef vector<vll> vvll;
typedef long double ld;
typedef vector<vpll> vvpll;


void solve();

ll rnd(ll l, ll r)
{
	ret rand() % (r - l + 1) + l;
}

ll bin_pow(ll n, ll x, ll mod)
{
	ll r = 1;
	ll um = n;
	while (x)
	{
		if (x & 1)
		{
			r *= um;
			r %= mod;
		}
		x >>= 1;
		um *= um;
		um %= mod;
	}
	ret r;
}

ll mtf(ll a, ll b, ll mod)
{
	ret a* bin_pow(b, mod - 2, mod) % mod;
}

int main()
{
	srand(time(0));
	if (local)
	{
		freopen("input.txt", "r", stdin);
		freopen("output.txt", "w", stdout);
	}
	int t = 1;
	cin >> t;
	while (t--)
	{
		solve();
	}
}

//void dfs(ll v, ll& b, vvpll& g, vll& used, vvpll& g1, vll& kol)
//{
//	if (used[v] == 0)
//		used[v] = 1;
//	for (auto x : g[v])
//	{
//		if (used[x.fi] == 1)
//		{
//			used[v] = -1;
//			//break;
//		}
//		else if (used[x.fi] == 0)
//		{
//			if (used[v] == -1)
//			{
//				used[x.first] = -1;
//			}
//			dfs(x.first, b, g, used, g1, kol);
//			if (used[x.first] == -1)
//			{
//				used[v] = -1;
//				//break;
//			}
//		}
//		else
//		{
//			if (used[v] == -1)
//			{
//				used[x.first] = -1;
//			}
//		}
//	}
//	if (used[v] != -1)
//	{
//		used[v] = 2;
//		for (auto x : g[v])
//		{
//			g1[v].pb({x.fi,x.se});
//			kol[x.first]++;
//		}
//	}
//}

void dfs(ll v, vvpll& g, vll& used, vvpll& g1, vll& kol, vpll& izm)
{
	if (used[v] == 0)
	{
		used[v] = 1;
	}
	for (auto x : g[v])
	{
		if (used[x.fi] == 1)
		{
			used[v] = -1;
		}
		if (izm[x.fi].first == -1)
		{
			if (izm[v].second != -1)
			{
				izm[x.first].first = izm[v].first ^ x.second;
				izm[x.first].se = izm[v].se ^ x.second;
				dfs(x.fi, g, used, g1, kol, izm);
				if (used[x.fi] == -1)
				{
					used[v] = -1;
				}
			}
			else
			{
				izm[x.first].first = izm[v].first ^ x.second;
				dfs(x.fi, g, used, g1, kol, izm);
				if (used[x.fi] == -1)
				{
					used[v] = -1;
				}
			}
		}
		else if (izm[x.first].second == -1)
		{
			if (izm[v].second == -1)
			{
				if ((izm[v].first ^ x.second) != izm[x.first].fi)
				{
					izm[x.fi].second = izm[v].first ^ x.second;
					dfs(x.first, g, used, g1, kol, izm);
					if (used[x.fi] == -1)
					{
						used[v] = -1;
					}
				}
			}
			else
			{
				if (izm[v].first != izm[x.first].fi)
				{
					izm[x.fi].second = izm[v].first ^ x.second;
				}
				else
				{
					izm[x.fi].second = izm[v].se ^ x.second;
				}
				dfs(x.first, g, used, g1, kol, izm);
				if (used[x.fi] == -1)
				{
					used[v] = -1;
				}
			}
		}
	}
	if (used[v] != -1)
	{
		used[v] = 0;
		if (izm[v].second == -1)
		{
			for (auto x : g[v])
			{
				g1[v].pb(x);
				kol[x.fi]++;
			}
		}
	}
}

bool dfs1(ll v, vvpll& g, bool bl, ll sum, vll& used, vpll& izm, vpll& rused)
{
	for (auto x : g[v])
	{
		if (used[x.fi] == -1 && ((sum ^ x.second) != izm[x.first].fi || bl || izm[x.fi].second != -1))
		{
			cout << -1 << endl;
			ret 0;
		}
		if (rused[x.fi].fi < 1 && rused[x.fi].se != (sum ^ x.se))
		{
			if (rused[x.fi].second != (sum ^ x.se) || rused[x.fi].fi == 0)
			{
				rused[x.fi].fi++;
				rused[x.first].second = sum ^ x.second;
			}
			bool b = dfs1(x.first, g, (bl || (rused[x.fi].fi)), sum ^ x.second, used, izm, rused);
			if (!b)
			{
				ret 0;
			}
		}
	}
	ret 1;
}

void solve()
{
	ll n, m;
	cin >> n >> m;
	vector<vector<pair<ll, ll>>> g(n);
	for0(i, m)
	{
		ll a, b, c;
		cin >> a >> b >> c;
		a--;
		b--;
		g[a].pb(make_pair(b, c));
	}
	ll a, b;
	cin >> a >> b;
	a--;
	b--;
	vll cnt(n);
	vll used(n);
	vll kol(n);
	vector<vector<pair<ll, ll>>> g1(n);
	vpll izm(n, { -1,-1 });
	izm[a] = { 0,-1 };
	dfs(a, g, used, g1, kol, izm);
	vector<vector<pair<ll, ll>>> rg(n);
	for0(i, n)
	{
		for0(j, g[i].size())
		{
			rg[g[i][j].fi].pb({ i,g[i][j].se });
		}
	}
	vpll rused(n, { -1,0 });
	rused[b] = { 0,0 };
	if (!dfs1(b, rg, 0, 0, used, izm, rused))
	{
		ret;
	}
	for0(i, n)
	{
		if (used[i] == 2)
			used[i] = 0;
	}
	vll sum(n);
	vll st;
	st.pb(a);
	kol[a] = 1;
	while (!st.empty())
	{
		ll v = st.back();
		used[v] = 1;
		st.pob();
		for (auto x : g1[v])
		{
			if ((kol[v] % 2))
				sum[x.fi] ^= (sum[v] ^ x.se);
			else
				sum[x.fi] ^= sum[v];
			cnt[x.fi]++;
			if (cnt[x.fi] == kol[x.fi])
			{
				st.pb(x.fi);
			}
		}
	}
	cout << sum[b] << endl;
	ret;
	if (used[b] == -1)
	{
		cout << -1 << endl;
	}
	else
	{
		cout << sum[b] << endl;
	}
}