#pragma GCC optimize("O3")

#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <cmath>
#include <cassert>
#include <queue>
#include <set>
#include <map>
#include <unordered_set>
#include <unordered_map>
#include <random>
#include <fstream>

using namespace std;

#ifdef LOCAL
ifstream input("input");
#define cin input
#define debug(x) cerr << #x << "=" << (x) << endl
#else
#define debug(x) ;
#endif

#define endl '\n'
#define int long long
#define float long double

random_device rd;
#ifndef int
mt19937 rng(rd());
#else
mt19937_64 rng(rd());
#endif

template<class T> int ckmin(T &a, const T &b) {
	if (b < a) {
		a = b;
		return 1;
	}
	return 0;
}

template<class T> int ckmax(T &a, const T &b) {
	if (b > a) {
		a = b;
		return 1;
	}
	return 0;
}

int mpow(int a, int n, int mod) {
	int ans = 1;
	while (n) {
		if (n & 1) {
			ans *= a;
			ans %= mod;
		}
		a *= a;
		a %= mod;
		n >>= 1;
	}
	return ans;
}

int n, m;
vector<vector<pair<int, int>>> g, G;
vector<int> used, order, col;
vector<vector<int>> by_c;

void dfs1(int u) {
	used[u] = 1;
	for (const pair<int, int> &e : g[u]) {
		int v = e.first;
		if (!used[v]) {
			dfs1(v);
		}
	}
	order.push_back(u);
}

void dfs2(int u, int C) {
	used[u] = 1;
	col[u] = C;
	by_c.back().push_back(u);
	for (const pair<int, int> &e : G[u]) {
		int v = e.first;
		if (!used[v]) {
			dfs2(v, C);
		}
	}
}

vector<vector<int>> xor1, xor2;

void dfs3(int u, int sum) {
	used[u] = 1;
	xor1[u].push_back(sum);
	for (const pair<int, int> &e : g[u]) {
		int v, w;
		tie(v, w) = e;
		int sum2 = sum ^ w;
		if (!used[v]) {
			dfs3(v, sum2);
		} else if ((int) xor1[v].size() == 1) {
			if (sum2 != xor1[v][0]) {
				dfs3(v, sum2);
			}
		} else {
			assert((int) xor1[v].size() == 2);
		}
	}
}

void dfs4(int u, int sum) {
	used[u] = 1;
	xor2[u].push_back(sum);
	for (const pair<int, int> &e : G[u]) {
		int v, w;
		tie(v, w) = e;
		int sum2 = sum ^ w;
		if (!used[v]) {
			dfs4(v, sum2);
		} else if ((int) xor2[v].size() == 1) {
			if (sum2 != xor2[v][0]) {
				dfs4(v, sum2);
			}
		} else {
			assert((int) xor2[v].size() == 2);
		}
	}
}

void solve() {
	g.clear();
	G.clear();
	order.clear();
	by_c.clear();
	cin >> n >> m;
	vector<tuple<int, int, int>> E;
	g.resize(n);
	G.resize(n);
	for (int i = 0; i < m; ++i) {
		int u, v, w;
		cin >> u >> v >> w;
		--u;
		--v;
		E.emplace_back(u, v, w);
		g[u].emplace_back(v, w);
		G[v].emplace_back(u, w);
	}
	used.assign(n, 0);
	for (int i = 0; i < n; ++i) {
		if (!used[i]) {
			dfs1(i);
		}
	}
	reverse(order.begin(), order.end());
	used.assign(n, 0);
	col.resize(n);
	int C = 0;
	for (const int &i : order) {
		if (!used[i]) {
			by_c.emplace_back();
			dfs2(i, C);
			++C;
		}
	}
	int s, t;
	cin >> s >> t;
	--s;
	--t;
	vector<int> cnt(C, 0);
	for (const tuple<int, int, int> &e : E) {
		int u, v, w;
		tie(u, v, w) = e;
		if (col[u] == col[v]) {
			++cnt[col[u]];
		}
	}
	xor1.clear();
	xor2.clear();
	xor1.resize(n);
	xor2.resize(n);
	used.assign(n, 0);
	dfs3(s, 0);
	used.assign(n, 0);
	dfs4(t, 0);
	for (int i = 0; i < n; ++i) {
		if (cnt[col[i]] == 0 || xor1[i].empty() || xor2[i].empty()) {
			continue;
		}
		if ((int) xor1[i].size() > 1 || (int) xor2[i].size() > 1 || xor1[i][0] != xor2[i][0]) {
			cout << -1 << endl;
			return;
		}
	}
	vector<int> dp1(C, 0), dp2(C, 0);
	dp1[col[s]] = 1;
	dp2[col[t]] = 1;
	for (int i = 0; i < C; ++i) {
		if (cnt[i]) {
			continue;
		}
		for (const int &u : by_c[i]) {
			for (const pair<int, int> &e : g[u]) {
				int v = e.first;
				int j = col[v];
				if (j != i) {
					dp1[j] ^= dp1[i];
				}
			}
		}
	}
	for (int i = C - 1; i >= 0; --i) {
		if (cnt[i]) {
			continue;
		}
		for (const int &u : by_c[i]) {
			for (const pair<int, int> &e : g[u]) {
				int v = e.first;
				int j = col[v];
				if (j != i) {
					dp2[i] ^= dp2[j];
				}
			}
		}
	}
	int ans = 0;
	for (const tuple<int, int, int> &e : E) {
		int u, v, w;
		tie(u, v, w) = e;
		u = col[u];
		v = col[v];
		if (dp1[u] % 2 == 1 && dp2[v] % 2 == 1 && cnt[u] == 0 && cnt[v] == 0) {
			ans ^= w;
		}
	}
	cout << ans << endl;
}

int32_t main() {
#ifndef LOCAL
	ios_base::sync_with_stdio(false);
	cout.tie(nullptr);
	cin.tie(nullptr);
#endif
	int T;
	cin >> T;
	for (int i = 1; i <= T; ++i) {
		cerr << "TEST #" << i << endl;
		solve();
	}
}
