#define _GLIBCXX_DEBUG

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <vector>
#include <cassert>
#include <functional>
#include <set>
#include <algorithm>
#include <map>
#include <random>
#include <chrono>

using namespace std;

const uint64_t MOD = 1000000000 + 7;

struct Req{
    uint64_t a, b, c, d;
};

uint64_t pow_mod(uint64_t a, uint64_t b){
    if (b == 0){
        return 1;
    }
    uint64_t p = pow_mod(a, b / 2);
    uint64_t s = (p * p) % MOD;
    if (b % 2){
        return (s * a) % MOD;
    }
    return s;
}

uint64_t mod_reverse(uint64_t x){
    assert(x);
    return pow_mod(x, MOD - 2);
}

struct DSU{
    int n;
    vector<int> parent;
    vector<pair<uint64_t, uint64_t>> r_seg;
    vector<int> r_tsize;

    uint64_t res_n = 1;
    uint64_t res_zeroes = 0;

    void umnozit(uint64_t X){
//        printf("DEBUG> Dobavlyau %lu\n", X);
        if (X){
            res_n = (res_n * X) % MOD;
        } else {
            res_zeroes++;
        }
    }

    void ubrat(uint64_t X){
//        printf("DEBUG> Ubirayu %lu\n", X);
        if (X){
            res_n = (res_n * mod_reverse(X)) % MOD;
        } else {
            res_zeroes--;
        }
    }

    uint64_t comb_in_seg(int i){
        if (r_seg[i].second >= r_seg[i].first){
            return (r_seg[i].second - r_seg[i].first + 1) % MOD;
        }
        return 0;
    }

    DSU(int _n, const vector<pair<uint64_t, uint64_t>>& start_segs) : n(_n), r_seg(start_segs){
        parent.resize(n);
        for (int i = 0; i < n; i++){
            parent[i] = i;
            umnozit(comb_in_seg(i));
        }
        r_tsize.assign(n, 1);
    }

    int get_root(int v){
        int& p = parent[v];
        if (p == v){
            return v;
        }
        int r = get_root(p);
        p = r;
        return r;
    }

    void merge(int a, int b){
        a = get_root(a);
        b = get_root(b);
        if (a != b){
            uint64_t prev_v_a = comb_in_seg(a);
            uint64_t prev_v_b = comb_in_seg(b);
            ubrat(prev_v_a);
            ubrat(prev_v_b);
            if (r_tsize[a] < r_tsize[b]){
                swap(a, b);  // razmer dereva a ne menshe b. Dobavlyaem b k a
            }
            parent[b] = a;
            r_tsize[a] += r_tsize[b];
            r_seg[a].first = max(r_seg[a].first, r_seg[b].first);
            r_seg[a].second = min(r_seg[a].second, r_seg[b].second);
            uint64_t new_v_a = comb_in_seg(a);
            umnozit(new_v_a);
        }
    }

    uint64_t get_ans(){
        return res_zeroes ? 0 : res_n;
    }
};

struct LCA{
    int mx_log = 19;
    int n;
    vector<int> parent;
    vector<uint64_t> height;
    vector<vector<int>> super_parent;

    LCA(const vector<int>& _p, const vector<uint64_t>& _h) : n(_p.size()), parent(_p), height(_h){
        super_parent.resize(mx_log + 1);
        super_parent[0] = parent;
        for (int L = 1; L <= mx_log; L++){
            super_parent[L].resize(n);
            for (int f = 0; f < n; f++){
                super_parent[L][f] = super_parent[L - 1][super_parent[L - 1][f]];
            }
        }
    }

    int go_up(uint64_t f, uint64_t steps){
        for (uint32_t ll = mx_log; ll <= mx_log; ll--){
            if ((steps >> ll) & 1){
                f = super_parent[ll][f];
            }
        }
        return f;
    }

    int lca(uint64_t a, uint64_t b){
        if (height[a] > height[b]){
            return lca(b, a);
        }
        b = go_up(b, height[b] - height[a]);
        if (a == b){
            return a;
        }
        for (uint32_t ll = mx_log; ll <= mx_log; ll--){
            int ap = super_parent[ll][a];
            int bp = super_parent[ll][b];
            if (ap != bp){
                a = ap;
                b = bp;
            }
        }
        return super_parent[0][a];
    }
};

int main(){
    int n;
    int m;
    cin >> n;
    vector<int> tree_parent(n, 0);
    vector<vector<int>> tree_children(n);
    vector<uint64_t> tree_height(n);
    tree_height[0] = 0;
    for (int i = 1; i < n; i++){
        int pi;
        cin >> pi;
        pi--;
        tree_parent[i] = pi;
        tree_children[pi].push_back(i);
    }
    function<void(int, uint64_t)> dfs = [&](int v, uint64_t h){
        tree_height[v] = h;
        for (int ch : tree_children[v]){
            dfs(ch, h + 1);
        }
    };
    dfs(0, 0);
    LCA lca_struct{tree_parent, tree_height};
    vector<pair<uint64_t, uint64_t>> price_pos(n);
    for (int i = 0; i < n; i++){
        cin >> price_pos[i].first >> price_pos[i].second;
    }
    DSU podscheti{n, price_pos};
    cin >> m;
    vector<Req> koroli(m);
    for (int i = 0; i < m; i++){
        cin >> koroli[i].a >> koroli[i].b >> koroli[i].c >> koroli[i].d;
        koroli[i].a --;
        koroli[i].b --;
        koroli[i].c --;
        koroli[i].d --;
    }
    auto check_straight = [&](uint64_t L, int b1, int b2){
//        printf("DEBUG> Check straight %lu | %d %d\n", L, b1, b2);
        for (int i = 0; i < L; i++){
//            printf("DEBUG> Merged %d %d\n", b1, b2);
            podscheti.merge(b1, b2);
            b1 = tree_parent[b1];
            b2 = tree_parent[b2];
        }
    };
    auto check_reverse = [&](uint64_t L, int b1, int b2){
//        printf("DEBUG> Check reverse %lu | %d %d\n", L, b1, b2);
        vector<int> path2(L);
        for (int i = 0; i < L; i++){
            path2[i] = b2;
            b2 = tree_parent[b2];
        }
        for (int i = 0; i < L; i++){
//            printf("DEBUG> Merged %d %d\n", b1, path2[L - 1 - i]);
            podscheti.merge(b1, path2[L - 1 - i]);
            b1 = tree_parent[b1];
        }
    };
    for (int i = 0; i < m; i++){
//        printf("DEBUG> Examonator %d\n", i);
        int a = koroli[i].a;
        int b = koroli[i].b;
        int c = koroli[i].c;
        int d = koroli[i].d;
        // I literally could make all of this 228 times simpler
        uint64_t L1 = lca_struct.lca(a, b);
        uint64_t L2 = lca_struct.lca(c, d);
        uint64_t Ast = tree_height[a] - tree_height[L1];
        uint64_t Bst = tree_height[b] - tree_height[L1];
        uint64_t Cst = tree_height[c] - tree_height[L2];
        uint64_t Dst = tree_height[d] - tree_height[L2];
        if (Ast < Cst){
            check_straight(Ast, a, c);
            a = L1;
            c = lca_struct.go_up(c, Ast);
            Cst -= Ast;
            Ast = 0;
        } else {
            check_straight(Cst, a, c);
            a = lca_struct.go_up(a, Cst);
            Ast -= Cst;
            c = L2;
            Cst = 0;
        }
        if (Bst < Dst){
            check_straight(Bst, b, d);
            b = L1;
            d = lca_struct.go_up(d, Bst);
            Dst -= Bst;
            Bst = 0;
        } else {
            check_straight(Dst, b, d);
            b = lca_struct.go_up(b, Dst);
            Bst -= Dst;
            d = L2;
            Dst = 0;
        }
        if (Ast == 0){
            // fix b and c
            assert(Bst == Cst);
            check_reverse(Bst + 1, b, c);
        } else {
            // fiz a and d
            assert(Ast == Dst);
            check_reverse(Ast + 1, a, d);
        }
        printf("%lu\n", podscheti.get_ans());
    }
    return 0;
}
