#include <bits/stdc++.h>

#include <utility>

#define forn(i, n) for (int i = 0; i < (int) (n); ++i)
#define rep(i, a, b) for (int i = (a); i <= (b); ++i)
#define repr(i, a, b) for (int i = (a); i >= (b); --i)
#define all(x) (x).begin(),(x).end()
#define len(x) (int)(x).size()

template<typename T>
std::ostream &operator<<(std::ostream &out, std::vector<T> &a) {
    for (auto &el: a)
        out << el << ' ';
    return out;
}

template<typename T>
std::istream &operator>>(std::istream &in, std::vector<T> &a) {
    for (auto &el: a)
        in >> el;
    return in;
}

template<typename T1, typename T2>
std::istream &operator>>(std::istream &in, std::pair<T1, T2> &a) {
    in >> a.first >> a.second;
    return in;
}

typedef long long ll;
typedef long double ld;
using namespace std;

const int mod = 1e9 + 7;

int add(int a, int b) {
    if (a + b >= mod)
        return a + b - mod;
    return a + b;
}

int mul(int a, int b) {
    return int(1LL * a * b % mod);
}

int sub(int a, int b) {
    if (a - b < 0)
        return a - b + mod;
    return a - b;
}

int binpow(int a, int n) {
    int res = 1;
    while (n) {
        if (n&1)
            res = mul(res, a);
        a = mul(a, a);
        n >>= 1;
    }
    return res;
}

int inv(int x) {
    return binpow(x, mod - 2);
}

struct DSU {
    vector<int> p, sz;
    vector<pair<int, int>> cuts;
    int val = 1;

    DSU (int n, vector<pair<int, int>> bord) : p(n), sz(n, 1) {
        iota(all(p), 0);
        cuts = bord;
        for (auto &cut : cuts) {
            val = mul(val, cut.second - cut.first + 1);
        }
    }

    int get(int v) {
        return p[v] == v ? v : p[v] = get(p[v]);
    }

    void merge(int a, int b) {
        a = get(a);
        b = get(b);
        if (a == b)
            return;
        if (sz[a] > sz[b])
            swap(a, b);
        p[a] = b;
        sz[b] += sz[a];
        val = mul(val, inv(cuts[a].second - cuts[a].first + 1));
        val = mul(val, inv(cuts[b].second - cuts[b].first + 1));
        cuts[b].first = max(cuts[a].first, cuts[b].first);
        cuts[b].second = min(cuts[a].second, cuts[b].second);
        if (cuts[b].first > cuts[b].second)
            val = 0;
        else
            val = mul(val, cuts[b].second - cuts[b].first + 1);
    }
};

const int maxn = 2e5 + 1, lg = 20;

vector<int> g[maxn];
int tin[maxn], tout[maxn], up[lg][maxn];
int timer = 0;
void dfs(int v, int p = -1) {
    tin[v] = timer++;

    if (p == -1) {
        for (auto &i : up)
            i[0] = v;
    } else {
        up[0][v] = p;
        for (int pw = 1; pw < lg; ++pw) {
            up[pw][v] = up[pw - 1][up[pw - 1][v]];
        }
    }

    for (auto to : g[v]) {
        if (to == p)
            continue;
        dfs(to, v);
    }

    tout[v] = timer++;
}

bool anc(int p, int v) {
    return tin[p] <= tin[v] && tout[v] <= tout[p];
}

int get_lca(int a, int b) {
    if (a == b)
        return a;
    if (anc(a, b))
        return a;
    if (anc(b, a))
        return b;

    for (int pw = lg - 1; pw >= 0; --pw) {
        if (!anc(up[pw][a], b)) {
            a = up[pw][a];
        }
    }

    if (anc(a, b))
        return a;
    return up[0][a];
}

void solve() {
    int n; cin >> n;
    for (int i = 1; i <= n - 1; ++i) {
        int p; cin >> p, --p;
        g[p].push_back(i);
    }

    dfs(0);
    vector<pair<int, int>> cuts;
    for (int i = 0; i < n; ++i) {
        int x, y;
        cin >> x >> y;
        cuts.emplace_back(x, y);
    }

    DSU dsu(n, cuts);
    int m; cin >> m;
    while (m--) {
        int a, b, c, d;
        cin >> a >> b >> c >> d, --a, --b, --c, --d;
        int lca1 = get_lca(a, b), lca2 = get_lca(c, d);
        vector<int> path1, path2;
        vector<int> tmp;

        while (a != lca1) {
            path1.push_back(a);
            a = up[0][a];
        }
        path1.push_back(lca1);
        while (b != lca1) {
            tmp.push_back(b);
            b = up[0][b];
        }
        reverse(all(tmp));
        for (auto &el : tmp)
            path1.push_back(el);
        tmp.clear();

        while (c != lca2) {
            path2.push_back(c);
            c = up[0][c];
        }
        path2.push_back(lca2);
        while (d != lca2) {
            tmp.push_back(d);
            d = up[0][d];
        }
        reverse(all(tmp));
        for (auto &el : tmp)
            path2.push_back(el);
        tmp.clear();

        assert(len(path1) == len(path2));

        for (int i = 0; i < len(path1); ++i) {
            dsu.merge(path1[i], path2[i]);
        }

        cout << dsu.val << '\n';
    }
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);
    solve();
    return 0;
}