#include <iostream>
#include <vector>
#include <algorithm>
#include <iomanip>
#include <random>
#include <set>

#define int long long
#define double long double

using namespace std;

const int MAXN = 201'000;
const int inf = 1e18;
const int mod = 1e9 + 7;

mt19937 rnd;

struct seg{
    int l, r;

    seg operator^(seg p){
        return {max(l, p.l), min(r, p.r)};
    }

    int len(){
        return max(0ll, r - l + 1);
    }

};

vector <int> g[MAXN];
int comp[MAXN];
seg c[MAXN];
int tin[MAXN], tout[MAXN], tt = 0;
int p[MAXN];
int rang[MAXN];
int ans = 1;

inline int getcomp(int u){
    if (u == comp[u]) return u;
    return comp[u] = getcomp(comp[u]);
}

inline int binpow(int a, int n){
    if (n == 0) return 1;
    if (n == 1) return a;
    int x = binpow(a, n / 2); x = (x * x) % mod;
    if (n % 2) return (x * a) % mod;
    return x;
}

inline void join(int u, int v){
    u = getcomp(u),v =  getcomp(v);
    if (u == v) return;
    ans = (ans * binpow(c[u].len(), mod - 2)) % mod;
    ans = (ans * binpow(c[v].len(), mod - 2)) % mod;
    if (rang[u] < rang[v]) {
        c[v]  = c[v] ^ c[u];
        ans = (ans * c[v].len()) % mod;
        comp[u] = v;
        if (rang[u] == rang[v]) rang[v]++;
    }
    else{
        c[u] = c[u] ^ c[v];
        ans = (ans * c[u].len()) % mod;
        comp[v] = u;
        if (rang[u] == rang[v]) rang[u]++;
    }
}

void dfs(int v, int pr = 0){
    p[v] = pr;
    tin[v] = tt++;
    for (int u : g[v]) if (u != pr) dfs(u, v);
    tout[v] = tt++;
}

inline bool is(int u, int v){
    return tin[u] <= tin[v] && tout[u] >= tout[v];
}

vector <int> path(int u, int v){
    if (u == v) return {u};
    if (is(v, u)){
        vector <int> res;
        do{
            res.push_back(u);
            u = p[u];
        } while (v != u);
        res.push_back(u);
        return res;
    }
    if (is(u, v)){
        vector <int> res;
        do{
            res.push_back(v);
            v = p[v];
        } while (v != u);
        res.push_back(u);
        reverse(res.begin(), res.end());
        return res;
    }
    vector <int> res;
    do{
        res.push_back(u);
        u = p[u];
    } while (!is(u, v));
    res.push_back(u);
    vector <int> res2;
    do{
        res2.push_back(v);
        v = p[v];
    } while (v != u);
    reverse(res2.begin(), res2.end());
    for (int i : res2) res.push_back(i);
    return res;
}

signed main() {
    ios_base::sync_with_stdio(0); cin.tie(0);
    cout << setprecision(11) << fixed;   cout.tie(0);
    int n; cin >> n;
    for (int i = 1; i < n; i ++){
        int p; cin >> p; p --;
        g[p].push_back(i);
        g[i].push_back(p);
    }
    for (int i = 0; i < n; i ++){
        comp[i] = i;
        rang[i] = 0;
        cin >> c[i].l >> c[i].r;
        ans = (ans * c[i].len()) % mod;
    }
    dfs(0);
    int m; cin >> m;
    for (int i = 0; i < m; i ++) {
        int a, b, cp, d;
        cin >> a >> b >> cp >> d;
        a--; b --; cp --; d--;
        vector <int> k1 = path(a, b);
        vector <int> k2 = path(cp, d);
        for (int j = 0; j < k1.size(); j ++) join(k1[j], k2[j]);
        cout << ans << endl;
    }
}
