#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

#define int long long

const int mod = 1e9 + 7;
const int maxn = 2e5;

void chmin(int &a, int b) {
	if (a > b) a = b;
}

void chmax(int &a, int b) {
	if (a < b) a = b;
}

int SUM(int a, int b) {
	return (a + b) % mod;
}

int MULT(int a, int b) {
	return (a * b) % mod;
}

struct DSU {
	int n;
	vector<pair<int, int>> vec;
	int r[maxn];
	int p[maxn];

	DSU(int n, vector<pair<int, int>> vec): n(n), vec(vec) {
		fill(r, r + n, 1);
		fill(p, p + n, -1);
	}

	int par(int v) {
		if (p[v] == -1) return v;
		return p[v] = par(p[v]);
	}

	bool unite(int a, int b) {
		a = par(a);
		b = par(b);
		if (a == b) return 0;
		if (r[a] < r[b]) swap(a, b);
		chmax(vec[a].first, vec[b].first);
		chmin(vec[a].second, vec[b].second);
		p[b] = a;
		r[a] += r[b];
		return 1;
	}

	int calc() {
		int ans = 1;
		for (int v = 0; v < n; ++v) {
			if (p[v] == -1) {
				ans = MULT(ans, (vec[v].first <= vec[v].second ? vec[v].second - vec[v].first + 1 : 0));
			}
		}
		return ans;
	}
};

const int maxlg = 20;

struct BinUp {
	int n;
	//int p[maxn];
	vector<int> p;
	int par[maxlg][maxn];
	vector<int> g[maxn];
	int d[maxn];

	BinUp(int n, vector<int> p): n(n), p(p) {
		//for (int lg = 0; lg < maxlg; ++lg) par[lg].resize(n);
		//d.resize(n);
		//g.resize(n);
		for (int i = 1; i < n; ++i) {
			g[p[i]].push_back(i);
		}
	}

	void dfs(int v, int dep) {
		d[v] = dep;
		for (int u: g[v]) dfs(u, dep + 1);
	}

	void build() {
		dfs(0, 0);
		for (int v = 0; v < n; ++v) {
			par[0][v] = p[v];
		}
		for (int lg = 1; lg < maxlg; ++lg) {
			for (int v = 0; v < n; ++v) {
				if (par[lg - 1][v] != -1) par[lg][v] = par[lg - 1][par[lg - 1][v]];
				else par[lg][v] = -1;
			}
		}
	}

	int lca(int v, int u) {
		if (d[v] > d[u]) swap(u, v);
		for (int lg = maxlg - 1; lg >= 0; --lg) {
			if (par[lg][u] == -1) continue;
			if (d[par[lg][u]] >= d[v]) u = par[lg][u];
		}
		if (v == u) {
			return v;
		}
		for (int lg = maxlg - 1; lg >= 0; --lg) {
			if (par[lg][v] != par[lg][u]) {
				v = par[lg][v];
				u = par[lg][u];
			}
		}
		return par[0][v];
	}

	int parent(int v) {
		return par[0][v];
	}
};

vector<vector<int>> g;

bool seek(int v, int p, int t, vector<int> &path) {
	if (v == t) return 1;
	for (int u: g[v]) if (u != p) {
		path.push_back(u);
		if (seek(u, v, t, path)) return 1;
		path.pop_back();
	}
	return 0;
}

signed main() {
	#ifndef LOCAL
	ios::sync_with_stdio(0);
	cin.tie(0);
	#endif
	int n, m;
	cin >> n;
	g.assign(n, vector<int>(0));
	vector<int> par(n);
	par[0] = -1;
	for (int i = 1; i < n; ++i) {
		int p;
		cin >> p;
		--p;
		par[i] = p;
		g[p].push_back(i);
		g[i].push_back(p);
	}
	vector<pair<int, int>> vec(n);
	for (auto &[a, b]: vec) cin >> a >> b;
	DSU dsu(n, vec);
	BinUp bin(n, par);
	bin.build();
	int lol = 0;
	cin >> m;
	for (int t = 0; t < m; ++t) {
		int a, b, c, d;
		cin >> a >> b >> c >> d;
		--a; --b; --c; --d;
		vector<int> patha, pathb, pathc, pathd;
		int ab = bin.lca(a, b), cd = bin.lca(c, d);
		// seek(a, -1, b, pathab);
		// seek(c, -1, d, pathcd);
		while (a != ab) {
			patha.push_back(a);
			a = bin.parent(a);
		}
		while (b != ab) {
			pathb.push_back(b);
			b = bin.parent(b);
		}
		while (c != cd) {
			pathc.push_back(c);
			c = bin.parent(c);
		}
		while (d != cd) {
			pathd.push_back(d);
			d = bin.parent(d);
		}
		patha.push_back(ab);
		pathc.push_back(cd);
		reverse(pathb.begin(), pathb.end());
		for (int x: pathb) patha.push_back(x);
		reverse(pathd.begin(), pathd.end());
		for (int x: pathd) pathc.push_back(x);
		//vector<int> pathab, pathcd;
		for (int i = 0; i < patha.size(); ++i) {
			dsu.unite(patha[i], pathc[i]);
		}
		cout << dsu.calc() << endl;
	}
}