#include "bits/stdc++.h"

using namespace std;
using ll = long long;
#define all(x) (x).begin(), (x).end()
#define sz(x) (int)(x).size()

#ifdef LOCAL
const int N = 10000;
#else
const int N = 2e5 + 10;
#endif

#define int ll

struct Fenwick {
	int n;
	vector<int> t;
	Fenwick(int n = 0) : n(n) {
		t.resize(n + 5, 0);
	}

	void add(int i, int x) {
		for (i++; i < n; i += i & -i)
			t[i] += x;
	}

	int sum(int r) {
		int res = 0;
		for (r++; r; r -= r & -r)
			res += t[r];
		return res;
	}

	int sum(int l, int r) { // [l; r]
		return sum(r) - sum(l - 1);
	}

	void print() {
		for (int i = 0; i < n; i++)
			cout << sum(i, i) << " ";
		cout << '\n';
	}
};

vector<int> pw1(N + 1, 1), pw2(N + 1, 1), ipw1(N + 1, 1), ipw2(N + 1, 1);
#ifdef LOCAL
const int P1 = 732913, P2 = N + 83214;
//int P1 = 10, P2 = 100;
#else
const int P1 = 732913, P2 = N + 83214;
#endif

constexpr int MOD2 = 998244353;
const int MOD1 = 1e9 + 7;

inline void add(int& a, int b) {
	if ((a += b) >= MOD1)
		a -= MOD1;
}
inline int mul(int a, int b) {
	return ((ll)a * b) % MOD1;
}

inline int binpow(int a, int n) {
	int res = 1;
	while (n) {
		if (n & 1)
			res = mul(res, a);
		a = mul(a, a);
		n >>= 1;
	}
	return res;
}
inline int binpow2(int a, int n) {
	int res = 1;
	while (n) {
		if (n & 1)
			res = ((ll)res * a) % MOD2;
		a = ((ll)a * a) % MOD2;
		n >>= 1;
	}
	return res;
}

int res = 1;
Fenwick h1(N), h2(N), rh1(N), rh2(N);
vector<int> values;
vector<vector<int>> sons;

struct dsu {
	int n;
	vector<int> p, s, l, r;
	dsu(int n) : n(n) {
		values.resize(n);
		iota(all(values), 0);
		sons.resize(n);
		for (int i = 0; i < n; i++)
			sons[i] = { i };

		for (int i = 0; i < n; i++) {
			h1.add(i, ((ll)(values[i] + 1) * pw1[i + 1]) % MOD1);
			h2.add(i, ((ll)(values[i] + 1) * pw2[i + 1]) % MOD2);

			rh1.add(i, ((ll)(values[i] + 1) * pw1[n - i]) % MOD1);
			rh2.add(i, ((ll)(values[i] + 1) * pw2[n - i]) % MOD2);
		}

		s.resize(n, 1);
	}

	void unite(int a, int b) {
		a = values[a], b = values[b];
		if (a == b)
			return;
		if (s[a] < s[b])
			swap(a, b);

		for (int x : sons[a]) {
			h1.add(x, ((b - a) * pw1[x + 1]) % MOD1);
			h2.add(x, ((b - a) * pw2[x + 1]) % MOD2);

			rh1.add(x, ((b - a) * pw1[n - x]) % MOD1);
			rh2.add(x, ((b - a) * pw2[n - x]) % MOD2);
			sons[b].push_back(x);
			values[x] = b;
		}
		sons[a] = {};

		res = mul(res, binpow(r[b] - l[b] + 1, MOD1 - 2));
		res = mul(res, binpow(r[a] - l[a] + 1, MOD1 - 2));

		l[b] = max(l[b], l[a]);
		r[b] = min(r[b], r[a]);
		r[b] = max(r[b], l[b] - 1);

		res = mul(res, r[b] - l[b] + 1);
	}
};

void solve_case() {
	int n; cin >> n;
	vector<int> p(n, 0), l(n), r(n);
	for (int i = 1; i < n; i++) {
		cin >> p[i];
		p[i]--;
	}

	for (int i = 1; i < sz(pw1); i++) {
		pw1[i] = ((ll)pw1[i - 1] * P1) % MOD1;
		pw2[i] = ((ll)pw2[i - 1] * P2) % MOD2;
	}

	ipw1.back() = binpow(pw1.back(), MOD1 - 2);
	ipw2.back() = binpow2(pw2.back(), MOD2 - 2);

	for (int i = sz(ipw1) - 2; i > 0; i--) {
		ipw1[i] = ((ll)P1 * ipw1[i + 1]) % MOD1;
		ipw2[i] = ((ll)P2 * ipw2[i + 1]) % MOD2;
	}

	for (int i = 0; i < n; i++) {
		cin >> l[i] >> r[i];
		res = mul(res, r[i] - l[i] + 1);
	}

	dsu D(n);
	D.l = l;
	D.r = r;

	/*h1.print();
	h2.print();
	rh1.print();
	rh2.print();

	cout << "\n";*/

	int m; cin >> m;
	while (m--) {
		int a, b, c, d; cin >> a >> b >> c >> d;
		/*if (abs(b - a) != abs(c - d)) {
			cout << "wrong\n";
		}*/

		/*if (!res) {
			cout << res << "\n";
			continue;
		}*/


		a--; b--; c--; d--;

		while (true) {
			int a_sign = a < b ? 1 : -1;
			int c_sign = c < d ? 1 : -1;

			{
				int m = abs(b - a);

				int na = a + a_sign * m;
				int nc = c + c_sign * m;

				int la = a, ra = na;
				int lc = c, rc = nc;

				pair<int, int> ha = {
					la < ra ? 
					((((h1.sum(la, ra) % MOD1) + MOD1) % MOD1) * ipw1[la]) % MOD1 : 
					((((rh1.sum(ra, la) % MOD1) + MOD1) % MOD1) * ipw1[n - ra]) % MOD1,
					
					la < ra ?
					((((h2.sum(la, ra) % MOD2) + MOD2) % MOD2) * ipw2[la]) % MOD2 :
					((((rh2.sum(ra, la) % MOD2) + MOD2) % MOD2) * ipw2[n - ra]) % MOD2
				};

				pair<int, int> hc = {
					lc < rc ?
					((((h1.sum(lc, rc) % MOD1) + MOD1) % MOD1) * ipw1[lc]) % MOD1 :
					((((rh1.sum(rc, lc) % MOD1) + MOD1) % MOD1) * ipw1[n - rc]) % MOD1,

					lc < rc ?
					((((h2.sum(lc, rc) % MOD2) + MOD2) % MOD2) * ipw2[lc]) % MOD2 :
					((((rh2.sum(rc, lc) % MOD2) + MOD2) % MOD2) * ipw2[n - rc]) % MOD2
				};

				if (ha == hc) {
					break;
				}
			}

			int l = -1, r = abs(b - a);
			while (r - l > 1) {
				int m = (l + r) / 2;

				int na = a + a_sign * m;
				int nc = c + c_sign * m;

				int la = a, ra = na;
				int lc = c, rc = nc;

				pair<int, int> ha = {
					la < ra ?
					((((h1.sum(la, ra) % MOD1) + MOD1) % MOD1) * ipw1[la]) % MOD1 :
					((((rh1.sum(ra, la) % MOD1) + MOD1) % MOD1) * ipw1[n - ra]) % MOD1,

					la < ra ?
					((((h2.sum(la, ra) % MOD2) + MOD2) % MOD2) * ipw2[la]) % MOD2 :
					((((rh2.sum(ra, la) % MOD2) + MOD2) % MOD2) * ipw2[n - ra]) % MOD2
				};

				pair<int, int> hc = {
					lc < rc ?
					((((h1.sum(lc, rc) % MOD1) + MOD1) % MOD1) * ipw1[lc]) % MOD1 :
					((((rh1.sum(rc, lc) % MOD1) + MOD1) % MOD1) * ipw1[n - rc]) % MOD1,

					lc < rc ?
					((((h2.sum(lc, rc) % MOD2) + MOD2) % MOD2) * ipw2[lc]) % MOD2 :
					((((rh2.sum(rc, lc) % MOD2) + MOD2) % MOD2) * ipw2[n - rc]) % MOD2
				};

				if (ha != hc) {
					r = m;
				}
				else {
					l = m;
				}
			}

			int na = a + a_sign * r;
			int nc = c + c_sign * r;

			D.unite(na, nc);
				a = na;
			c = nc;

			/*h1.print();
			h2.print();
			rh1.print();
			rh2.print();

			cout << "\n";*/
		}

		cout << res << "\n";
	}
}

signed main() {
	cin.tie(0)->sync_with_stdio(0);
#ifdef LOCAL
	(void)freopen("../input.txt", "r", stdin);
#endif

	int T = 1;
	//cin >> T;
	while (T--) {
		solve_case();
	}

	return 0;
}
