#include <iostream>
#include <utility>
#include <cmath>
#include <algorithm>
#include <vector>
#include <set>
#include <map>
#include <random>

using namespace std;

typedef long long ll;
typedef long double ld;

const int MOD = 1e9 + 7;

struct LCA{
    int n, k;
    vector<int> depths, parents;
    vector<vector<int>> dp;

    void dfs(int v, int par, vector<vector<int>>& g, vector<bool>& used){
        used[v] = true;
        parents[v] = par;
        if(par != v)
            depths[v] = depths[par] + 1;
        for(auto to : g[v]){
            if(!used[to])
                dfs(to, v, g, used);
        }

    }

    int get_log2(int n){
        ll num = 1;
        k = 0;
        while (num < n) {
            num *= 2;
            k++;
        }
        return k;
    }

    void LA(){
        for(int i = 0; i < n; i++)
            dp[0][i] = parents[i];
        for(int i = 1; i <= k; i++){
            for(int j = 0; j < n; j++){
                dp[i][j] = dp[i - 1][dp[i - 1][j]];
            }
        }
    }

    void build(vector<vector<int>>& g){
        n = g.size();
        vector<bool> used(n);
        depths.assign(n, 0);
        parents.assign(n, 0);
        dfs(0,0,g,used);

        int k = get_log2(n);
        dp.assign(k + 1, vector<int>(n));
        LA();
    }

    int up(int v, int level){
        for(int i = k; i >= 0; i--){
            if((depths[v] - (1ll << i)) >= level){
                v = dp[i][v];
            }
        }
        return v;
    }

    int lca(int v, int u){
        if(depths[v] < depths[u]) {
            u = up(u, depths[v]);
        }
        if (depths[u] < depths[v]) {
            v = up(v, depths[u]);
        }
        if(v == u)
            return v;

        for(int i = k; i >= 0; i--){
            if(dp[i][v] != dp[i][u]){
                v = dp[i][v];
                u = dp[i][u];
            }
        }
        return dp[0][v];
    }


};

vector<int> get_way(int a, int b, vector<int>& parent){
    vector<int> way;
    while(a != b){
        way.push_back(a);
        a = parent[a];
    }
    return way;
}

vector<int> get_way(int a, int b, int lca, vector<int>& parent){
    vector<int> way;
    if(lca == a){
        way = get_way(b,a,parent);
        way.push_back(a);
        reverse(way.begin(), way.end());
    }
    else if(lca == b){
        way = get_way(a,b,parent);
        way.push_back(b);
    }
    else{
        way = get_way(a,lca,parent);
        way.push_back(lca);
        auto w1 = get_way(b,lca,parent);
        reverse(w1.begin(), w1.end());
        for(auto i : w1)
            way.push_back(i);
    }
    return way;
}

struct dsu{
    vector<int> p, size;
    vector<pair<int, int>> subs;

    void build(vector<pair<int, int>>& s){
        subs = s;
        for(int i = 0; i < s.size(); i++){
            p.push_back(i);
            size.push_back(1);
        }
    }

    int root(int v){
        if(p[v] == v)
            return v;
        return p[v] = root(p[v]);
    }

    void union_sets(int v, int u){
        v = root(v), u = root(u);
        if(v == u)
            return;

        if(size[u] > size[v])
            swap(v, u);

        p[u] = v;
        size[v] += size[u];
        size[u] = 1;
        int l = max(subs[v].first, subs[u].first);
        int r = min(subs[v].second, subs[u].second);
        if (subs[v].first > subs[v].second || subs[u].first > subs[u].second){
            subs[v] = {1, 0};
        }
        else if(r < l)
            subs[v] = {1, 0};
        else
            subs[v] = {l,r};
        subs[u] = {0,0};
     }
};

int main() {
    //freopen("input", "r", stdin);
    ios::sync_with_stdio(0);
    cin.tie(nullptr);

    int n;
    cin >> n;
    vector<vector<int>> g(n);
    vector<int> parent(n);
    for(int i = 1;  i< n; i++){
        int pi;
        cin >> pi;
        pi--;
        parent[i] = pi;
        g[pi].push_back(i);
        g[i].push_back(pi);
    }

    LCA l;
    l.build(g);

    vector<pair<int, int>> dist(n);
    for(auto& i : dist){
        cin >> i.first >> i.second;
    }
    dsu s;
    s.build(dist);
    int m;
    cin >> m;
    for(int test = 0; test < m; test++){
        int a, b, c, d;
        cin >> a >> b >> c >> d;
        a--, b--, c--, d--;
        int lca1 = l.lca(a,b);
        int lca2 = l.lca(c,d);
       // cout<<lca1<<' ' << lca2<<"\n";
        auto way1 = get_way(a,b,lca1, parent);
        auto way2 = get_way(c,d,lca2,parent);
//        for(auto w : way1)
//            cout << w << ' ';
//        cout<<"\n";

        for(int i = 0; i < way1.size(); i++){
            s.union_sets(way1[i], way2[i]);
        }
        ll ans = 1;
        for(auto i : s.subs){
            ans = ans * (i.second - i.first + 1) % MOD;
        }
        cout<<ans<<"\n";
    }

    return 0;
}
