#pragma GCC optimize("O3")
#pragma GCC target("avx,avx2")

#include <iostream>
#include <vector> 
#include <algorithm>
#include <iomanip>
#include <numeric>
#include <functional>

using namespace std;

#define all(v) (v).begin(), (v).end()
#define rall(v) (v).rbegin(), (v).rend()


using int64 = long long;


//#define int long long

const int64 MOD = (int64)1e9 + 7;

int64 fpow(int64 a, int64 n) {
	int64 ans = 1;
	int64 cpw = a;
	while (n != 0) {
		if (n & 1) {
			ans *= cpw;
			ans %= MOD;
		}
		cpw *= cpw;
		cpw %= MOD;
		n >>= 1;
	}
	return ans;
}

class DSU {
private:
	vector <int> p, h;
	vector <pair <int, int>> sg;
	int64 ans;

	int get(int v) {
		if (p[v] == v) return v;
		return p[v] = get(p[v]);
	}

public:
	DSU(const vector <pair <int, int>>& v) {
		sg = v;
		ans = 1;
		for (auto& [l, r] : sg) {
			ans *= (r - l + 1);
			ans %= MOD;
		}
		p.resize(v.size()); iota(all(p), 0);
		h.assign(v.size(), 0);
	}

	void unite(int u, int v) {
		u = get(u);
		v = get(v);
		if (u == v) return;

		if (h[u] < h[v]) swap(u, v);
		if (h[u] == h[v]) ++h[u];
		p[v] = u;

		ans *= fpow(sg[u].second - sg[u].first + 1, MOD - 2);
		ans %= MOD;
		ans *= fpow(sg[v].second - sg[v].first + 1, MOD - 2);
		ans %= MOD;
		sg[u].first = max(sg[u].first, sg[v].first);
		sg[u].second = min(sg[u].second, sg[v].second);
		if (sg[u].first > sg[u].second) ans = 0;
		ans *= (sg[u].second - sg[u].first + 1);
		ans %= MOD;
	}

	int getAns() {
		return ans;
	}
};

void solve() {
	int n; cin >> n;
	vector <vector <int>> up(18, vector <int>(n));
	vector <vector <int>> g(n);
	up[0][0] = 0;
	for (int i = 1; i < n; ++i) {
		cin >> up[0][i];
		--up[0][i];
		g[up[0][i]].push_back(i);
	}

	for (int l = 1; l < up.size(); ++l) {
		for (int i = 0; i < n; ++i) {
			up[l][i] = up[l - 1][up[l - 1][i]];
		}
	}

	vector <int> h(n, -1);
	function <void(int)> dfs = [&](int v) {
		h[v] = h[up[0][v]] + 1;
		for (auto u : g[v]) dfs(u);
	};
	dfs(0);

	auto lca = [&](int u, int v) {
		if (h[u] < h[v]) swap(u, v);
		int d = h[u] - h[v];
		for (int i = 0; i < up.size(); ++i) {
			if (d >> i & 1) u = up[i][u];
		}
		if (u == v) return u;

		for (int i = up.size() - 1; i >= 0; --i) {
			if (up[i][u] != up[i][v]) {
				u = up[i][u];
				v = up[i][v];
			}
		}
		return up[0][u];
	};

	vector <pair <int, int>> sg(n);
	for (auto& [l, r] : sg) cin >> l >> r;

	DSU dsu(sg);

	int m; cin >> m;
	while (m--) {
		int a, b, c, d;
		cin >> a >> b >> c >> d;
		--a; --b; --c; --d;

		int l1 = lca(a, b);
		int l2 = lca(c, d);
		vector <int> v1, v2;
		{
			vector <int> v3;
			while (a != l1) {
				v1.push_back(a);
				a = up[0][a];
			}
			v1.push_back(l1);
			while (b != l1) {
				v3.push_back(b);
				b = up[0][b];
			}
			reverse(all(v3));
			v1.insert(v1.end(), all(v3));
		}
		{
			vector <int> v3;
			while (c != l2) {
				v2.push_back(c);
				c = up[0][c];
			}
			v2.push_back(l2);
			while (d != l2) {
				v3.push_back(d);
				d = up[0][d];
			}
			reverse(all(v3));
			v2.insert(v2.end(), all(v3));
		}

		for (int i = 0; i < v1.size(); ++i) {
			dsu.unite(v1[i], v2[i]);
		}

		cout << dsu.getAns() << '\n';
	}
}


int32_t main() {
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);
	cout.tie(nullptr);
	cout << fixed << setprecision(15);

	int t = 1;
	//cin >> t;
	while (t--) {
		solve();
		cout << '\n';
	}

	return 0;
}