#include <iostream>
#include <vector>
#include <iomanip>
#include <algorithm>
#include <string>
#include <chrono>
#include <fstream>
#include <random>
#include <set>
#include <map>
#include <unordered_map>
#include <unordered_set>

using namespace std;

#define ll long long
#define ld long double
#define pii pair<int, int>

mt19937 rnd(chrono::system_clock().now().time_since_epoch().count());

vector <vector <int>> g;
vector <int> height;

const ll MOD = 1e9 + 7;

vector <pii> price;

const int MAXN = 524288;

int pos = MAXN;

int tree[2 * MAXN];

vector <int> parent;
vector <int> tin;

void dfs_h(int v, int p, int h) {
    height[v] = h;
    for (int n: g[v]) {
        if (n != p) {
            dfs_h(n, v, h + 1);
        }
    }
}

void dfs_way(int v, int p) {
    tree[pos] = v;
    pos++;
    for (int n: g[v]) {
        if (n != p) {
            dfs_way(n, v);
            tree[pos] = v;
            pos++;
        }
    }
}

void build_tree() {
    for (int i = MAXN - 1; i > 0 ; i--) {
        if (height[tree[i << 1]] < height[tree[(i << 1) + 1]]) {
            tree[i] = tree[i << 1];
        } else {
            tree[i] = tree[(i << 1) + 1];
        }
    }
}

int get_min(int v, int l, int r, int ln, int rn) {
    if (l >= rn || r <= ln) {
        return 200022;
    }
    if (l >= ln && r <= rn) {
        return tree[v];
    }
    int m = (l + r) >> 1;
    int v1, v2;
    v1 = get_min(v << 1, l, m, ln, rn);
    v2 = get_min((v << 1) + 1, m, r, ln, rn);
    if (height[v1] < height[v2]) {
        return v1;
    } else {
        return v2;
    }
}

int lca(int v1, int v2) {
    int t1 = tin[v1], t2 = tin[v2];
    if (t1 > t2) {
        swap(t1, t2);
    }
    if (t1 == t2) {
        return v1;
    }
    return get_min(1, 0, MAXN, t1, t2 + 1);
}

ll fast_pow(ll a, ll p) {
    if (p == 0) {
        return 1;
    }
    ll res;
    if (p % 2 == 0) {
        res = fast_pow(a, p / 2);
        res *= res;
    } else {
        res = fast_pow(a, p - 1);
        res *= a;
    }
    res %= MOD;
    return res;
}

vector <int> root;
vector <int> comp_sz;
int left;

int get_parent(int v) {
    int res;
    if (root[v] == v) {
        return v;
    } else {
        res = get_parent(root[v]);
        root[v] = res;
        return res;
    }
}

signed main() {
    int n;
    cin >> n;
    parent.resize(n);
    g.resize(n);
    root.resize(n);
    for (int i = 0 ; i < n; i++) {
        root[i] = i;
    }
    comp_sz.assign(n, 1);
    height.resize(200050);
    price.resize(n);
    tin.assign(n, -1);
    height[200022] = 1e9;
    int t;
    for (int i = 1 ; i < n; i++) {
        cin >> t;
        t--;
        g[i].push_back(t);
        g[t].push_back(i);
        parent[i] = t;
    }
    parent[0] = 0;
    for (int i = 0 ; i < n; i++) {
        cin >> price[i].first >> price[i].second;
    }
    dfs_h(0, -1, 0);
    dfs_way(0, -1);
    for (int i = MAXN; i < pos; i++) {
        if (tin[tree[i]] == -1) {
            tin[tree[i]] = i - MAXN;
        }
    }
    for (int i = pos; i < 2 * MAXN; i++) {
        tree[i] = 200022;
    }
    build_tree();
    ll ans = 1;
    for (auto i: price) {
        ans *= (i.second - i.first + 1);
        ans %= MOD;
    }
    int q;
    cin >> q;
    int v1, v2, v3, v4;
    for (int i = 0 ; i < q; i++) {
        cin >> v1 >> v2 >> v3 >> v4;
        if (ans == 0) {
            cout << ans << "\n";
            continue;
        }
        v1--;
        v2--;
        v3--;
        v4--;
        int l1 = lca(v1, v2), l2 = lca(v3, v4);
        vector <int> w1, w2, w3, w4;
        while (v1 != l1) {
            w1.push_back(v1);
            v1 = parent[v1];
        }
        while (v3 != l2) {
            w3.push_back(v3);
            v3 = parent[v3];
        }
        while (v2 != l1) {
            w2.push_back(v2);
            v2 = parent[v2];
        }
        while (v4 != l2) {
            w4.push_back(v4);
            v4 = parent[v4];
        }
        w1.push_back(l1);
        w3.push_back(l2);
        for (int j = w2.size() - 1; j >= 0; j--) {
            w1.push_back(w2[j]);
        }
        for (int j = w4.size() - 1; j >= 0; j--) {
            w3.push_back(w4[j]);
        }
        for (int j = 0 ; j < w1.size(); j++) {
            int p1 = get_parent(w1[j]), p2 = get_parent(w3[j]);
            if (p1 != p2) {
                if (comp_sz[p1] < comp_sz[p2]) {
                    swap(p1, p2);
                }
                int val = price[p1].second - price[p1].first + 1;
                val = max(0, val);
                ans *= fast_pow(val, MOD - 2);
                ans %= MOD;
                val = price[p2].second - price[p2].first + 1;
                val = max(0, val);
                ans *= fast_pow(val, MOD - 2);
                ans %= MOD;
                price[p1].first = max(price[p1].first, price[p2].first);
                price[p1].second = min(price[p1].second, price[p2].second);
                val = price[p1].second - price[p1].first + 1;
                val = max(0, val);
                ans *= val;
                ans %= MOD;
                comp_sz[p1] += comp_sz[p2];
                root[p2] = p1;
            }
        }
        cout << ans << "\n";
    }
}