#include <iostream>
#include <vector>
#include <queue>
#include <deque>
#include <stack>
#include <numeric>
#include <algorithm>
#include <set>
#include <map>

using namespace std;

#define sz(a) (int)a.size()
#define vc vector
#define pii pair <int, int>
#define all(a) a.begin(), a.end()
#define fi first
#define se second
#define pb push_back

const int N = 5e5 + 1;
const int mod = 1e9 + 7;
const int base = N;
int pw[N];
int mul (int a, int b) {
    return a * 1ll * b % mod;
}

int add (int a, int b) {
    if (a + b >= mod)
        return a + b - mod;
    return a + b;
}

struct node {
    int hash = 0, len = 0;
    node () {}
    node (int x) {
        hash = x;
        len = 1;
    }
    node operator + (node const other) const {
        node c;
        c.hash = add(mul(hash, pw[other.len]), other.hash);
        c.len = len + other.len;
        return c;
    }
};

struct segtree {
    vc <node> t1, t2; int n;
    segtree () {}
    segtree (int n) : n(n) {
        t1.resize(n * 4);
        t2.resize(n * 4);
        build(1, 0, n);
    }
    void apply (int v) {
        t1[v] = t1[v << 1] + t1[v << 1 | 1];
        t2[v] = t2[v << 1 | 1] + t2[v << 1];
    }
    void build (int v, int tl, int tr) {
        if (tr - tl == 1) {
            t1[v] = t2[v] = tl + 1;
            return;
        }
        int tm = (tl + tr) / 2;
        build(v << 1, tl, tm);
        build(v << 1 | 1, tm, tr);
        apply(v);
    }
    node get1 (int v, int tl, int tr, int l, int r) {
        if (tr <= l || r <= tl) return node();
        if (l <= tl && tr <= r) return t1[v];
        int tm = (tl + tr) / 2;
        return get1(v << 1, tl, tm, l, r) + get1(v << 1 | 1, tm, tr, l, r);
    }
    node get2 (int v, int tl, int tr, int l, int r) {
        if (tr <= l || r <= tl) return node();
        if (l <= tl && tr <= r) return t2[v];
        int tm = (tl + tr) / 2;
        return get2(v << 1 | 1, tm, tr, l, r) + get2(v << 1, tl, tm, l, r);
    }
    void upd (int v, int tl, int tr, int i, int x) {
        if (tr - tl == 1) {
            t1[v] = t2[v] = x + 1;
            return;
        }
        int tm = (tl + tr) / 2;
        if (i < tm)
            upd(v << 1, tl, tm, i, x);
        else
            upd(v << 1 | 1, tm, tr, i, x);
        apply(v);
    }
    node get1 (int l, int r) {
        return get1(1, 0, n, l, r + 1);
    }
    node get2 (int l, int r) {
        return get2(1, 0, n, l, r + 1);
    }
    void upd (int i, int x) {
        upd(1, 0, n, i, x);
    }
    node get (int l, int r, int u) {
        if (u == 1)
            return get1(l, r);
        return get2(r, l);
    }
};

vc <int> g[N];
int l[N], r[N];
int tin[N], tout[N];
int pr[N];
int ans = 1;


vc <int> path;

int bin_pow (int a, int b) {
    int res = 1;
    while (b) {
        if (b & 1)
            res = mul(res, a);
        a = mul(a, a);
        b >>= 1;
    }
    return res;
}

bool is_ancestor (int u, int v) {
    return tin[u] <= tin[v] && tout[v] <= tout[u];
}

void get_path (int u, int v) {
    path.clear();
    vc <int> tt;
    while (!is_ancestor(v, u)) {
        tt.pb(v);
        v = pr[v];
    }
    while (!is_ancestor(u, v)) {
        path.pb(u);
        u = pr[u];
    }
    path.pb(u);
    reverse(all(tt));
    for (int j : tt)
        path.pb(j);
}

int timer = 0;
void dfs (int u, int p = -1) {
    tin[u] = timer++;
    for (int v : g[u]) {
        if (v != p)
            dfs(v, u);
    }
    tout[u] = timer++;
}

int p[N];
vc <int> ls[N];
segtree t;

int unite (int u, int v) {
    u = p[u];
    v = p[v];
    if (u == v)
        return 0;
    if (sz(ls[u]) < sz(ls[v]))
        swap(u, v);
    for (int x : ls[v]) {
        p[x] = u;
        t.upd(x, u);
        ls[u].pb(x);
    }
    ans = mul(ans, bin_pow(r[v] - l[v] + 1, mod - 2));
    ans = mul(ans, bin_pow(r[u] - l[u] + 1, mod - 2));
    l[u] = max(l[u], l[v]);
    r[u] = min(r[u], r[v]);
    if (l[u] > r[u]) {
        ans = 0;
    } else {
        ans = mul(ans, r[u] - l[u] + 1);
    }
    return 1;
}

void solve () {
    int n;
    cin >> n;
    pw[0] = 1;
    for (int i = 1; i < N; ++i)
        pw[i] = mul(pw[i - 1], base);
    int ba = 1;
    for (int i = 1; i < n; ++i) {
        int v;
        cin >> v; --v; pr[i] = v;
        g[i].pb(v);
        g[v].pb(i);
        ba &= v == i - 1;
    }
    t = segtree(n);
    dfs(0);
    iota(p, p + n, 0);
    for (int i = 0; i < n; ++i) {
        cin >> l[i] >> r[i];
        ans = mul(ans, r[i] - l[i] + 1);
        ls[i].pb(i);
    }
    int m;
    cin >> m;
    if (ba) {

        for (int i = 0; i < m; ++i) {
            int a, b, c, d;
            cin >> a >> b >> c >> d; --a; --b; --c; --d;
            int u1 = (a < b ? 1 : -1);
            int u2 = (c < d ? 1 : -1);
            int len = abs(b - a) + 1;
            while (a != b && c != d) {
                int L = -1, R = len - 1;
                while (R - L > 1) {
                    int mid = (L + R) / 2;
                    node ans1 = t.get(a, a + u1 * mid, u1);
                    node ans2 = t.get(c, c + u2 * mid, u2);
                    if (ans1.hash == ans2.hash) {
                        L = mid;
                    } else {
                        R = mid;
                    }
                }
                a += u1 * R;
                c += u2 * R;
                len -= R;
                unite(a, c);
            }
            cout << ans << '\n';
        }
        return;
    }
    for (int i = 0; i < m; ++i) {
        int a, b, c, d;
        cin >> a >> b >> c >> d; --a; --b; --c; --d;
        get_path(a, b);
        vc <int> v1 = path;
        get_path(c, d);
        vc <int> v2 = path;
        for (int j = 0; j < sz(v1); ++j) {
            unite(v1[j], v2[j]);
        }
        cout << ans << '\n';
    }
}

signed main () {
    ios_base::sync_with_stdio(false); cin.tie(0); cout.tie(0);
    int t = 1;
    // cin >> t;
    while (t--)
        solve();
}