#include <iostream>
#include <vector>
#include <algorithm>
#include <set>
#include <climits>
#include <cassert>

using namespace std;

typedef long long ll;
typedef pair<long long, long long> Pair;
const long long MOD = 1e9 + 7;
const long long INF = INT_MAX;

#define endl "\n"
#define debug cout << "# sus\n"

vector<vector<ll>> g;
vector<ll> depth, tin, tout;
ll timer = 0;

void dfs(ll v, ll p = -1, ll cur = 1) {
    depth[v] = cur;
    tin[v] = timer++;
    for (ll u : g[v]) {
        if (u != p) {
            dfs(u, v, cur + 1);
        }
    }
    tout[v] = timer++;
}

bool ancestor(ll u, ll v) {
    return tin[u] <= tin[v] && tout[u] >= tout[v];
}

ll lca(ll u, ll v) {
    ll node = -1;
    for (ll i = 0; i < (ll) g.size(); i++) {
        if (ancestor(i, u) && ancestor(i, v) && (node == -1 || depth[node] < depth[i])) node = i;
    }
    return node;
}

vector<set<ll>> nw_g;

vector<Pair> all;
vector<bool> used;

ll l, r;

void dfs2(ll v) {
    used[v] = true;
    l = max(l, all[v].first);
    r = min(r, all[v].second);
    for (ll u : nw_g[v]) {
        if (!used[u]) {
            dfs2(u);
        }
    }
}

signed main() {
    ios_base::sync_with_stdio(false), cin.tie(nullptr);
    ll n;
    cin >> n;
    g.resize(n);
    vector<ll> par(n);
    for (ll i = 1; i < n; i++) {
        ll p;
        cin >> p;
        p--;
        par[i] = p;
        g[i].push_back(p);
        g[p].push_back(i);
    }
    depth.resize(n);
    tin.resize(n);
    tout.resize(n);
    dfs(0);
    all.resize(n);
    for (ll i = 0; i < n; i++) cin >> all[i].first >> all[i].second;
    ll m;
    cin >> m;
    used.resize(n);
    nw_g.resize(n);
    while (m--) {
        ll a, b, c, d;
        cin >> a >> b >> c >> d;
        a--, b--, c--, d--;
        ll lca1 = lca(a, b), lca2 = lca(c, d);
        auto get = [&] (ll u, ll v, ll lca) {
            if (u == lca && v == lca) return vector<ll> ({lca});
            vector<ll> way, way1;
            if (u != lca) {
                ll node = u;
                while (node != lca) {
                    way.push_back(node);
                    node = par[node];
                }
            }
            way.push_back(lca);
            if (v != lca) {
                ll node = v;
                while (node != lca) {
                    way1.push_back(node);
                    node = par[node];
                }
                reverse(way1.begin(), way1.end());
                for (ll cur : way1) way.push_back(cur);
            }
            return way;
        };
        vector<ll> way_ab, way_cd;
        way_ab = get(a, b, lca1);
        way_cd = get(c, d, lca2);
       assert((ll) way_ab.size() == (ll) way_cd.size());
        for (ll i = 0; i < min((ll) way_cd.size(), (ll) way_ab.size()); i++) {
            nw_g[way_ab[i]].insert(way_cd[i]);
            nw_g[way_cd[i]].insert(way_ab[i]);
        }
        fill(used.begin(), used.end(), false);
        ll ans = 1;
        for (ll i = 0; i < n; i++) {
            if (used[i]) continue;
            l = -INF, r = INF;
            dfs2(i);
            ans *= max(0LL, r - l + 1);
        }
        cout << ans << endl;
    }
    return 0;
}
