#include <iostream>
#include <vector>
#include <cmath>
#include <string>
#include <map>
#include <set>
#include <deque>
#include <random>
#include <unordered_map>

using namespace std;
typedef long long ll;

constexpr ll mod = 1e9 + 7;

vector<ll>tree;
void build(int l, int r, int v, vector<int>&arr) {
	if (l + 1 == r) {
		tree[v] = arr[l];
		return;
	}
	int m = (l + r) / 2;
	build(l, m, 2 * v + 1, arr);
	build(m, r, 2 * v + 2, arr);
	tree[v] = tree[2 * v + 1] * tree[2 * v + 2] % mod;
}
void upd(int i, int x, int l, int r, int v) {
	if (i < l || r <= i)
		return;
	if (l + 1 == r) {
		tree[v] = x;
		return;
	}
	int m = (l + r) / 2;
	upd(i, x, l, m, 2 * v + 1);
	upd(i, x, m, r, 2 * v + 2);
	tree[v] = tree[2 * v + 1] * tree[2 * v + 2] % mod;
}

struct DSU {
	vector<int>p_, sz_;
	vector<int>l_, r_;
	int n_;
	DSU(int n, vector<int>l, vector<int>r):l_(l), r_(r), n_(n) {
		p_.resize(n);
		sz_.resize(n, 1);
		vector<int>lens(n);
		for (int i = 0; i < n; i++) {
			p_[i] = i;
			lens[i] = r_[i] - l_[i] + 1;
		}
		tree.resize(4 * n);
		build(0, n, 0, lens);
	}
	int get(int v) {
		if (p_[v] == v)
			return v;
		return p_[v] = get(p_[v]);
	}
	bool join(int v, int u) {
		v = get(v), u = get(u);
		if (v == u)
			return true;
		if (sz_[v] < sz_[u])
			swap(v, u);
		l_[v] = max(l_[v], l_[u]);
		r_[v] = min(r_[v], r_[u]);
		if (r_[v] < l_[v])
			return false;
		upd(u, 1, 0, n_, 0);
		upd(v, r_[v] - l_[v] + 1, 0, n_, 0);
		p_[u] = v;
		sz_[v] += sz_[u];
		return true;
	}
};

vector<int>p;
vector<vector<int>> g;
vector<int>tin, tout;
int T = 0;
void dfs(int v = 0) {
	tin[v] = T++;
	for (auto u : g[v]) {
		dfs(u);
	}
	tout[v] = T++;
}

bool is_pr(int p, int v) {
	return tin[p] <= tin[v] && tout[v] <= tout[p];
}

vector<int>go(int v, int end, bool flag = false) {
	vector<int>res;
	while (!is_pr(v, end)) {
		res.push_back(v);
		v = p[v];
	}
	if (flag)
		res.push_back(v);
	return res;
}

int main() {
	cin.tie(0);
	ios_base::sync_with_stdio(0);
	int n;
	cin >> n;
	p.resize(n);
	tin.resize(n);
	tout.resize(n);
	g.resize(n);
	
	for (int i = 1; i < n; i++) {
		cin >> p[i];
		p[i]--;
		g[p[i]].push_back(i);
	}
	dfs();
	vector<int>l(n), r(n);
	for (int i = 0; i < n; i++) {
		cin >> l[i] >> r[i];
	}
	DSU dsu(n, l, r);
	int m;
	cin >> m;
	bool flag = true;
	while (m--) {
		int a, b, c, d;
		cin >> a >> b >> c >> d;
		if (!flag) {
			cout << "0\n";
			continue;
		}
		a--, b--, c--, d--;
		vector<int>path_a = go(a, b, true), path_c = go(c, d, true);
		vector<int>sec_path_a = go(b, a), sec_path_c = go(d, c);
		reverse(sec_path_a.begin(), sec_path_a.end());
		reverse(sec_path_c.begin(), sec_path_c.end());
		for (int i = 0; i < sec_path_a.size(); i++) {
			path_a.push_back(sec_path_a[i]);
		}
		for (int i = 0; i < sec_path_c.size(); i++) {
			path_c.push_back(sec_path_c[i]);
		}
		for (int i = 0; i < path_a.size() && flag; i++) {
			if (!dsu.join(path_a[i], path_c[i])) {
				flag = false;
			}
		}
		if (!flag) {
			cout << "0\n";
			continue;
		}
		cout << tree[0] << '\n';
	}
	return 0;
}