#include <iostream>
#include <algorithm>
#include <vector>
#include <string>
#include <queue>
#include <deque>
#include <set>
#include <map>

using namespace std;

typedef long long ll;
typedef long double ld;

#define all(a) a.begin(), a.end()
#define rall(a) a.rbegin(), a.rend()
#define len(a) (ll)a.size()
#define endl '\n'
#define debug(a) cout << #a << ": " << a << endl;

const ll mod = 1e9 + 7;
const ll mxl = 24;

// author: @cot-in

void fin() {
#ifdef cot_in
    freopen("in.txt", "r", stdin);
    freopen("out.txt", "w", stdout);
#endif
}

ll t = 0;

ll mul(ll a, ll b) {
    return (a * b) % mod;
}

bool ancestor(ll v, ll u, vector<ll> &tin, vector<ll> &tout) {
    return (tin[v] <= tin[u] && tout[v] >= tout[u]);
}

void dfs(ll v, vector<vector<ll>> &g, vector<bool> &used, vector<ll> &p, vector<ll> &tin, vector<ll> &tout) {
    used[v] = true;
    tin[v] = t++;
    for (auto u: g[v]) {
        if (!used[u]) {
            p[u] = v;
            dfs(u, g, used, p, tin, tout);
        }
    }
    tout[v] = t++;
}

void build(ll n, vector<vector<ll>> &bup, vector<ll> &p) {
    for (ll v = 0; v < n; v++)
        bup[v][0] = p[v];
    for (ll k = 1; k < mxl; k++) {
        for (ll v = 0; v < n; v++) {
            ll w = bup[v][k - 1];
            bup[v][k] = bup[w][k - 1];
        }
    }
}

ll lca(ll v, ll u, vector<vector<ll>> &bup, vector<ll> &p, vector<ll> &tin, vector<ll> &tout) {
    if (ancestor(v, u, tin, tout))
        return v;
    if (ancestor(u, v, tin, tout))
        return u;
    for (ll k = mxl - 1; k >= 0; k--) {
        ll w = bup[v][k];
        if (!ancestor(w, u, tin, tout))
            v = w;
    }
    return p[v];
}

vector<ll> get_path(ll v, ll u, vector<vector<ll>> &bup, vector<ll> &p, vector<ll> &tin, vector<ll> &tout) {
    ll fl = lca(v, u, bup, p, tin, tout);
    vector<ll> path;
    if (fl == v) { // v - предок u
        ll cur = u;
        while (cur != v) {
            path.push_back(cur);
            cur = p[cur];
        }
        path.push_back(cur);
        reverse(all(path));
    } else if (fl == u) { // u - предок v
        ll cur = v;
        while (cur != u) {
            path.push_back(cur);
            cur = p[cur];
        }
        path.push_back(cur);
    } else { // fl - lca v и u
        ll cur = v;
        while (cur != fl) {
            path.push_back(cur);
            cur = p[cur];
        }
        cur = u;
        vector<ll> add_path;
        while (cur != fl) {
            add_path.push_back(cur);
            cur = p[cur];
        }
        add_path.push_back(cur);
        reverse(all(add_path));
        for (auto el: add_path)
            path.push_back(el);
    }
    return path;
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr), fin();
    ll n, m;
    cin >> n;
    vector<vector<ll>> g(n), bup(n, vector<ll>(mxl, 0));
    vector<bool> used(n, false);
    vector<ll> tin(n), tout(n), p(n, 0);
    vector<pair<ll, ll>> vc(n);
    for (ll u = 1; u < n; u++) {
        ll v;
        cin >> v;
        v--;
        g[v].push_back(u);
        g[u].push_back(v);
    }
    dfs(0, g, used, p, tin, tout);
    build(n, bup, p);
    for (ll i = 0; i < n; i++)
        cin >> vc[i].first >> vc[i].second;
    cin >> m;
    while (m--) {
        ll a, b, c, d;
        cin >> a >> b >> c >> d;
        a--;
        b--;
        c--;
        d--;
        vector<ll> path1 = get_path(a, b, bup, p, tin, tout);
        vector<ll> path2 = get_path(c, d, bup, p, tin, tout);
//        cout << "path 1: " << endl;
//        for (auto el: path1) {
//            cout << el + 1 << ' ';
//        }
//        cout << endl;
//        cout << "path 2: " << endl;
//        for (auto el: path2) {
//            cout << el + 1 << ' ';
//        }
//        cout << endl;
        ll length = len(path1);
        ll cnt = 1;
        for (ll _ = 0; _ < n + 2; _++) {
            for (ll i = 0; i < length; i++) {
                ll v = path1[i], u = path2[i];
                ll l = max(vc[v].first, vc[u].first);
                ll r = min(vc[v].second, vc[u].second);
                if (l > r)
                    cnt = 0;
                vc[v] = vc[u] = make_pair(l, r);
            }
        }
//        cout << "current vc: " << endl;
//        for (auto [l, r]: vc) {
//            cout << l << ' ' << r << endl;
//        }
        vector<bool> was(n, false);
        for (ll i = 0; i < length; i++) {
            ll v = path1[i], u = path2[i];
            if (!was[v] && !was[u])
                cnt = mul(cnt, vc[v].second - vc[v].first + 1);
            was[v] = was[u] = true;
        }
        for (ll v = 0; v < n; v++) {
            if (!was[v])
                cnt = mul(cnt, vc[v].second - vc[v].first + 1);
        }
        cout << cnt << endl;
    }
    return 0;
}