#include <iostream>
#include <vector>
#include <set>
#include <algorithm>
#include <queue>
#include <cmath>
#include <cassert>
#include <unordered_map>
#include <unordered_set>
#include <map>
#include <random>
#include <numeric>

using namespace std;

using ll = long long;
using ld = long double;

const int INT_INF = 2e9 + 10;
const int LL_INF = 2e18 + 10;

template <typename T> istream& operator >> (istream& in, vector<T>& a) {
    for (auto& i : a) {
        in >> i;
    }
    return in;
}

const int MOD = 1e9 + 7;

int mult(int a, int b) {
    return ll(a) * b % MOD;
}

void gcd_ext(int a, int b, int& x, int& y) {
    if (b == 0) {
        x = 1;
        y = 0;
        return;
    }
    int x1, y1;
    gcd_ext(b, a % b, x1, y1);
    x = y1;
    y = x1 - mult(a / b, y1);
}

int inverse(int a) {
    int x, y;
    gcd_ext(a, MOD, x, y);
    if (x < 0) {
        x += MOD;
    }
    return x;
}

int divide(int a, int b) {
    return mult(a, inverse(b));
}

vector<vector<int>> g;
vector<int> p;

struct Range {
    int l, r;
    int d;
    Range() : l(0), r(0), d(0) {}
    Range(int _l, int _r) : l(_l), r(_r), d(_r - _l + 1) {}
};

Range intersect(Range a, Range b) {
    Range res;
    res.l = max(a.l, b.l);
    res.r = min(a.r, b.r);
    res.d = max(0, res.r - res.l + 1);
    return res;
}

vector<Range> a;

struct DSU {
    int n;
    vector<int> sz;
    vector<int> p;
    vector<Range> range;
    DSU() : n(0) {}
    DSU(int _n) : n(_n) {
        sz.resize(n, 1);
        p.resize(n);
        for (int i = 0; i < n; i++) {
            p[i] = i;
        }
        range.resize(n);
        for (int i = 0; i < n; i++) {
            range[i] = a[i];
        }
    }
    int get_head(int u) {
        if (p[u] == u) {
            return u;
        }
        int head = get_head(p[u]);
        p[u] = head;
        return head;
    }
    int unite(int a, int b) {
        a = get_head(a);
        b = get_head(b);
        if (sz[a] < sz[b]) {
            swap(a, b);
        }
        sz[a] += sz[b];
        p[b] = a;
        range[a] = intersect(range[a], range[b]);
        return range[a].d;
    }
    int get_d(int u) {
        return range[get_head(u)].d;
    }
    bool all_united() {
        return sz[get_head(0)] == n;
    }
};

//const int MAXLOG = 20;
//vector<vector<int>> binup;

int t = 0;
vector<int> tin, tout;
void dfs(int u) {
    tin[u] = t++;
    for (auto v : g[u]) {
        dfs(v);
    }
    tout[u] = t++;
}

int is_anc(int anc, int u) {
    return tin[anc] <= tin[u] && tout[anc] >= tout[u];
}

//int lca(int u, int v) {
//    if (is_anc(u, v)) {
//        return u;
//    }
//    if (is_anc(v, u)) {
//        return v;
//    }
//    for (int log = MAXLOG - 1; log >= 0; log--) {
//        if (!is_anc(binup[log][u], v)) {
//            u = binup[log][u];
//        }
//    }
//    return p[u];
//}

int main() {
    int n;
    cin >> n;
    g.resize(n);
    p.resize(n);
    p[0] = 0;
    a.resize(n);
    tin.resize(n);
    tout.resize(n);
    for (int i = 1; i < n; i++) {
        cin >> p[i];
        p[i]--;
        g[p[i]].push_back(i);
    }
    dfs(0);
    for (int i = 0; i < n; i++) {
        int l, r;
        cin >> l >> r;
        a[i] = Range(l, r);
    }
    int m;
    cin >> m;
    int ans = 1;
    for (auto& i : a) {
        ans = mult(ans, i.d);
    }
    DSU dsu(n);
    //calc binup
    /*for (int i = 0; i < n; i++) {
        binup[0][i] = p[i];
    }
    for (int log = 1; log < MAXLOG; log++) {
        for (int u = 0; u < n; u++) {
            binup[log][u] = binup[log - 1][binup[log - 1][u]];
        }
    }*/
    for (int q_it = 0; q_it < m; q_it++) {
        vector<int> from(2), to(2);
        for (int i = 0; i < 2; i++) {
            cin >> from[i] >> to[i];
            from[i]--;
            to[i]--;
        }
        vector<vector<int>> path(2);
        for (int i = 0; i < 2; i++) {
            int u = from[i];
            int v = to[i];
            vector<int> up, down;
            while (!is_anc(u, v)) {
                up.push_back(u);
                u = p[u];
            }
            up.push_back(u);
            while (v != u) {
                down.push_back(v);
                v = p[v];
            }
            reverse(down.begin(), down.end());
            for (auto j : up) {
                path[i].push_back(j);
            }
            for (auto j : down) {
                path[i].push_back(j);
            }
        }
        for (int i = 0; i < path[0].size(); i++) {
            int u = path[0][i];
            int v = path[1][i];
            if (dsu.get_head(u) != dsu.get_head(v)) {
                ans = divide(ans, dsu.get_d(u));
                ans = divide(ans, dsu.get_d(v));
                ans = mult(ans, dsu.unite(u, v));
            }
        }
        if (dsu.all_united() || ans == 0) {
            while (q_it < m) {
                cout << ans << '\n';
                q_it++;
            }
            break;
        }
        cout << ans << '\n';
    }
}
