#include <iostream>
#include <vector>
#include <stack>
#define endl '\n'
using namespace std;

const int MAXN = 200000;
const int LOG = 18;
const int MOD = 1e9 + 7;

int depth[MAXN + 5];
int par[MAXN + 5][LOG];
vector<int> G[MAXN + 5];
int le[MAXN + 5], ri[MAXN + 5];

void DFS(int u)
{
    for (auto v : G[u])
    {
        depth[v] = depth[u] + 1;
        DFS(v);
    }
}

int LCA(int u, int v)
{
    int du = depth[u], dv = depth[v];
    if (du > dv)
    {
        swap(u, v);
        swap(du, dv);
    }

    for (int j = LOG - 1; j >= 0; -- j)
    {
        if (dv - (1ll << j) >= du)
        {
            v = par[v][j];
            dv -= (1ll << j);
        }
    }

    for (int j = LOG - 1; j >= 0; -- j)
    {
        if (par[u][j] != par[v][j])
        {
            u = par[u][j];
            v = par[v][j];
        }
    }

    if (u != v) return par[u][0];
    return u;
}

int p[MAXN + 5], sz[MAXN + 5];
bool used[MAXN + 5];

int find_par(int u)
{
    if (p[u] == u) return u;
    return (p[u] = find_par(p[u]));
}

void unite(int u, int v)
{
    u = find_par(u);
    v = find_par(v);

    if (sz[u] < sz[v]) swap(u, v);
    p[v] = u;
    le[u] = max(le[u], le[v]);
    ri[u] = min(ri[u], ri[v]);
    sz[u] += sz[v];
}

int main()
{
    ios :: sync_with_stdio(false);
    cin.tie(NULL); cout.tie(NULL);

    int n;
    cin >> n;

    par[1][0] = -1;
    p[1] = 1;
    sz[1] = 1;
    for (int i = 2; i <= n; ++ i)
    {
        p[i] = i;
        sz[i] = 1;
        cin >> par[i][0];
        G[par[i][0]].push_back(i);
    }

    for (int j = 1; j < LOG; ++ j)
    {
        for (int i = 1; i <= n; ++ i)
        {
            par[i][j] = par[par[i][j - 1]][j - 1];
        }
    }

    depth[1] = 0;
    DFS(1);

    for (int i = 1; i <= n; ++ i)
    {
        cin >> le[i] >> ri[i];
    }

    int m;
    cin >> m;

    for (int i = 1; i <= m; ++ i)
    {
        int a, b, c, d;
        cin >> a >> b >> c >> d;

        for (int j = 1; j <= n; ++ j)
        {
            used[j] = false;
        }

        int ab = LCA(a, b);
        int cd = LCA(c, d);

        vector<int> va;
        while (a != ab)
        {
            va.push_back(a);
            a = par[a][0];
        }
        va.push_back(a);

        stack<int> sa;
        while (b != ab)
        {
            sa.push(b);
            b = par[b][0];
        }

        while (!sa.empty())
        {
            va.push_back(sa.top());
            sa.pop();
        }

        vector<int> vc;
        while (c != cd)
        {
            vc.push_back(c);
            c = par[c][0];
        }
        vc.push_back(c);

        stack<int> sc;
        while (d != cd)
        {
            sc.push(d);
            d = par[d][0];
        }

        while (!sc.empty())
        {
            vc.push_back(sc.top());
            sc.pop();
        }

        for (int j = 0; j < va.size(); ++ j)
        {
            if (find_par(va[j]) != find_par(vc[j])) unite(va[j], vc[j]);
        }

        int ans = 1;
        for (int j = 1; j <= n; ++ j)
        {
            int p = find_par(j);
            if (used[p]) continue;
            used[p] = true;
            if (le[j] > ri[j])
            {
                ans = 0;
                break;
            }
            ans = (1ll * ans * (ri[j] - le[j] + 1)) % MOD;
        }
        cout << ans << endl;
    }

    return 0;
}
