#include <iostream>
#include <cmath>
#include <fstream>
#include <algorithm>
#include <vector>
#include <map>
#include <set>
#include <ext/pb_ds/assoc_container.hpp>
#include <numeric>

using namespace __gnu_pbds;
using namespace std;

#define int long long
#define all(x) (x).begin(), (x).end()
#define sz(x) (int) ((x).size())

template<typename T>
using ordered_set = tree<T, null_type, less<T>, rb_tree_tag, tree_order_statistics_node_update>;

#ifdef LOCAL
#include "algo/debug.h"
#else
#define debug(...) 42
#endif

const int mod = 1e9 + 7;
const int P = 239017;
const int N = 5e5 + 5;

int deg[N], degrev[N];

int mul(int a, int b) {
  return a * b % mod;
}

int sum(int a, int b) {
  a += b;
  if (a >= mod) {
    a -= mod;
  }
  if (a < 0) {
    a += mod;
  }
  return a;
}

int binpow(int a, int n) {
  int res = 1;
  while (n) {
    if (n % 2) {
      res = mul(res, a);
    }
    a = mul(a, a);
    n /= 2;
  }

  return res;
}

int rev(int x) {
  return binpow(x, mod - 2);
}

struct Fenwick {
  vector<int> t;
  int n;

  Fenwick(int n): n(n), t(n + 1) {}

  void upd(int i, int delta) {
    i += 1;
    delta = mul(delta, deg[n - i]);
    for (; i <= n; i += i & -i) {
      t[i] = sum(t[i], delta);
    }
  }

  int get(int r) {
    int res = 0;
    for (; r > 0; r -= r & -r) {
      res = sum(res, t[r]);
    }
    return res;
  }

  int get(int l, int r) {
    if (l > r) {
      return 0ll;
    }
    l += 1;
    r += 1;
    return mul(sum(get(r), -get(l - 1)), degrev[n - r]);
  }
};

void solve() {
  deg[0] = degrev[0] = 1;
  for (int i = 1; i < N; i++) {
    deg[i] = mul(deg[i - 1], P);
    degrev[i] = rev(deg[i]);
  }

  int n;
  cin >> n;
  vector<int> p(n, -1);
  vector<vector<int>> g(n);
  for (int i = 1; i < n; i++) {
    cin >> p[i];
    --p[i];
    g[p[i]].push_back(i);
  }

  int res = 1;
  vector<int> lb(n), rb(n);
  for (int i = 0; i < n; i++) {
    cin >> lb[i] >> rb[i];
    res = mul(res, rb[i] - lb[i] + 1);
  }

  vector<int> sizeSubtree(n);
  function<void(int)> hldReorder = [&] (int v) {
    sizeSubtree[v] = 1;
    for (auto &u : g[v]) {
      hldReorder(u);
      sizeSubtree[v] += sizeSubtree[u];
    }
    sort(all(g[v]), [&] (const int& lhs, const int& rhs) {
      return sizeSubtree[lhs] > sizeSubtree[rhs];
    });
  };
  hldReorder(0);

  int timer = 0;
  vector<int> tin(n), tout(n), head(n);
  function<void(int)> hld = [&] (int v) {
    tin[v] = timer++;
    for (auto &u : g[v]) {
      head[u] = (u == g[v][0] ? head[v] : u);
      hld(u);
    }
    tout[v] = timer;
  };
  hld(0);

  auto anc = [&] (int u, int v) -> bool {
    return tin[u] <= tin[v] && tin[v] < tout[u];
  };

  vector<int> kok(n);
  for (int i = 0; i < n; i++) {
    kok[tin[i]] = i;
  }

  Fenwick ft(2 * n);
  for (int i = 0; i < n; i++) {
    ft.upd(i, kok[i] + 1);
    ft.upd(2 * n - 1 - i, kok[i] + 1);
  }

  vector<int> par(n), size(n);
  iota(all(par), 0);
  fill(all(size), 1);

  function<int(int)> get = [&] (int v) {
    if (par[v] == v) {
      return v;
    }
    int np = get(par[v]);
    ft.upd(tin[v], np - par[v]);
    ft.upd(2 * n - 1 - tin[v], np - par[v]);
    par[v] = np;
    return np;
  };

  auto unite = [&] (int a, int b) {
    a = get(a);
    b = get(b);
    if (a == b) {
      return;
    }
    if (size[b] > size[a]) {
      swap(a, b);
    }
    ft.upd(tin[b], a - b);
    ft.upd(2 * n - 1 - tin[b], a - b);
    par[b] = a;
    size[a] += size[b];
    res = mul(res, rev(rb[b] - lb[b] + 1));
    res = mul(res, rev(rb[a] - lb[a] + 1));
    lb[a] = max(lb[a], lb[b]);
    rb[a] = min(rb[a], rb[b]);
    res = mul(res, max(0ll, rb[a] - lb[a] + 1));
  };

  auto get_segs = [&] (int a, int b) -> vector<pair<int, int>> {
    if (a == b) {
      return vector<pair<int, int>> {make_pair(tin[a], tin[a])};
    }
    vector<pair<int, int>> segs;
    while (!anc(head[a], b)) {
      segs.emplace_back(tin[a], tin[head[a]]);
      a = p[head[a]];
    }
    vector<pair<int, int>> segs_rev;
    while (!anc(head[b], a)) {
      segs_rev.emplace_back(tin[head[b]], tin[b]);
      b = p[head[b]];
    }
    segs.emplace_back(tin[a], tin[b]);
    reverse(all(segs_rev));
    for (auto &x : segs_rev) {
      segs.push_back(x);
    }

    return segs;
  };

  auto get_borders = [&] (int l, int r, int id_from, int id_to) {
    if (l <= r) {
      return make_pair(l + id_from, l + id_to);
    }
    return make_pair(l - id_from, l - id_to);
  };

  auto getHash = [&] (int l, int r, int len) -> int {
    int id_beg = (l <= r ? l : 2 * n - 1 - l);
    return ft.get(id_beg, id_beg + len - 1);
  };

  auto get_id = [&] (int l, int r, int pos) {
    if (l <= r) {
      return l + pos;
    }
    return l - pos;
  };

  int m;
  cin >> m;
  for (int w = 0; w < m; w++) {
    int a, b, c, d;
    cin >> a >> b >> c >> d;
    --a, --b, --c, --d;
    vector<pair<int, int>> segs1 = get_segs(a, b), segs2 = get_segs(c, d);
    vector<pair<int, int>> s1, s2;
    int ptr1 = 0, ptr2 = 0, used1 = 0, used2 = 0;
    while (ptr1 < sz(segs1) && ptr2 < sz(segs2)) {
      int len1 = abs(segs1[ptr1].second - segs1[ptr1].first) + 1;
      int len2 = abs(segs2[ptr2].second - segs2[ptr2].first) + 1;
      int can1 = len1 - used1;
      int can2 = len2 - used2;
      int add = min(can1, can2);
      s1.push_back(get_borders(segs1[ptr1].first, segs1[ptr1].second, used1, used1 + add - 1));
      s2.push_back(get_borders(segs2[ptr2].first, segs2[ptr2].second, used2, used2 + add - 1));
      used1 += add;
      used2 += add;
      if (used1 == len1) {
        ptr1++;
        used1 = 0;
      }
      if (used2 == len2) {
        ptr2++;
        used2 = 0;
      }
    }

    assert(sz(s1) == sz(s2));
    for (int i = 0; i < sz(s1); i++) {
      auto [l1, r1] = s1[i];
      auto [l2, r2] = s2[i];

      int len = abs(l1 - r1) + 1;
      while (getHash(l1, r1, len) != getHash(l2, r2, len)) {
        int l = 0, r = len + 1;
        while (l + 1 < r) {
          int mid = (l + r) / 2;
          if (getHash(l1, r1, mid) == getHash(l2, r2, mid)) {
            l = mid;
          } else {
            r = mid;
          }
        }
        unite(kok[get_id(l1, r1, l)], kok[get_id(l2, r2, l)]);
      }
    }

    cout << res << '\n';
  }
}

int32_t main() {
#ifdef LOCAL
  freopen("input.txt", "r", stdin);
#endif
  ios_base::sync_with_stdio(false);
  cin.tie(nullptr);
  solve();
  return 0;
}