#include <iostream>
#include <vector>
#include <algorithm>
#include <string>
#include <map>
#include <set>
#include <math.h>
#include <climits>

using namespace std;

#define ll long long
#define lll __int128
#define ld long double
#define lld __float128
#define endl '\n'
#define fast ios_base::sync_with_stdio(0); cin.tie(0); cout.tie(0);

//a.x * b.y - a.y - b * x

int gcd(int a, int b) {
    if (a == 0) return b;
    return gcd(b % a, a);
}

struct Treap {
    Treap* left = nullptr;
    Treap* right = nullptr;
    ll height, val;

    Treap(ll newy) {
        val = newy;
        height = rand();
    }
};

Treap* merge(Treap* a, Treap* b) {
    if (a == nullptr) return b;
    if (b == nullptr) return a;
    if (a->height < b->height) {
        a->right = merge(a->right, b);
        return a;
    }
    else {
        b->left = merge(a, b->left);
        return b;
    }
}

pair<Treap*, Treap*> split(Treap* t, int val) {
    if (t == nullptr) return { nullptr, nullptr };
    if (t->val < val) {
        auto sp = split(t->right, val);
        t->right = sp.first;
        return {t, sp.second};
    }
    else {
        auto sp = split(t->left, val);
        t->left = sp.second;
        return {sp.first, t};
    }
}

struct vertex {
    ld answer, push = 0;
    vertex(ld num) {
        answer = num;
    }
    vertex() {}
    vertex operator+ (vertex w) {
        return vertex(max(answer, w.answer));
    }
};

struct tree {
    vector<ld> nums;
    vector<vertex> simple;

    tree(vector<ld> a) {
        nums = a;
        simple.resize(a.size() * 4);
        build(1, 0, a.size());
    }

    tree(int len) {
        nums = vector<ld>(len, 0);
        simple.resize(len * 4);
        build(1, 0, len);
    }

    void build(int v, int l, int r) {
        if (r - l == 1) {
            simple[v] = vertex(nums[l]);
        }
        else {
            int mid = (r + l) / 2;
            build(2 * v, l, mid);
            build(2 * v + 1, mid, r);
            simple[v] = simple[2 * v] + simple[2 * v + 1];
        }
    }

    vertex answer(int v, int l, int r, int lq, int rq) {
        if (l >= rq || r <= lq) {
            return vertex(0);
        }
        else if (l >= lq && r <= rq) {
            simple[v].answer = max(simple[v].answer, simple[v].push);
            return simple[v];
        }
        else {
            int mid = (l + r) / 2;
            simple[2 * v].push = max(simple[2 * v].push, simple[v].push);
            simple[2 * v + 1].push = max(simple[2 * v + 1].push, simple[v].push);
            return answer(2 * v, l, mid, lq, rq) + answer(2 * v + 1, mid, r, lq, rq);
        }
    }

    vertex change(int v, int l, int r, int position, ld newVal) {
        if (r - l == 1) {
            nums[position] = newVal;
            simple[v] = vertex(newVal);
        }
        int mid = (r + l) / 2;
        if (position < mid) {
            change(2 * v, l, mid, position, newVal);
            simple[v] = simple[2 * v] + simple[2 * v + 1];
        }
        else {
            change(2 * v + 1, mid, r, position, newVal);
            simple[v] = simple[2 * v] + simple[2 * v + 1];
        }
    }

    void mass(int v, int l, int r, int lq, int rq, ld newMax) {
        if (l >= rq || r <= lq) {
            return;
        }
        else if (l >= lq && r <= rq) {
            simple[v].push = max(newMax, simple[v].push);
        }
        else {
            int mid = (l + r) / 2;
            mass(2 * v    , l, mid, lq, rq, newMax);
            mass(2 * v + 1, mid, r, lq, rq, newMax);
        }
    }
};

vector<pair<int, int>> ree(int a) {
    vector<pair<int, int>> answer;
    //part count and size
    for (int i = 2; i * i <= a; ++i) {
        answer.push_back({ i, a / i });
    }
    for (int i = 1; i * i <= a; ++i) {
        answer.push_back({ a / i, i });
    }
    sort(answer.begin(), answer.end());
    answer.resize(unique(answer.begin(), answer.end()) - answer.begin());
    return answer;
}

ll roundTop(ll a, ll b) {
    ll h = a / b;
    if (h * b != a) return h + 1;
    else return h;
}

void solve() {
    int n, k;
    cin >> n >> k;
    vector<int> a(n);
    for (int i = 0; i < n; ++i) {
        cin >> a[i];
    }
    vector < vector<pair<int, int>>> definition(n);
    for (int i = 0; i < n; ++i) {
        definition[i] = ree(a[i]);
    }
    //vector<vector<ld>> dp(n+1, vector<ld>(k+1, 0)); // [i] - count of part, ans = max size
    tree prev(k+1), now(k+1);
    prev.mass(1, 0, k + 1, 1, 2, 1);
    for (int i = 1; i <= n; ++i) {
        now = prev;
        for (auto y : definition[i-1]) {
            for (int j = y.first; j < k; j += y.first) {
                ld prevAtPos = prev.answer(1, 0, k + 1, j / y.first, j / y.first + 1).answer * (ld)y.second / (ld)a[i - 1];
                now.mass(1, 0, k + 1, 0, j + 1, prevAtPos);
            }
            int pos2 = roundTop(k, y.first);
            ld prevAtPos = prev.answer(1, 0, k + 1, pos2, pos2 + 1).answer * (ld)y.second / (ld)a[i - 1];
            now.mass(1, 0, k + 1, 0, k + 1, prevAtPos);
        }
        prev = now;
    }
    cout.precision(20);
    cout << fixed;
    ld maxSquare = now.answer(1, 0, k+1, k, k+1).answer;
    ld ans = maxSquare * (ld)k;
    cout << ans << endl;
}

void test() {
    int n = 5;
    auto r = ree(n);
    for (auto p : r) {
        cout << p.first << " " << p.second << endl;
    }
}

int main() {
    fast;
    srand((int)time(nullptr));
    int t = 1;
    //cin >> t;
    while (t--) solve();
    return 0;
}